@@ -2,7 +2,7 @@
     smb4kbookmarkhandler  -  This class handles the bookmarks.
                              -------------------
     begin                : Fr Jan 9 2004
-    copyright            : (C) 2004-2013 by Alexander Reinholdt
+    copyright            : (C) 2004-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -37,6 +37,7 @@
 #include "smb4kshare.h"
 #include "smb4ksettings.h"
 #include "smb4knotification.h"
+#include "smb4kprofilemanager.h"
 
 // Qt includes
 #include <QtCore/QDir>
@@ -75,7 +76,11 @@ Smb4KBookmarkHandler::Smb4KBookmarkHandler( QObject *parent )
     KGlobal::dirs()->makeDir( dir );
   }
 
-  loadBookmarks();
+  readBookmarks(&d->bookmarks, &d->groups, false);
+  
+  // Connections
+  connect(Smb4KProfileManager::self(), SIGNAL(activeProfileChanged(QString)), 
+          this, SLOT(slotActiveProfileChanged(QString)));
 }
 
 
@@ -111,7 +116,7 @@ void Smb4KBookmarkHandler::addBookmark( Smb4KShare *share, QWidget *parent )
 }
 
 
-void Smb4KBookmarkHandler::addBookmarks( const QList<Smb4KShare *> &list, QWidget *parent )
+void Smb4KBookmarkHandler::addBookmarks(const QList<Smb4KShare *> &list, QWidget *parent)
 {
   // Prepare the list of bookmarks and show the save dialog.
   QList<Smb4KBookmark *> new_bookmarks;
@@ -121,8 +126,7 @@ void Smb4KBookmarkHandler::addBookmarks( const QList<Smb4KShare *> &list, QWidge
     // Check if the share is a printer
     if ( list.at( i )->isPrinter() )
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->cannotBookmarkPrinter( list.at( i ) );
+      Smb4KNotification::cannotBookmarkPrinter(list.at( i ));
       continue;
     }
     else
@@ -159,16 +163,17 @@ void Smb4KBookmarkHandler::addBookmarks( const QList<Smb4KShare *> &list, QWidge
     // Skip bookmarks already present
     if ( known_bookmark )
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->bookmarkExists( known_bookmark );
+      Smb4KNotification::bookmarkExists(known_bookmark);
       continue;
     }
     else
     {
       // Do nothing
     }
-
-    new_bookmarks << new Smb4KBookmark( list.at( i ) );
+    
+    Smb4KBookmark *bookmark = new Smb4KBookmark(list.at(i));
+    bookmark->setProfile(Smb4KProfileManager::self()->activeProfile());
+    new_bookmarks << bookmark;
   }
   
   if ( !new_bookmarks.isEmpty() )
@@ -177,32 +182,10 @@ void Smb4KBookmarkHandler::addBookmarks( const QList<Smb4KShare *> &list, QWidge
 
     if ( dlg->exec() == KDialog::Accepted )
     {
-      // Check the label of the new bookmarks.
-      for ( int i = 0; i < new_bookmarks.size(); ++i )
-      {
-        if ( !new_bookmarks.at( i )->label().isEmpty() )
-        {
-          Smb4KBookmark *bookmark = findBookmarkByLabel( new_bookmarks.at( i )->label() );
-
-          if ( bookmark )
-          {
-            Smb4KNotification *notification = new Smb4KNotification();
-            notification->bookmarkLabelInUse( new_bookmarks.at( i ) );
-
-            new_bookmarks[i]->setLabel( QString( "%1 (1)" ).arg( new_bookmarks.at( i )->label() ) );
-          }
-          else
-          {
-            // Do nothing
-          }
-        }
-        else
-        {
-          // Do nothing
-        }
-      }
-      
-      addBookmarks( new_bookmarks, false );
+      // The bookmark dialog uses an internal list of bookmarks, 
+      // so use that one instead of the temporary list created 
+      // above.
+      addBookmarks(dlg->bookmarks(), false);
     }
     else
     {
@@ -215,6 +198,12 @@ void Smb4KBookmarkHandler::addBookmarks( const QList<Smb4KShare *> &list, QWidge
   {
     // Do nothing
   }
+  
+  // Clear the list of bookmarks.
+  while (!new_bookmarks.isEmpty())
+  {
+    delete new_bookmarks.takeFirst();
+  }
 }
 
 
@@ -235,10 +224,21 @@ void Smb4KBookmarkHandler::addBookmarks(const QList< Smb4KBookmark* >& list, boo
     // Do nothing
   }
   
-  // Append the new bookmarks to the internal list.
-  for ( int i = 0; i < list.size(); ++i )
+  // Append the new bookmarks to the internal list and check
+  // the label simultaneously.
+  for (int i = 0; i < list.size(); ++i)
   {
-    d->bookmarks << new Smb4KBookmark( *list.at( i ) );
+    if (!list.at(i)->label().isEmpty() && findBookmarkByLabel(list.at(i)->label()))
+    {
+      Smb4KNotification::bookmarkLabelInUse(list.at(i));
+      Smb4KBookmark *new_bookmark = new Smb4KBookmark(*list.at(i));
+      new_bookmark->setLabel(QString("%1 (1)").arg(list.at(i)->label()));
+      d->bookmarks << new_bookmark;
+    }
+    else
+    {
+      d->bookmarks << new Smb4KBookmark(*list.at(i));
+    }
   }
       
   // Append new groups to the internal list.
@@ -272,7 +272,7 @@ void Smb4KBookmarkHandler::removeBookmark(Smb4KBookmark* bookmark)
     for ( int i = 0; i < d->bookmarks.size(); ++i )
     {
       if ( QString::compare( bookmark->unc(), d->bookmarks.at(i)->unc(), Qt::CaseInsensitive ) == 0 &&
-          QString::compare( bookmark->groupName(), d->bookmarks.at(i)->groupName(), Qt::CaseInsensitive ) == 0 )
+           QString::compare( bookmark->groupName(), d->bookmarks.at(i)->groupName(), Qt::CaseInsensitive ) == 0 )
       {
         delete d->bookmarks.takeAt(i);
         break;
@@ -354,26 +354,60 @@ void Smb4KBookmarkHandler::removeGroup(const QString& name)
 }
 
 
-void Smb4KBookmarkHandler::writeBookmarkList( const QList<Smb4KBookmark *> &list )
+void Smb4KBookmarkHandler::writeBookmarkList(const QList<Smb4KBookmark *> &list, bool listOnly)
 {
-  QFile xmlFile( KGlobal::dirs()->locateLocal( "data", "smb4k/bookmarks.xml", KGlobal::mainComponent() ) );
-
-  if ( !list.isEmpty() )
-  {
-    if ( xmlFile.open( QIODevice::WriteOnly | QIODevice::Text ) )
+  QList<Smb4KBookmark *> allBookmarks;
+  QStringList allGroups; // Can be ignored.
+  
+  if (!listOnly)
+  {  
+    // First read all entries. Then remove all, that belong to
+    // the currently active profile.
+    readBookmarks(&allBookmarks, &allGroups, true);
+    
+    QMutableListIterator<Smb4KBookmark *> it(allBookmarks);
+    
+    while (it.hasNext())
     {
-      QXmlStreamWriter xmlWriter( &xmlFile );
-      xmlWriter.setAutoFormatting( true );
-      xmlWriter.writeStartDocument();
-      xmlWriter.writeStartElement( "bookmarks" );
-      xmlWriter.writeAttribute( "version", "1.1" );
-
-      for ( int i = 0; i < list.size(); ++i )
+      Smb4KBookmark *bookmark = it.next();
+      
+      if (QString::compare(bookmark->profile(), Smb4KProfileManager::self()->activeProfile()) == 0)
       {
-        if ( !list.at( i )->url().isValid() )
+        it.remove();
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  for (int i = 0; i < list.size(); ++i)
+  {
+    allBookmarks << new Smb4KBookmark(*list[i]);
+  }
+  
+  QFile xmlFile(KGlobal::dirs()->locateLocal("data", "smb4k/bookmarks.xml", KGlobal::mainComponent()));
+
+  if (!allBookmarks.isEmpty())
+  {
+    if (xmlFile.open(QIODevice::WriteOnly|QIODevice::Text))
+    {
+      QXmlStreamWriter xmlWriter(&xmlFile);
+      xmlWriter.setAutoFormatting(true);
+      xmlWriter.writeStartDocument();
+      xmlWriter.writeStartElement("bookmarks");
+      xmlWriter.writeAttribute("version", "1.1");
+
+      for (int i = 0; i < allBookmarks.size(); ++i)
+      {
+        if (!allBookmarks.at(i)->url().isValid())
         {
-          Smb4KNotification *notification = new Smb4KNotification();
-          notification->invalidURLPassed();
+          Smb4KNotification::invalidURLPassed();
           continue;
         }
         else
@@ -381,16 +415,16 @@ void Smb4KBookmarkHandler::writeBookmarkList( const QList<Smb4KBookmark *> &list
           // Do nothing
         }
 
-        xmlWriter.writeStartElement( "bookmark" );
-        xmlWriter.writeAttribute( "profile", d->bookmarks.at( i )->profile() );
-        xmlWriter.writeAttribute( "group", d->bookmarks.at( i )->groupName() );
+        xmlWriter.writeStartElement("bookmark");
+        xmlWriter.writeAttribute("profile", allBookmarks.at(i)->profile());
+        xmlWriter.writeAttribute("group", allBookmarks.at(i)->groupName());
 
-        xmlWriter.writeTextElement( "workgroup", d->bookmarks.at( i )->workgroupName() );
-        xmlWriter.writeTextElement( "unc", d->bookmarks.at( i )->unc() );
-        xmlWriter.writeTextElement( "login", d->bookmarks.at( i )->login() );
-        xmlWriter.writeTextElement( "ip", d->bookmarks.at( i )->hostIP() );
-        xmlWriter.writeTextElement( "type", d->bookmarks.at( i )->typeString() );
-        xmlWriter.writeTextElement( "label", d->bookmarks.at( i )->label() );
+        xmlWriter.writeTextElement("workgroup", allBookmarks.at(i)->workgroupName());
+        xmlWriter.writeTextElement("unc", allBookmarks.at(i)->unc());
+        xmlWriter.writeTextElement("login", allBookmarks.at(i)->login());
+        xmlWriter.writeTextElement("ip", allBookmarks.at(i)->hostIP());
+        xmlWriter.writeTextElement("type", allBookmarks.at(i)->typeString());
+        xmlWriter.writeTextElement("label", allBookmarks.at(i)->label());
 
         xmlWriter.writeEndElement();
       }
@@ -401,19 +435,24 @@ void Smb4KBookmarkHandler::writeBookmarkList( const QList<Smb4KBookmark *> &list
     }
     else
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->openingFileFailed( xmlFile );
-      return;
+      Smb4KNotification::openingFileFailed(xmlFile);
     }
   }
   else
   {
     xmlFile.remove();
   }
+  
+  while (!allBookmarks.isEmpty())
+  {
+    delete allBookmarks.takeFirst();
+  }
+  
+  allGroups.clear();
 }
 
 
-void Smb4KBookmarkHandler::loadBookmarks()
+void Smb4KBookmarkHandler::readBookmarks(QList<Smb4KBookmark *> *bookmarks, QStringList *groups, bool allBookmarks)
 {
   // Locate the XML file.
   QFile xmlFile( KGlobal::dirs()->locateLocal( "data", "smb4k/bookmarks.xml", KGlobal::mainComponent() ) );
@@ -438,63 +477,72 @@ void Smb4KBookmarkHandler::loadBookmarks()
         {
           if ( xmlReader.name() == "bookmark" )
           {
-            Smb4KBookmark *bookmark = new Smb4KBookmark();
-
-            bookmark->setProfile( xmlReader.attributes().value( "profile" ).toString() );
-            bookmark->setGroupName( xmlReader.attributes().value( "group" ).toString() );
-
-            while ( !(xmlReader.isEndElement() && xmlReader.name() == "bookmark") )
+            QString profile = xmlReader.attributes().value("profile").toString();
+            
+            if (allBookmarks || QString::compare(Smb4KProfileManager::self()->activeProfile(), profile, Qt::CaseSensitive) == 0)
             {
-              xmlReader.readNext();
-
-              if ( xmlReader.isStartElement() )
+              Smb4KBookmark *bookmark = new Smb4KBookmark();
+              bookmark->setProfile(profile);
+              bookmark->setGroupName(xmlReader.attributes().value("group").toString());
+              
+              while ( !(xmlReader.isEndElement() && xmlReader.name() == "bookmark") )
               {
-                if ( xmlReader.name() == "workgroup" )
+                xmlReader.readNext();
+
+                if ( xmlReader.isStartElement() )
                 {
-                  bookmark->setWorkgroupName( xmlReader.readElementText() );
-                }
-                else if ( xmlReader.name() == "unc" )
-                {
-                  bookmark->setURL( xmlReader.readElementText() );
-                }
-                else if ( xmlReader.name() == "login" )
-                {
-                  bookmark->setLogin( xmlReader.readElementText() );
-                }
-                else if ( xmlReader.name() == "ip" )
-                {
-                  bookmark->setHostIP( xmlReader.readElementText() );
-                }
-                else if ( xmlReader.name() == "type" )
-                {
-                  bookmark->setTypeString( xmlReader.readElementText() );
-                }
-                else if ( xmlReader.name() == "label" )
-                {
-                  bookmark->setLabel( xmlReader.readElementText() );
+                  if ( xmlReader.name() == "workgroup" )
+                  {
+                    bookmark->setWorkgroupName( xmlReader.readElementText() );
+                  }
+                  else if ( xmlReader.name() == "unc" )
+                  {
+                    bookmark->setURL( xmlReader.readElementText() );
+                  }
+                  else if ( xmlReader.name() == "login" )
+                  {
+                    bookmark->setLogin( xmlReader.readElementText() );
+                  }
+                  else if ( xmlReader.name() == "ip" )
+                  {
+                    bookmark->setHostIP( xmlReader.readElementText() );
+                  }
+                  else if ( xmlReader.name() == "type" )
+                  {
+                    bookmark->setTypeString( xmlReader.readElementText() );
+                  }
+                  else if ( xmlReader.name() == "label" )
+                  {
+                    bookmark->setLabel( xmlReader.readElementText() );
+                  }
+                  else
+                  {
+                    // Do nothing
+                  }
+
+                  continue;
                 }
                 else
                 {
-                  // Do nothing
+                  continue;
                 }
-
-                continue;
+              }
+              
+              *bookmarks << bookmark;
+              
+              if (!groups->contains(bookmark->groupName()))
+              {
+                *groups << bookmark->groupName();
               }
               else
               {
-                continue;
-              }
-            }
-
-            d->bookmarks.append( bookmark );
-            
-            if ( !d->groups.contains( bookmark->groupName() ) )
-            {
-              d->groups << bookmark->groupName();
+                // Do nothing
+              }             
+              
             }
             else
             {
-              // Do nothing
+              continue;
             }
           }
           else
@@ -513,8 +561,7 @@ void Smb4KBookmarkHandler::loadBookmarks()
 
     if ( xmlReader.hasError() )
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->readingFileFailed( xmlFile, xmlReader.errorString() );
+      Smb4KNotification::readingFileFailed(xmlFile, xmlReader.errorString());
     }
     else
     {
@@ -525,8 +572,7 @@ void Smb4KBookmarkHandler::loadBookmarks()
   {
     if ( xmlFile.exists() )
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->openingFileFailed( xmlFile );
+      Smb4KNotification::openingFileFailed(xmlFile);
     }
     else
     {
@@ -683,4 +729,97 @@ void Smb4KBookmarkHandler::update() const
   }
 }
 
+
+void Smb4KBookmarkHandler::migrateProfile(const QString& from, const QString& to)
+{
+  QList<Smb4KBookmark *> allBookmarks;
+  QStringList allGroups; // Can be ignored.
+ 
+  // Read all entries for later conversion.
+  readBookmarks(&allBookmarks, &allGroups, true);
+  
+  // Replace the old profile name with the new one.
+  for (int i = 0; i < allBookmarks.size(); ++i)
+  {
+    if (QString::compare(allBookmarks.at(i)->profile(), from, Qt::CaseSensitive) == 0)
+    {
+      allBookmarks[i]->setProfile(to);
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  // Write the new list to the file.
+  writeBookmarkList(allBookmarks, true);
+  
+  // Profile settings changed, so invoke the slot.
+  slotActiveProfileChanged(Smb4KProfileManager::self()->activeProfile());
+  
+  // Clear the temporary lists of bookmarks and groups.
+  while (!allBookmarks.isEmpty())
+  {
+    delete allBookmarks.takeFirst();
+  }
+  
+  allGroups.clear();
+}
+
+
+void Smb4KBookmarkHandler::removeProfile(const QString& name)
+{
+  QList<Smb4KBookmark *> allBookmarks;
+  QStringList allGroups; // Can be ignored.
+ 
+  // Read all entries for later removal.
+  readBookmarks(&allBookmarks, &allGroups, true);
+  
+  QMutableListIterator<Smb4KBookmark *> it(allBookmarks);
+  
+  while (it.hasNext())
+  {
+    Smb4KBookmark *bookmark = it.next();
+    
+    if (QString::compare(bookmark->profile(), name, Qt::CaseSensitive) == 0)
+    {
+      it.remove();
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  // Write the new list to the file.
+  writeBookmarkList(allBookmarks, true);
+  
+  // Profile settings changed, so invoke the slot.
+  slotActiveProfileChanged(Smb4KProfileManager::self()->activeProfile());
+  
+  // Clear the temporary lists of bookmarks and groups.
+  while (!allBookmarks.isEmpty())
+  {
+    delete allBookmarks.takeFirst();
+  }
+  
+  allGroups.clear();
+}
+
+
+void Smb4KBookmarkHandler::slotActiveProfileChanged(const QString &/*activeProfile*/)
+{
+  // Clear the list of bookmarks and the list of groups.
+  while (!d->bookmarks.isEmpty())
+  {
+    delete d->bookmarks.takeFirst();
+  }
+  
+  d->groups.clear();
+  
+  // Reload the bookmarks and groups.
+  readBookmarks(&d->bookmarks, &d->groups, false);
+}
+
+
 #include "smb4kbookmarkhandler.moc"
