@@ -2,7 +2,7 @@
     smb4kbookmarkhandler  -  This class handles the bookmarks.
                              -------------------
     begin                : Fr Jan 9 2004
-    copyright            : (C) 2004-2013 by Alexander Reinholdt
+    copyright            : (C) 2004-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -43,6 +43,7 @@ class Smb4KShare;
 class Smb4KBookmarkHandlerPrivate;
 class Smb4KBookmarkDialog;
 class Smb4KBookmarkEditor;
+class Smb4KProfileManager;
 
 
 /**
@@ -57,6 +58,7 @@ class KDE_EXPORT Smb4KBookmarkHandler : public QObject
   Q_OBJECT  
   
   friend class Smb4KBookmarkHandlerPrivate;
+  friend class Smb4KProfileManager;
 
   public:
     /**
@@ -160,7 +162,7 @@ class KDE_EXPORT Smb4KBookmarkHandler : public QObject
     /**
      * Opens the bookmark editor
      * 
-     * @param parent        The parent widget
+     * @param parent          The parent widget
      */
     void editBookmarks( QWidget *parent = 0 );
 
@@ -169,15 +171,26 @@ class KDE_EXPORT Smb4KBookmarkHandler : public QObject
      * Signal emitted when the list of bookmarks has been updated.
      */
     void updated();
-
+    
+  protected Q_SLOTS:
+    /**
+     * This slot is called if the active profile changed.
+     * 
+     * @param activeProfile   The name of the active profile
+     */
+    void slotActiveProfileChanged(const QString &activeProfile);
+    
   private:
     /**
-     * This function loads the list of bookmarks from the bookmarks file.
-     * When it finishes, the updated() signal is emitted. So, if you
-     * want to access the list of bookmarks immediately after they were read,
-     * connect a slot to that signal.
+     * This function reads the list of bookmarks from the bookmarks file.
+     * 
+     * @param bookmarks       The list that should be filled with the bookmarks
+     * @param groups          The list that should be filled with the bookmark groups 
+     * @param allBookmarks    Read all bookmarks ignoring profiles
      */
-    void loadBookmarks();
+    void readBookmarks(QList<Smb4KBookmark *> *bookmarks,
+                       QStringList *groups,
+                       bool allBookmarks);
 
     /**
      * This function updates the data of the bookmarks, i.e. is searches for
@@ -187,15 +200,17 @@ class KDE_EXPORT Smb4KBookmarkHandler : public QObject
     void update() const;
 
     /**
-     * This function writes a new list of bookmarks. The old list will be
-     * deleted. It should be used, if you manipulated the list of bookmarks
-     * i. e. by a bookmark editor. When this function finishes, the
-     * bookmarksUpdated() signal will be emitted.
+     * This function writes the bookmarks to the disk. If @p listOnly is
+     * set to TRUE, only the list that was passed will be written to the 
+     * file replacing the existing bookmarks. If it is FALSE (the default),
+     * the list will be merged with the existing bookmarks. 
      *
      * @param list          The (new) list of bookmarks that is to be written
      *                      to the bookmark file
+     * @param listOnly      If TRUE only the passed list will be written to
+     *                      the file.
      */
-    void writeBookmarkList( const QList<Smb4KBookmark *> &list );
+    void writeBookmarkList(const QList<Smb4KBookmark *> &list, bool listOnly = false);
 
     /**
      * This function adds several bookmarks at once. It takes a list of
@@ -209,6 +224,25 @@ class KDE_EXPORT Smb4KBookmarkHandler : public QObject
     void addBookmarks( const QList<Smb4KBookmark *> &list,
                        bool replace = false );
     
+    /**
+     * Migrates one profile to another.
+     * 
+     * This function is meant to be used by the profile manager.
+     * 
+     * @param from        The name of the old profile.
+     * @param to          The name of the new profile.
+     */
+    void migrateProfile(const QString &from, const QString &to);
+    
+    /**
+     * Removes a profile from the list of profiles.
+     * 
+     * This function is meant to be used by the profile manager.
+     * 
+     * @param name        The name of the profile.
+     */
+    void removeProfile(const QString &name);
+    
     /**
      * Pointer to Smb4KBookmarkHandlerPrivate class
      */
