@@ -2,7 +2,7 @@
     smb4kcustomoptions - This class carries custom options
                              -------------------
     begin                : Fr 29 Apr 2011
-    copyright            : (C) 2011-2013 by Alexander Reinholdt
+    copyright            : (C) 2011-2015 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -37,10 +37,6 @@
 // KDE includes
 #include <kuser.h>
 
-// system specific includes
-#include <unistd.h>
-#include <sys/types.h>
-
 
 class Smb4KCustomOptionsPrivate
 {
@@ -48,14 +44,14 @@ class Smb4KCustomOptionsPrivate
     QString workgroup;
     KUrl url;
     QHostAddress ip;
-    Smb4KCustomOptions::Type type;
+    NetworkItem type;
     Smb4KCustomOptions::Remount remount;
     QString profile;
     int smbPort;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
     int fileSystemPort;
-    Smb4KCustomOptions::WriteAccess writeAccess;
     Smb4KCustomOptions::SecurityMode securityMode;
+    Smb4KCustomOptions::WriteAccess writeAccess;
 #endif
     Smb4KCustomOptions::ProtocolHint protocolHint;
     Smb4KCustomOptions::Kerberos kerberos;
@@ -67,73 +63,73 @@ class Smb4KCustomOptionsPrivate
 };
 
 
-Smb4KCustomOptions::Smb4KCustomOptions( Smb4KHost *host )
-: d( new Smb4KCustomOptionsPrivate )
+Smb4KCustomOptions::Smb4KCustomOptions(Smb4KHost *host)
+: d(new Smb4KCustomOptionsPrivate)
 {
   d->workgroup      = host->workgroupName();
   d->url            = host->url();
   d->type           = Host;
   d->remount        = UndefinedRemount;
   d->smbPort        = host->port() != -1 ? host->port() : 139;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
   d->fileSystemPort = 445;
-  d->writeAccess    = UndefinedWriteAccess;
   d->securityMode   = UndefinedSecurityMode;
+  d->writeAccess    = UndefinedWriteAccess;
 #endif
   d->protocolHint   = UndefinedProtocolHint;
   d->kerberos       = UndefinedKerberos;
-  d->user           = KUser( getuid() );
-  d->group          = KUserGroup( getgid() );
-  d->ip.setAddress( host->ip() );
+  d->user           = KUser(KUser::UseRealUserID);
+  d->group          = KUserGroup(KUser::UseRealUserID);
+  d->ip.setAddress(host->ip());
   d->wol_first_scan = false;
   d->wol_mount      = false;
 }
 
-Smb4KCustomOptions::Smb4KCustomOptions( Smb4KShare *share )
-: d( new Smb4KCustomOptionsPrivate )
+Smb4KCustomOptions::Smb4KCustomOptions(Smb4KShare *share)
+: d(new Smb4KCustomOptionsPrivate)
 {
   d->url            = share->url();
   d->workgroup      = share->workgroupName();
   d->type           = Share;
   d->remount        = UndefinedRemount;
   d->smbPort        = 139;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
   d->fileSystemPort = share->port() != -1 ? share->port() : 445;
-  d->writeAccess    = UndefinedWriteAccess;
   d->securityMode   = UndefinedSecurityMode;
+  d->writeAccess    = UndefinedWriteAccess;
 #endif
   d->protocolHint   = UndefinedProtocolHint;
   d->kerberos       = UndefinedKerberos;
-  d->user           = KUser( share->uid() );
-  d->group          = KUserGroup( share->gid() );
-  d->ip.setAddress( share->hostIP() );
+  d->user           = KUser(share->uid());
+  d->group          = KUserGroup(share->gid());
+  d->ip.setAddress(share->hostIP());
   d->wol_first_scan = false;
   d->wol_mount      = false;
 }
 
 
-Smb4KCustomOptions::Smb4KCustomOptions( const Smb4KCustomOptions &o )
-: d( new Smb4KCustomOptionsPrivate )
+Smb4KCustomOptions::Smb4KCustomOptions(const Smb4KCustomOptions &o)
+: d(new Smb4KCustomOptionsPrivate)
 {
   *d = *o.d;
 }
 
 
 Smb4KCustomOptions::Smb4KCustomOptions()
-: d( new Smb4KCustomOptionsPrivate )
+: d(new Smb4KCustomOptionsPrivate)
 {
   d->type           = Unknown;
   d->remount        = UndefinedRemount;
   d->smbPort        = 139;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
   d->fileSystemPort = 445;
-  d->writeAccess    = UndefinedWriteAccess;
   d->securityMode   = UndefinedSecurityMode;
+  d->writeAccess    = UndefinedWriteAccess;
 #endif
   d->protocolHint   = UndefinedProtocolHint;
   d->kerberos       = UndefinedKerberos;
-  d->user           = KUser( getuid() );
-  d->group          = KUserGroup( getgid() );
+  d->user           = KUser(KUser::UseRealUserID);
+  d->group          = KUserGroup(KUser::UseRealUserID);
   d->wol_first_scan = false;
   d->wol_mount      = false;
 }
@@ -144,11 +140,11 @@ Smb4KCustomOptions::~Smb4KCustomOptions()
 }
 
 
-void Smb4KCustomOptions::setHost( Smb4KHost *host )
+void Smb4KCustomOptions::setHost(Smb4KHost *host)
 {
-  Q_ASSERT( host );
+  Q_ASSERT(host);
   
-  switch ( d->type )
+  switch (d->type)
   {
     case Unknown:
     {
@@ -156,10 +152,10 @@ void Smb4KCustomOptions::setHost( Smb4KHost *host )
       d->url            = host->url();
       d->type           = Host;
       d->smbPort        = host->port() != -1 ? host->port() : 139;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
       d->fileSystemPort = 445;
 #endif
-      d->ip.setAddress( host->ip() );
+      d->ip.setAddress(host->ip());
       break;
     }
     default:
@@ -170,11 +166,11 @@ void Smb4KCustomOptions::setHost( Smb4KHost *host )
 }
 
 
-void Smb4KCustomOptions::setShare( Smb4KShare *share )
+void Smb4KCustomOptions::setShare(Smb4KShare *share)
 {
-  Q_ASSERT( share );
+  Q_ASSERT(share);
   
-  switch ( d->type )
+  switch (d->type)
   {
     case Unknown:
     {
@@ -182,26 +178,26 @@ void Smb4KCustomOptions::setShare( Smb4KShare *share )
       d->workgroup      = share->workgroupName();
       d->type           = Share;
       d->smbPort        = 139;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
       d->fileSystemPort = share->port() != -1 ? share->port() : 445;
 #endif
-      d->user           = KUser( share->uid() );
-      d->group          = KUserGroup( share->gid() );
-      d->ip.setAddress( share->hostIP() );
+      d->user           = KUser(share->uid());
+      d->group          = KUserGroup(share->gid());
+      d->ip.setAddress(share->hostIP());
       break;
     }
     case Host:
     {
-      if ( QString::compare( unc(), share->hostUNC(), Qt::CaseInsensitive ) == 0 )
+      if (QString::compare(unc(), share->hostUNC(), Qt::CaseInsensitive) == 0)
       {
         d->url            = share->url();
         d->type           = Share;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
         d->fileSystemPort = share->port() != -1 ? share->port() : d->fileSystemPort;
 #endif
-        d->user           = KUser( share->uid() );
-        d->group          = KUserGroup( share->gid() );
-        d->ip.setAddress( share->hostIP() );
+        d->user           = KUser(share->uid());
+        d->group          = KUserGroup(share->gid());
+        d->ip.setAddress(share->hostIP());
       }
       else
       {
@@ -217,15 +213,15 @@ void Smb4KCustomOptions::setShare( Smb4KShare *share )
 }
 
 
-Smb4KCustomOptions::Type Smb4KCustomOptions::type() const
+Smb4KGlobal::NetworkItem Smb4KCustomOptions::type() const
 {
   return d->type;
 }
 
 
-void Smb4KCustomOptions::setRemount( Smb4KCustomOptions::Remount remount )
+void Smb4KCustomOptions::setRemount(Smb4KCustomOptions::Remount remount)
 {
-  switch ( d->type )
+  switch (d->type)
   {
     case Share:
     {
@@ -247,7 +243,7 @@ Smb4KCustomOptions::Remount Smb4KCustomOptions::remount() const
 }
 
 
-void Smb4KCustomOptions::setProfile( const QString &profile )
+void Smb4KCustomOptions::setProfile(const QString &profile)
 {
   d->profile = profile;
 }
@@ -259,7 +255,7 @@ QString Smb4KCustomOptions::profile() const
 }
 
 
-void Smb4KCustomOptions::setWorkgroupName( const QString &workgroup )
+void Smb4KCustomOptions::setWorkgroupName(const QString &workgroup)
 {
   d->workgroup = workgroup;
 }
@@ -270,16 +266,16 @@ QString Smb4KCustomOptions::workgroupName() const
 }
 
 
-void Smb4KCustomOptions::setURL( const KUrl &url )
+void Smb4KCustomOptions::setURL(const KUrl &url)
 {
   d->url = url;
 }
 
 
-void Smb4KCustomOptions::setURL( const QString &url )
+void Smb4KCustomOptions::setURL(const QString &url)
 {
-  d->url.setUrl( url, KUrl::TolerantMode );
-  d->url.setProtocol( "smb" );
+  d->url.setUrl(url, KUrl::TolerantMode);
+  d->url.setProtocol("smb");
 }
 
 
@@ -293,13 +289,13 @@ QString Smb4KCustomOptions::unc() const
 {
   QString unc;
   
-  switch ( d->type )
+  switch (d->type)
   {
     case Host:
     {
-      if ( !hostName().isEmpty() )
+      if (!hostName().isEmpty())
       {
-        unc = QString( "//%1" ).arg( hostName() );
+        unc = QString("//%1").arg(hostName());
       }
       else
       {
@@ -309,9 +305,9 @@ QString Smb4KCustomOptions::unc() const
     }
     case Share:
     {
-      if ( !hostName().isEmpty() && !shareName().isEmpty() )
+      if (!hostName().isEmpty() && !shareName().isEmpty())
       {
-        unc = QString( "//%1/%2" ).arg( hostName() ).arg( shareName() );
+        unc = QString("//%1/%2").arg(hostName()).arg(shareName());
       }
       else
       {
@@ -337,9 +333,9 @@ QString Smb4KCustomOptions::hostName() const
 
 QString Smb4KCustomOptions::shareName() const
 {
-  if ( d->url.path().startsWith( '/' ) )
+  if (d->url.path().startsWith('/'))
   {
-    return d->url.path().remove( 0, 1 );
+    return d->url.path().remove(0, 1);
   }
   else
   {
@@ -350,9 +346,9 @@ QString Smb4KCustomOptions::shareName() const
 }
 
 
-void Smb4KCustomOptions::setIP( const QString &ip )
+void Smb4KCustomOptions::setIP(const QString &ip)
 {
-  d->ip.setAddress( ip );
+  d->ip.setAddress(ip);
 }
 
 
@@ -362,15 +358,15 @@ QString Smb4KCustomOptions::ip() const
 }
 
 
-void Smb4KCustomOptions::setSMBPort( int port )
+void Smb4KCustomOptions::setSMBPort(int port)
 {
   d->smbPort = port;
 
-  switch ( d->type )
+  switch (d->type)
   {
     case Host:
     {
-      d->url.setPort( port );
+      d->url.setPort(port);
       break;
     }
     default:
@@ -387,16 +383,16 @@ int Smb4KCustomOptions::smbPort() const
 }
 
 
-#ifndef Q_OS_FREEBSD
-void Smb4KCustomOptions::setFileSystemPort( int port )
+#if defined(Q_OS_LINUX)
+void Smb4KCustomOptions::setFileSystemPort(int port)
 {
   d->fileSystemPort = port;
   
-  switch ( d->type )
+  switch (d->type)
   {
     case Share:
     {
-      d->url.setPort( port );
+      d->url.setPort(port);
       break;
     }
     default:
@@ -413,18 +409,6 @@ int Smb4KCustomOptions::fileSystemPort() const
 }
 
 
-void Smb4KCustomOptions::setWriteAccess( Smb4KCustomOptions::WriteAccess access )
-{
-  d->writeAccess = access;
-}
-
-
-Smb4KCustomOptions::WriteAccess Smb4KCustomOptions::writeAccess() const
-{
-  return d->writeAccess;
-}
-
-
 void Smb4KCustomOptions::setSecurityMode(Smb4KCustomOptions::SecurityMode mode)
 {
   d->securityMode = mode;
@@ -435,10 +419,22 @@ Smb4KCustomOptions::SecurityMode Smb4KCustomOptions::securityMode() const
 {
   return d->securityMode;
 }
+
+
+void Smb4KCustomOptions::setWriteAccess(Smb4KCustomOptions::WriteAccess access)
+{
+  d->writeAccess = access;
+}
+
+
+Smb4KCustomOptions::WriteAccess Smb4KCustomOptions::writeAccess() const
+{
+  return d->writeAccess;
+}
 #endif
 
 
-void Smb4KCustomOptions::setProtocolHint( Smb4KCustomOptions::ProtocolHint protocol )
+void Smb4KCustomOptions::setProtocolHint(Smb4KCustomOptions::ProtocolHint protocol)
 {
   d->protocolHint = protocol;
 }
@@ -450,7 +446,7 @@ Smb4KCustomOptions::ProtocolHint Smb4KCustomOptions::protocolHint() const
 }
 
 
-void Smb4KCustomOptions::setUseKerberos( Smb4KCustomOptions::Kerberos kerberos )
+void Smb4KCustomOptions::setUseKerberos(Smb4KCustomOptions::Kerberos kerberos)
 {
   d->kerberos = kerberos;
 }
@@ -462,9 +458,9 @@ Smb4KCustomOptions::Kerberos Smb4KCustomOptions::useKerberos() const
 }
 
 
-void Smb4KCustomOptions::setUID( K_UID uid )
+void Smb4KCustomOptions::setUID(K_UID uid)
 {
-  d->user = KUser( uid );
+  d->user = KUser(uid);
 }
 
 
@@ -480,9 +476,9 @@ QString Smb4KCustomOptions::owner() const
 }
 
 
-void Smb4KCustomOptions::setGID( K_GID gid )
+void Smb4KCustomOptions::setGID(K_GID gid)
 {
-  d->group = KUserGroup( gid );
+  d->group = KUserGroup(gid);
 }
 
 
@@ -498,7 +494,7 @@ QString Smb4KCustomOptions::group() const
 }
 
 
-void Smb4KCustomOptions::setMACAddress( const QString &macAddress )
+void Smb4KCustomOptions::setMACAddress(const QString &macAddress)
 {
   d->mac = macAddress;
 }
@@ -510,7 +506,7 @@ QString Smb4KCustomOptions::macAddress() const
 }
 
 
-void Smb4KCustomOptions::setWOLSendBeforeNetworkScan( bool send )
+void Smb4KCustomOptions::setWOLSendBeforeNetworkScan(bool send)
 {
   d->wol_first_scan = send;
 }
@@ -522,7 +518,7 @@ bool Smb4KCustomOptions::wolSendBeforeNetworkScan() const
 }
 
 
-void Smb4KCustomOptions::setWOLSendBeforeMount( bool send )
+void Smb4KCustomOptions::setWOLSendBeforeMount(bool send)
 {
   d->wol_mount = send;
 }
@@ -538,21 +534,26 @@ QMap<QString, QString> Smb4KCustomOptions::customOptions() const
 {
   QMap<QString,QString> entries;
 
-  switch ( d->remount )
+  switch (d->remount)
   {
-    case DoRemount:
+    case RemountOnce:
     {
-      entries.insert( "remount", "true" );
+      entries.insert("remount", "once");
       break;
     }
-    case NoRemount:
+    case RemountAlways:
     {
-      entries.insert( "remount", "false" );
+      entries.insert("remount", "always");
       break;
     }
+    case RemountNever:
+    {
+      entries.insert("remount", "never");
+      break;      
+    }
     case UndefinedRemount:
     {
-      entries.insert( "remount", QString() );
+      entries.insert("remount", QString());
       break;
     }
     default:
@@ -561,25 +562,60 @@ QMap<QString, QString> Smb4KCustomOptions::customOptions() const
     }
   }
 
-  entries.insert( "smb_port", QString( "%1" ).arg( smbPort() ) );
-#ifndef Q_OS_FREEBSD
-  entries.insert( "filesystem_port", QString( "%1" ).arg( fileSystemPort() ) );
+  entries.insert("smb_port", QString("%1").arg(smbPort()));
+#if defined(Q_OS_LINUX)
+  entries.insert("filesystem_port", QString("%1").arg(fileSystemPort()));
   
-  switch ( d->writeAccess )
+  switch (d->securityMode)
   {
-    case ReadWrite:
+    case NoSecurityMode:
     {
-      entries.insert( "write_access", "true" );
+      entries.insert("security_mode", "none");
       break;
     }
-    case ReadOnly:
+    case Krb5:
     {
-      entries.insert( "write_access", "false" );
+      entries.insert("security_mode", "krb5");
       break;
     }
-    case UndefinedWriteAccess:
+    case Krb5i:
     {
-      entries.insert( "write_access", QString() );
+      entries.insert("security_mode", "krb5i");
+      break;
+    }
+    case Ntlm:
+    {
+      entries.insert("security_mode", "ntlm");
+      break;
+    }
+    case Ntlmi:
+    {
+      entries.insert("security_mode", "ntlmi");
+      break;
+    }
+    case Ntlmv2:
+    {
+      entries.insert("security_mode", "ntlmv2");
+      break;
+    }
+    case Ntlmv2i:
+    {
+      entries.insert("security_mode", "ntlmv2i");
+      break;
+    }
+    case Ntlmssp:
+    {
+      entries.insert("security_mode", "ntlmssp");
+      break;
+    }
+    case Ntlmsspi:
+    {
+      entries.insert("security_mode", "ntlmsspi");
+      break;
+    }
+    case UndefinedSecurityMode:
+    {
+      entries.insert("security_mode", QString());
       break;
     }
     default:
@@ -587,57 +623,22 @@ QMap<QString, QString> Smb4KCustomOptions::customOptions() const
       break;
     }
   }
-  
-  switch ( d->securityMode )
+
+  switch (d->writeAccess)
   {
-    case NoSecurityMode:
+    case ReadWrite:
     {
-      entries.insert( "security_mode", "none" );
+      entries.insert("write_access", "true");
       break;
     }
-    case Krb5:
+    case ReadOnly:
     {
-      entries.insert( "security_mode", "krb5" );
+      entries.insert("write_access", "false");
       break;
     }
-    case Krb5i:
+    case UndefinedWriteAccess:
     {
-      entries.insert( "security_mode", "krb5i" );
-      break;
-    }
-    case Ntlm:
-    {
-      entries.insert( "security_mode", "ntlm" );
-      break;
-    }
-    case Ntlmi:
-    {
-      entries.insert( "security_mode", "ntlmi" );
-      break;
-    }
-    case Ntlmv2:
-    {
-      entries.insert( "security_mode", "ntlmv2" );
-      break;
-    }
-    case Ntlmv2i:
-    {
-      entries.insert( "security_mode", "ntlmv2i" );
-      break;
-    }
-    case Ntlmssp:
-    {
-      entries.insert( "security_mode", "ntlmssp" );
-      break;
-    }
-    case Ntlmsspi:
-    {
-      entries.insert( "security_mode", "ntlmsspi" );
-      break;
-    }
-    case UndefinedSecurityMode:
-    {
-      entries.insert( "security_mode", QString() );
+      entries.insert("write_access", QString());
       break;
     }
     default:
@@ -647,31 +648,31 @@ QMap<QString, QString> Smb4KCustomOptions::customOptions() const
   }
 #endif
 
-  switch ( d->protocolHint )
+  switch (d->protocolHint)
   {
     case Automatic:
     {
-      entries.insert( "protocol", "auto" );
+      entries.insert("protocol", "auto");
       break;
     }
     case RPC:
     {
-      entries.insert( "protocol", "rpc" );
+      entries.insert("protocol", "rpc");
       break;
     }
     case RAP:
     {
-      entries.insert( "protocol", "rap" );
+      entries.insert("protocol", "rap");
       break;
     }
     case ADS:
     {
-      entries.insert( "protocol", "ads" );
+      entries.insert("protocol", "ads");
       break;
     }
     case UndefinedProtocolHint:
     {
-      entries.insert( "protocol", QString() );
+      entries.insert("protocol", QString());
       break;
     }
     default:
@@ -680,21 +681,21 @@ QMap<QString, QString> Smb4KCustomOptions::customOptions() const
     }
   }
 
-  switch ( d->kerberos )
+  switch (d->kerberos)
   {
     case UseKerberos:
     {
-      entries.insert( "kerberos", "true" );
+      entries.insert("kerberos", "true");
       break;
     }
     case NoKerberos:
     {
-      entries.insert( "kerberos", "false" );
+      entries.insert("kerberos", "false");
       break;
     }
     case UndefinedKerberos:
     {
-      entries.insert( "kerberos", QString() );
+      entries.insert("kerberos", QString());
       break;
     }
     default:
@@ -703,22 +704,22 @@ QMap<QString, QString> Smb4KCustomOptions::customOptions() const
     }
   }
   
-  entries.insert( "uid", QString( "%1" ).arg( d->user.uid() ) );
-  entries.insert( "owner", d->user.loginName() );
-  entries.insert( "gid", QString( "%1" ).arg( d->group.gid() ) );
-  entries.insert( "group", d->group.name() );
-  entries.insert( "mac_address", d->mac );
-  entries.insert( "wol_send_before_first_scan", d->wol_first_scan ? "true" : "false" );
-  entries.insert( "wol_send_before_mount", d->wol_mount ? "true" : "false" );
+  entries.insert("uid", QString("%1").arg(d->user.uid()));
+  entries.insert("owner", d->user.loginName());
+  entries.insert("gid", QString("%1").arg(d->group.gid()));
+  entries.insert("group", d->group.name());
+  entries.insert("mac_address", d->mac);
+  entries.insert("wol_send_before_first_scan", d->wol_first_scan ? "true" : "false");
+  entries.insert("wol_send_before_mount", d->wol_mount ? "true" : "false");
   
   return entries;
 }
 
 
-bool Smb4KCustomOptions::equals( Smb4KCustomOptions *options ) const
+bool Smb4KCustomOptions::equals(Smb4KCustomOptions *options, bool fullCheck) const
 {
   // Type
-  if ( d->type != options->type() )
+  if (d->type != options->type())
   {
     return false;
   }
@@ -726,9 +727,9 @@ bool Smb4KCustomOptions::equals( Smb4KCustomOptions *options ) const
   {
     // Do nothing
   }
-  
+
   // Profile
-  if ( QString::compare( d->profile, options->profile() ) != 0 )
+  if (QString::compare(d->profile, options->profile()) != 0)
   {
     return false;
   }
@@ -738,7 +739,7 @@ bool Smb4KCustomOptions::equals( Smb4KCustomOptions *options ) const
   }
   
   // Workgroup
-  if ( QString::compare( d->workgroup, options->workgroupName(), Qt::CaseInsensitive ) != 0 )
+  if (QString::compare(d->workgroup, options->workgroupName(), Qt::CaseInsensitive) != 0)
   {
     return false;
   }
@@ -747,9 +748,8 @@ bool Smb4KCustomOptions::equals( Smb4KCustomOptions *options ) const
     // Do nothing
   }
   
-  // URL - Instead of checking if the whole network item equals
-  //  the one defined here, it is sufficient to check the URL.
-  if ( d->url != options->url() )
+  // UNC
+  if (QString::compare(unc(), options->unc(), Qt::CaseInsensitive) != 0)
   {
     return false;
   }
@@ -758,119 +758,146 @@ bool Smb4KCustomOptions::equals( Smb4KCustomOptions *options ) const
     // Do nothing
   }
   
-  // IP address
-  if ( QString::compare( d->ip.toString(), options->ip() ) != 0 )
+  // Full check
+  if (fullCheck)
   {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
+    // IP address
+    if (QString::compare(d->ip.toString(), options->ip()) != 0)
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Remounting
+    if (d->remount != options->remount())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // SMB port
+    if (d->smbPort != options->smbPort())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+#if defined(Q_OS_LINUX)
+    // File system port (used for mounting)
+    if (d->fileSystemPort != options->fileSystemPort())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Security mode
+    if (d->securityMode != options->securityMode())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
 
-  // SMB port
-  if ( d->smbPort != options->smbPort() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-#ifndef Q_OS_FREEBSD
-  
-  // File system port (used for mounting)
-  if ( d->fileSystemPort != options->fileSystemPort() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-
-  // Write access
-  if ( d->writeAccess != options->writeAccess() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
+    // Write access
+    if (d->writeAccess != options->writeAccess())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
 #endif
 
-  // Protocol hint
-  if ( d->protocolHint != options->protocolHint() )
-  {
-    return false;
+    // Protocol hint
+    if (d->protocolHint != options->protocolHint())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Kerberos
+    if (d->kerberos != options->useKerberos())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // UID
+    if (d->user.uid() != options->uid())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // GID
+    if (d->group.gid() != options->gid())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // MAC address
+    if (QString::compare(d->mac, options->macAddress()) != 0)
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Send WOL packages before first scan
+    if (d->wol_first_scan != options->wolSendBeforeNetworkScan())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Send WOL packages before mount
+    if (d->wol_mount != options->wolSendBeforeMount())
+    {
+      return false;
+    }
+    else
+    {
+      // Do nothing
+    }
   }
   else
   {
     // Do nothing
   }
   
-  // Kerberos
-  if ( d->kerberos != options->useKerberos() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  // UID
-  if ( d->user.uid() != options->uid() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  // GID
-  if ( d->group.gid() != options->gid() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  // MAC address
-  if ( QString::compare( d->mac, options->macAddress() ) != 0 )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  // Send WOL packages before first scan
-  if ( d->wol_first_scan != options->wolSendBeforeNetworkScan() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  // Send WOL packages before mount
-  if ( d->wol_mount != options->wolSendBeforeMount() )
-  {
-    return false;
-  }
-  else
-  {
-    // Do nothing
-  }
-
   return true;
 }
 
@@ -878,7 +905,7 @@ bool Smb4KCustomOptions::equals( Smb4KCustomOptions *options ) const
 bool Smb4KCustomOptions::isEmpty()
 {
   // Type
-  if ( d->type != Unknown )
+  if (d->type != Unknown)
   {
     return false;
   }
@@ -888,7 +915,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // Profile
-  if ( !d->profile.isEmpty() )
+  if (!d->profile.isEmpty())
   {
     return false;
   }
@@ -898,7 +925,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // Workgroup
-  if ( !d->workgroup.isEmpty() )
+  if (!d->workgroup.isEmpty())
   {
     return false;
   }
@@ -908,7 +935,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // URL
-  if ( !d->url.isEmpty() )
+  if (!d->url.isEmpty())
   {
     return false;
   }
@@ -918,7 +945,17 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // IP address
-  if ( !d->ip.isNull() )
+  if (!d->ip.isNull())
+  {
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Remounting
+  if (d->remount != Smb4KCustomOptions::UndefinedRemount)
   {
     return false;
   }
@@ -928,7 +965,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
 
   // SMB port
-  if ( d->smbPort != 139 )
+  if (d->smbPort != 139)
   {
     return false;
   }
@@ -937,10 +974,19 @@ bool Smb4KCustomOptions::isEmpty()
     // Do nothing
   }
   
-#ifndef Q_OS_FREEBSD
-  
+#if defined(Q_OS_LINUX)
   // File system port (used for mounting)
-  if ( d->fileSystemPort != 445 )
+  if (d->fileSystemPort != 445)
+  {
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Security mode
+  if (d->securityMode != Smb4KCustomOptions::UndefinedSecurityMode)
   {
     return false;
   }
@@ -950,7 +996,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
 
   // Write access
-  if ( d->writeAccess != UndefinedWriteAccess )
+  if (d->writeAccess != UndefinedWriteAccess)
   {
     return false;
   }
@@ -961,7 +1007,7 @@ bool Smb4KCustomOptions::isEmpty()
 #endif
 
   // Protocol hint
-  if ( d->protocolHint != UndefinedProtocolHint )
+  if (d->protocolHint != UndefinedProtocolHint)
   {
     return false;
   }
@@ -971,7 +1017,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // Kerberos
-  if ( d->kerberos != UndefinedKerberos )
+  if (d->kerberos != UndefinedKerberos)
   {
     return false;
   }
@@ -981,7 +1027,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // UID
-  if ( d->user.uid() != getuid() )
+  if (d->user.uid() != KUser(KUser::UseRealUserID).uid())
   {
     return false;
   }
@@ -991,7 +1037,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // GID
-  if ( d->group.gid() != getgid() )
+  if (d->group.gid() != KUserGroup(KUser::UseRealUserID).gid())
   {
     return false;
   }
@@ -1001,7 +1047,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // MAC address
-  if ( !d->mac.isNull() )
+  if (!d->mac.isNull())
   {
     return false;
   }
@@ -1011,7 +1057,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // Send WOL packages before first scan
-  if ( d->wol_first_scan )
+  if (d->wol_first_scan)
   {
     return false;
   }
@@ -1021,7 +1067,7 @@ bool Smb4KCustomOptions::isEmpty()
   }
   
   // Send WOL packages before mount
-  if ( d->wol_mount )
+  if (d->wol_mount)
   {
     return false;
   }
