@@ -3,7 +3,7 @@
     Smb4KCustomOptionsManager class
                              -------------------
     begin                : Fr 29 Apr 2011
-    copyright            : (C) 2011-2012 by Alexander Reinholdt
+    copyright            : (C) 2011-2015 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -65,13 +65,16 @@ class Smb4KCustomOptionsDialog : public KDialog
     
   private:
     void setupView();
-    bool defaultValues();
+    bool checkDefaultValues();
+    void setDefaultValues();
+    void saveValues();
     Smb4KCustomOptions *m_options;
+    QCheckBox *m_remount;
     KIntNumInput *m_smb_port;
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
     KIntNumInput *m_fs_port;
-    KComboBox *m_write_access;
     KComboBox *m_security_mode;
+    KComboBox *m_write_access;
 #endif
     KComboBox *m_protocol_hint;
     KComboBox *m_user_id;
