@@ -3,7 +3,7 @@
     QtQuick
                              -------------------
     begin                : Mo 02 Sep 2013
-    copyright            : (C) 2013 by Alexander Reinholdt
+    copyright            : (C) 2013-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -42,6 +42,10 @@
 #include "smb4kprint.h"
 #include "smb4kbookmarkobject.h"
 #include "smb4kcustomoptionsmanager.h"
+#include "smb4kprofilemanager.h"
+
+// Qt includes
+#include <QtCore/QDebug>
 
 using namespace Smb4KGlobal;
 
@@ -49,97 +53,118 @@ using namespace Smb4KGlobal;
 Smb4KDeclarative::Smb4KDeclarative(QObject* parent)
 : QObject(parent), d( new Smb4KDeclarativePrivate )
 {
-  connect( Smb4KScanner::self(), SIGNAL(workgroups(QList<Smb4KWorkgroup*>)), this, SLOT(slotWorkgroupsListChanged()) );
-  connect( Smb4KScanner::self(), SIGNAL(hosts(Smb4KWorkgroup*,QList<Smb4KHost*>)), this, SLOT(slotHostsListChanged()) );
-  connect( Smb4KScanner::self(), SIGNAL(shares(Smb4KHost*,QList<Smb4KShare*>)), this, SLOT(slotSharesListChanged()) );
-  connect( Smb4KScanner::self(), SIGNAL(aboutToStart(Smb4KBasicNetworkItem*,int)), this, SIGNAL(busy()) );
-  connect( Smb4KScanner::self(), SIGNAL(finished(Smb4KBasicNetworkItem*,int)), this, SIGNAL(idle()) );
+  // Initialize the core.
+  Smb4KGlobal::initCore(true, false);
   
-  connect( Smb4KMounter::self(), SIGNAL(mounted(Smb4KShare*)), this, SLOT(slotMountedSharesListChanged()) );
-  connect( Smb4KMounter::self(), SIGNAL(unmounted(Smb4KShare*)), this, SLOT(slotMountedSharesListChanged()) );
-  connect( Smb4KMounter::self(), SIGNAL(aboutToStart(Smb4KShare*,int)), this, SIGNAL(busy()) );
-  connect( Smb4KMounter::self(), SIGNAL(finished(Smb4KShare*,int)), this, SIGNAL(idle()) );
+  // Connections
+  connect(Smb4KScanner::self(), SIGNAL(workgroups(QList<Smb4KWorkgroup*>)), this, SLOT(slotWorkgroupsListChanged()));
+  connect(Smb4KScanner::self(), SIGNAL(hosts(Smb4KWorkgroup*,QList<Smb4KHost*>)), this, SLOT(slotHostsListChanged()));
+  connect(Smb4KScanner::self(), SIGNAL(shares(Smb4KHost*,QList<Smb4KShare*>)), this, SLOT(slotSharesListChanged()));
+  connect(Smb4KScanner::self(), SIGNAL(aboutToStart(Smb4KBasicNetworkItem*,int)), this, SIGNAL(busy()));
+  connect(Smb4KScanner::self(), SIGNAL(finished(Smb4KBasicNetworkItem*,int)), this, SIGNAL(idle()));
   
-  connect( Smb4KPrint::self(), SIGNAL(aboutToStart(Smb4KShare*)), this, SIGNAL(busy()) );
-  connect( Smb4KPrint::self(), SIGNAL(finished(Smb4KShare*)), this, SIGNAL(idle()) );
+  connect(Smb4KMounter::self(), SIGNAL(mounted(Smb4KShare*)), this, SLOT(slotMountedSharesListChanged()));
+  connect(Smb4KMounter::self(), SIGNAL(unmounted(Smb4KShare*)), this, SLOT(slotMountedSharesListChanged()));
+  connect(Smb4KMounter::self(), SIGNAL(aboutToStart(Smb4KShare*,int)), this, SIGNAL(busy()));
+  connect(Smb4KMounter::self(), SIGNAL(finished(Smb4KShare*,int)), this, SIGNAL(idle()));
   
-  connect( Smb4KBookmarkHandler::self(), SIGNAL(updated()), this, SLOT(slotBookmarksListChanged()) );
+  connect(Smb4KPrint::self(), SIGNAL(aboutToStart(Smb4KShare*)), this, SIGNAL(busy()));
+  connect(Smb4KPrint::self(), SIGNAL(finished(Smb4KShare*)), this, SIGNAL(idle()));
+  
+  connect(Smb4KBookmarkHandler::self(), SIGNAL(updated()), this, SLOT(slotBookmarksListChanged()));
+  
+  connect(Smb4KProfileManager::self(), SIGNAL(profilesListChanged(QStringList)), this, SLOT(slotProfilesListChanged(QStringList)));
+  connect(Smb4KProfileManager::self(), SIGNAL(activeProfileChanged(QString)), this, SLOT(slotActiveProfileChanged(QString)));
+  connect(Smb4KProfileManager::self(), SIGNAL(profileUsageChanged(bool)), this, SLOT(slotProfileUsageChanged(bool)));
   
   // Do the initial loading of items.
   slotBookmarksListChanged();
+  slotProfilesListChanged(Smb4KProfileManager::self()->profilesList());
+  slotActiveProfileChanged(Smb4KProfileManager::self()->activeProfile());
+  slotProfileUsageChanged(Smb4KProfileManager::self()->useProfiles());
 }
 
 
 Smb4KDeclarative::~Smb4KDeclarative()
 {
-  while ( !d->workgroupObjects.isEmpty() )
+  while (!d->workgroupObjects.isEmpty())
   {
     delete d->workgroupObjects.takeFirst();
   }
 
-  while ( !d->hostObjects.isEmpty() )
+  while (!d->hostObjects.isEmpty())
   {
     delete d->hostObjects.takeFirst();
   }
 
-  while ( !d->shareObjects.isEmpty() )
+  while (!d->shareObjects.isEmpty())
   {
     delete d->shareObjects.takeFirst();
   }
   
-  while ( !d->mountedObjects.isEmpty() )
+  while (!d->mountedObjects.isEmpty())
   {
     delete d->mountedObjects.takeFirst();
   }
   
-  while ( !d->bookmarkObjects.isEmpty() )
+  while (!d->bookmarkObjects.isEmpty())
   {
     delete d->bookmarkObjects.takeFirst();
   }
   
-  while ( !d->bookmarkGroupObjects.isEmpty() )
+  while (!d->bookmarkGroupObjects.isEmpty())
   {
     delete d->bookmarkGroupObjects.takeFirst();
   }
+  
+  while (!d->profileObjects.isEmpty())
+  {
+    delete d->profileObjects.takeFirst();
+  }
 }
 
 
 QDeclarativeListProperty<Smb4KNetworkObject> Smb4KDeclarative::workgroups()
 {
-  return QDeclarativeListProperty<Smb4KNetworkObject>( this, d->workgroupObjects );
+  return QDeclarativeListProperty<Smb4KNetworkObject>(this, d->workgroupObjects);
 }
 
 
 QDeclarativeListProperty<Smb4KNetworkObject> Smb4KDeclarative::hosts()
 {
-  return QDeclarativeListProperty<Smb4KNetworkObject>( this, d->hostObjects );
+  return QDeclarativeListProperty<Smb4KNetworkObject>(this, d->hostObjects);
 }
 
 
 QDeclarativeListProperty<Smb4KNetworkObject> Smb4KDeclarative::shares()
 {
-  return QDeclarativeListProperty<Smb4KNetworkObject>( this, d->shareObjects );
+  return QDeclarativeListProperty<Smb4KNetworkObject>(this, d->shareObjects);
 }
 
 
 QDeclarativeListProperty<Smb4KNetworkObject> Smb4KDeclarative::mountedShares()
 {
-  return QDeclarativeListProperty<Smb4KNetworkObject>( this, d->mountedObjects );
+  return QDeclarativeListProperty<Smb4KNetworkObject>(this, d->mountedObjects);
 }
 
 
 QDeclarativeListProperty<Smb4KBookmarkObject> Smb4KDeclarative::bookmarks()
 {
-  return QDeclarativeListProperty<Smb4KBookmarkObject>( this, d->bookmarkObjects );
+  return QDeclarativeListProperty<Smb4KBookmarkObject>(this, d->bookmarkObjects);
 }
 
 
 QDeclarativeListProperty<Smb4KBookmarkObject> Smb4KDeclarative::bookmarkGroups()
 {
-  return QDeclarativeListProperty<Smb4KBookmarkObject>( this, d->bookmarkGroupObjects );
+  return QDeclarativeListProperty<Smb4KBookmarkObject>(this, d->bookmarkGroupObjects);
 }
 
 
+QDeclarativeListProperty<Smb4KProfileObject> Smb4KDeclarative::profiles()
+{
+  return QDeclarativeListProperty<Smb4KProfileObject>(this, d->profileObjects);
+}
+
 
 void Smb4KDeclarative::lookup( Smb4KNetworkObject *object )
 {
@@ -147,12 +172,12 @@ void Smb4KDeclarative::lookup( Smb4KNetworkObject *object )
   {
     switch ( object->type() )
     {
-      case Smb4KNetworkObject::Network:
+      case Network:
       {
         Smb4KScanner::self()->lookupDomains();
         break;
       }
-      case Smb4KNetworkObject::Workgroup:
+      case Workgroup:
       {
         // Check if the workgroup is known.
         Smb4KWorkgroup *workgroup = findWorkgroup( object->url().host().toUpper() );
@@ -167,7 +192,7 @@ void Smb4KDeclarative::lookup( Smb4KNetworkObject *object )
         }
         break;
       }
-      case Smb4KNetworkObject::Host:
+      case Host:
       {
         // Check if the host is known.
         Smb4KHost *host = findHost( object->url().host().toUpper() );
@@ -182,7 +207,7 @@ void Smb4KDeclarative::lookup( Smb4KNetworkObject *object )
         }
         break;
       }
-      case Smb4KNetworkObject::Share:
+      case Share:
       {
         break;
       }
@@ -209,7 +234,7 @@ Smb4KNetworkObject *Smb4KDeclarative::findNetworkItem( const QUrl &url, int type
   {  
     switch ( type )
     {
-      case Smb4KNetworkObject::Workgroup:
+      case Workgroup:
       {
         for ( int i = 0; i < d->workgroupObjects.size(); ++i )
         {
@@ -225,7 +250,7 @@ Smb4KNetworkObject *Smb4KDeclarative::findNetworkItem( const QUrl &url, int type
         }
         break;
       }
-      case Smb4KNetworkObject::Host:
+      case Host:
       {
         for ( int i = 0; i < d->hostObjects.size(); ++i )
         {
@@ -241,7 +266,7 @@ Smb4KNetworkObject *Smb4KDeclarative::findNetworkItem( const QUrl &url, int type
         }
         break;
       }
-      case Smb4KNetworkObject::Share:
+      case Share:
       {
         for ( int i = 0; i < d->shareObjects.size(); ++i )
         {
@@ -532,7 +557,7 @@ void Smb4KDeclarative::openCustomOptionsDialog(Smb4KNetworkObject *object)
   {
     switch ( object->type() )
     {
-      case Smb4KNetworkObject::Host:
+      case Host:
       {
         for ( int i = 0; i < hostsList().size(); ++i )
         {
@@ -548,7 +573,7 @@ void Smb4KDeclarative::openCustomOptionsDialog(Smb4KNetworkObject *object)
         }
         break;
       }
-      case Smb4KNetworkObject::Share:
+      case Share:
       {
         for ( int i = 0; i < sharesList().size(); ++i )
         {
@@ -613,6 +638,38 @@ void Smb4KDeclarative::abortPrinter()
 }
 
 
+QString Smb4KDeclarative::activeProfile() const
+{
+  QString activeProfile;
+  
+  for (int i = 0; i < d->profileObjects.size(); ++i)
+  {
+    if (d->profileObjects.at(i)->isActiveProfile())
+    {
+      activeProfile = d->profileObjects.at(i)->profileName();
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  return activeProfile;
+}
+
+
+void Smb4KDeclarative::setActiveProfile(const QString& profile)
+{
+  Smb4KProfileManager::self()->setActiveProfile(profile);
+}
+
+
+bool Smb4KDeclarative::profileUsage() const
+{
+  return Smb4KProfileManager::self()->useProfiles();
+}
+
+
 void Smb4KDeclarative::slotWorkgroupsListChanged()
 {
   // (Re)fill the list of workgroup objects.
@@ -649,7 +706,7 @@ void Smb4KDeclarative::slotHostsListChanged()
 
 void Smb4KDeclarative::slotSharesListChanged()
 {
-  // (Re)fill the list of share object.
+  // (Re)fill the list of share objects.
   while ( !d->shareObjects.isEmpty() )
   {
     delete d->shareObjects.takeFirst();
@@ -708,4 +765,56 @@ void Smb4KDeclarative::slotBookmarksListChanged()
 }
 
 
+void Smb4KDeclarative::slotProfilesListChanged(const QStringList& profiles)
+{
+  while (!d->profileObjects.isEmpty())
+  {
+    delete d->profileObjects.takeFirst();
+  }
+  
+  for (int i = 0; i < profiles.size(); ++i)
+  {
+    Smb4KProfileObject *profile = new Smb4KProfileObject();
+    profile->setProfileName(profiles.at(i));
+    
+    if (QString::compare(profiles.at(i), Smb4KProfileManager::self()->activeProfile()) == 0)
+    {
+      profile->setActiveProfile(true);
+    }
+    else
+    {
+      profile->setActiveProfile(false);
+    }
+    
+    d->profileObjects << profile;
+  }
+  
+  emit profilesListChanged();
+}
+
+
+void Smb4KDeclarative::slotActiveProfileChanged(const QString& activeProfile)
+{
+  for (int i = 0; i < d->profileObjects.size(); ++i)
+  {
+    if (QString::compare(d->profileObjects.at(i)->profileName(), activeProfile) == 0)
+    {
+      d->profileObjects[i]->setActiveProfile(true);
+    }
+    else
+    {
+      d->profileObjects[i]->setActiveProfile(false);
+    }
+  }
+  emit activeProfileChanged();
+}
+
+
+void Smb4KDeclarative::slotProfileUsageChanged(bool /*use*/)
+{
+  emit profileUsageChanged();
+}
+
+
+
 #include "smb4kdeclarative.moc"
