@@ -3,7 +3,7 @@
     Smb4KGlobal namespace.
                              -------------------
     begin                : Di Jul 24 2007
-    copyright            : (C) 2007-2014 by Alexander Reinholdt
+    copyright            : (C) 2007-2015 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -110,15 +110,24 @@ class Smb4KGlobalPrivate : public QObject
      * a busy cursor when they are doing something or not.
      */
     bool modifyCursor;
+    
+#ifdef Q_OS_LINUX
+    /**
+     * This list contains all whitelisted arguments for the mount.cifs binary and
+     * is only present under the Linux operatin system.
+     */
+    QStringList whitelistedMountArguments;
+#endif
 
   protected Q_SLOTS:
     /**
      * This slot does last things before the application quits
      */
     void slotAboutToQuit();
-
+    
   private:
     QMap<QString,QString> m_samba_options;
+    bool m_samba_options_read;
 };
 
 #endif
