@@ -2,7 +2,7 @@
     smb4kmounter.cpp  -  The core class that mounts the shares.
                              -------------------
     begin                : Die Jun 10 2003
-    copyright            : (C) 2003-2013 by Alexander Reinholdt
+    copyright            : (C) 2003-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -42,6 +42,7 @@
 #include "smb4kcustomoptionsmanager.h"
 #include "smb4kcustomoptions.h"
 #include "smb4kbookmark.h"
+#include "smb4kprofilemanager.h"
 
 // Qt includes
 #include <QtCore/QDir>
@@ -63,6 +64,7 @@
 #include <kio/job.h>
 #include <kio/jobclasses.h>
 #include <kdiskfreespaceinfo.h>
+#include <KUser>
 
 using namespace Smb4KGlobal;
 
@@ -91,21 +93,19 @@ Smb4KMounter::Smb4KMounter( QObject *parent )
   d->remountAttempts = 0;
   d->checks = 0;
   d->dialog = 0;
-  d->aboutToQuit = false;
   d->firstImportDone = false;
   d->importsAllowed = true;
+  d->internalReason = false;
+  d->activeProfile = Smb4KProfileManager::self()->activeProfile();
 
-  connect( QCoreApplication::instance(), SIGNAL(aboutToQuit()),
-           this,                         SLOT(slotAboutToQuit()) );
-
-  connect( Smb4KSolidInterface::self(),  SIGNAL(buttonPressed(Smb4KSolidInterface::ButtonType)),
-           this,                         SLOT(slotHardwareButtonPressed(Smb4KSolidInterface::ButtonType)) );
-
-  connect( Smb4KSolidInterface::self(),  SIGNAL(wokeUp()),
-           this,                         SLOT(slotComputerWokeUp()) );
-
-  connect( Smb4KSolidInterface::self(),  SIGNAL(networkStatusChanged(Smb4KSolidInterface::ConnectionStatus)),
-           this,                         SLOT(slotNetworkStatusChanged(Smb4KSolidInterface::ConnectionStatus)) );
+  connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()),
+          this,                         SLOT(slotAboutToQuit()));
+  connect(Smb4KSolidInterface::self(),  SIGNAL(networkStatusChanged(Smb4KSolidInterface::ConnectionStatus)),
+          this,                         SLOT(slotNetworkStatusChanged(Smb4KSolidInterface::ConnectionStatus)));
+  connect(Smb4KProfileManager::self(),  SIGNAL(migratedProfile(QString,QString)),
+          this,                         SLOT(slotProfileMigrated(QString,QString)));
+  connect(Smb4KProfileManager::self(),  SIGNAL(activeProfileChanged(QString)),
+          this,                         SLOT(slotActiveProfileChanged(QString)));
 }
 
 
@@ -233,7 +233,9 @@ bool Smb4KMounter::isRunning()
 
 void Smb4KMounter::triggerRemounts( bool fill_list )
 {
-  if ( Smb4KSettings::remountShares() || d->hardwareReason )
+  if (Smb4KSettings::remountShares() /* one-time remounts */ || 
+      !Smb4KCustomOptionsManager::self()->sharesToRemount().isEmpty() /* permanent remounts */ ||
+      d->internalReason /* internal reason */)
   {
     if ( fill_list )
     {
@@ -343,7 +345,7 @@ void Smb4KMounter::import( bool check_inaccessible )
 
   for ( int i = 0; i < mount_points.size(); ++i )
   {
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
     if ( QString::compare( mount_points.at( i )->mountType(), "cifs" ) == 0 )
 #else
     if ( QString::compare( mount_points.at( i )->mountType(), "smbfs" ) == 0 )
@@ -352,7 +354,7 @@ void Smb4KMounter::import( bool check_inaccessible )
       Smb4KShare *share = new Smb4KShare( mount_points.at( i )->mountedFrom() );
       share->setPath( mount_points.at( i )->mountPoint() );
 
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
       share->setFileSystem( Smb4KShare::CIFS );
 
       // Check if the share is new and we have to open /proc/mounts (if it exists)
@@ -397,8 +399,7 @@ void Smb4KMounter::import( bool check_inaccessible )
         }
         else
         {
-          Smb4KNotification *notification = new Smb4KNotification();
-          notification->openingFileFailed( proc_mounts );
+          Smb4KNotification::openingFileFailed(proc_mounts);
           return;
         }
 
@@ -526,7 +527,6 @@ void Smb4KMounter::import( bool check_inaccessible )
       {
         // Do nothing
       }
-//       qDebug() << "Domain and ip address?";
 #endif
       share->setIsMounted( true );
       d->importedShares << share;
@@ -579,7 +579,7 @@ void Smb4KMounter::import( bool check_inaccessible )
     found = false;
   }
   
-#ifndef Q_OS_FREEBSD
+#if defined(Q_OS_LINUX)
   // Now stat the imported shares to get information about them.
   // Do not use Smb4KShare::canonicalPath() here, otherwise we might
   // get lock-ups with inaccessible shares.
@@ -682,8 +682,7 @@ void Smb4KMounter::mountShare( Smb4KShare *share, QWidget *parent )
   // with an error message.
   if ( !share->url().isValid() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->invalidURLPassed();
+    Smb4KNotification::invalidURLPassed();
     return;
   }
   else
@@ -906,8 +905,7 @@ void Smb4KMounter::mountShares( const QList<Smb4KShare *> &shares, QWidget *pare
     // with an error message.
     if ( !share->url().isValid() )
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->invalidURLPassed();
+      Smb4KNotification::invalidURLPassed();
       continue;
     }
     else
@@ -1036,8 +1034,7 @@ void Smb4KMounter::unmountShare( Smb4KShare *share, bool silent, QWidget *parent
   // with an error message.
   if ( !share->url().isValid() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->invalidURLPassed();
+    Smb4KNotification::invalidURLPassed();
     return;
   }
   else
@@ -1071,8 +1068,7 @@ void Smb4KMounter::unmountShare( Smb4KShare *share, bool silent, QWidget *parent
     {
       if ( !silent )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->unmountingNotAllowed( share );
+        Smb4KNotification::unmountingNotAllowed(share);
       }
       else
       {
@@ -1114,7 +1110,7 @@ void Smb4KMounter::unmountShare( Smb4KShare *share, bool silent, QWidget *parent
   // inaccessible share (Linux only)
   bool force = false;
 
-#ifdef Q_OS_LINUX
+#if defined(Q_OS_LINUX)
   if ( share->isInaccessible() )
   {
     force = Smb4KSettings::forceUnmountInaccessible();
@@ -1128,7 +1124,7 @@ void Smb4KMounter::unmountShare( Smb4KShare *share, bool silent, QWidget *parent
   // Create a new job and add it to the subjobs
   Smb4KUnmountJob *job = new Smb4KUnmountJob( this );
   job->setObjectName( QString( "UnmountJob_%1" ).arg( share->canonicalPath() ) );
-  job->setupUnmount( share, force, silent, (d->aboutToQuit || d->hardwareReason), parent );
+  job->setupUnmount( share, force, silent, d->internalReason, parent );
 
   connect( job, SIGNAL(result(KJob*)), SLOT(slotJobFinished(KJob*)) );
   connect( job, SIGNAL(aboutToStart(QList<Smb4KShare*>)), SLOT(slotAboutToStartUnmounting(QList<Smb4KShare*>)) );
@@ -1198,8 +1194,7 @@ void Smb4KMounter::unmountShares( const QList<Smb4KShare *> &shares, bool silent
         {
           if ( !silent )
           {
-            Smb4KNotification *notification = new Smb4KNotification();
-            notification->unmountingNotAllowed( share );
+            Smb4KNotification::unmountingNotAllowed(share);
           }
           else
           {
@@ -1226,7 +1221,7 @@ void Smb4KMounter::unmountShares( const QList<Smb4KShare *> &shares, bool silent
           }
           else
           {
-            if ( d->aboutToQuit )
+            if (d->internalReason)
             {
               continue;
             }
@@ -1254,7 +1249,7 @@ void Smb4KMounter::unmountShares( const QList<Smb4KShare *> &shares, bool silent
   // inaccessible share
   bool force = false;
 
-#ifdef Q_OS_LINUX
+#if defined(Q_OS_LINUX)
   if ( have_inaccessible_shares )
   {
     force = Smb4KSettings::forceUnmountInaccessible();
@@ -1268,7 +1263,7 @@ void Smb4KMounter::unmountShares( const QList<Smb4KShare *> &shares, bool silent
   // Create a new job and add it to the subjobs
   Smb4KUnmountJob *job = new Smb4KUnmountJob( this );
   job->setObjectName( QString( "UnmountJob_bulk-%1" ).arg( shares.size() ) );
-  job->setupUnmount( shares_to_unmount, force, silent, (d->aboutToQuit || d->hardwareReason), parent );
+  job->setupUnmount( shares_to_unmount, force, silent, d->internalReason, parent );
     
   connect( job, SIGNAL(result(KJob*)), SLOT(slotJobFinished(KJob*)) );
   connect( job, SIGNAL(aboutToStart(QList<Smb4KShare*>)), SLOT(slotAboutToStartUnmounting(QList<Smb4KShare*>)) );
@@ -1290,15 +1285,15 @@ void Smb4KMounter::unmountShares( const QList<Smb4KShare *> &shares, bool silent
 }
 
 
-void Smb4KMounter::unmountAllShares( QWidget *parent )
+void Smb4KMounter::unmountAllShares(QWidget* parent)
 {
-  if ( !d->aboutToQuit )
+  if (!d->internalReason)
   {
-    unmountShares( mountedSharesList(), false, parent );
+    unmountShares(mountedSharesList(), false, parent);
   }
   else
   {
-    unmountShares( mountedSharesList(), true, parent );
+    unmountShares(mountedSharesList(), true, parent);
   }
 }
 
@@ -1356,79 +1351,30 @@ void Smb4KMounter::check( Smb4KShare *share )
 
 void Smb4KMounter::saveSharesForRemount()
 {
-  if ( (Smb4KSettings::remountShares() && d->aboutToQuit) || d->hardwareReason )
+  // Save the shares for remount.
+  for (int i = 0; i < mountedSharesList().size(); ++i)
   {
-    // Save currently mounted shares.
-    for ( int i = 0; i < mountedSharesList().size(); ++i )
+    if (!mountedSharesList().at(i)->isForeign())
     {
-      if ( !mountedSharesList().at( i )->isForeign() )
-      {
-        Smb4KCustomOptionsManager::self()->addRemount( mountedSharesList().at( i ) );
-      }
-      else
-      {
-        Smb4KCustomOptionsManager::self()->removeRemount( mountedSharesList().at( i ) );
-      }
-    }
-
-    // Save failed remounts.
-    for ( int i = 0; i < d->remounts.size(); ++i )
-    {
-      Smb4KCustomOptionsManager::self()->addRemount( d->remounts.at( i ) );
-    }
-  }
-  else
-  {
-    if ( !Smb4KSettings::remountShares() )
-    {
-      Smb4KCustomOptionsManager::self()->clearRemounts();
+      Smb4KCustomOptionsManager::self()->addRemount(mountedSharesList().at(i), false);
     }
     else
     {
-      // Do nothing
+      Smb4KCustomOptionsManager::self()->removeRemount(mountedSharesList().at(i),false);
     }
   }
-
-  while ( !d->remounts.isEmpty() )
+  
+  // Also save the failed remounts.
+  for (int i = 0; i < d->remounts.size(); ++i)
+  {
+    Smb4KCustomOptionsManager::self()->addRemount(d->remounts.at(i),false);
+  }
+  
+  // Clear the failed remounts list.
+  while (!d->remounts.isEmpty())
   {
     delete d->remounts.takeFirst();
-  }
-}
-
-
-void Smb4KMounter::cleanup()
-{
-  if ( !d->obsoleteMountpoints.isEmpty() )
-  {
-    while ( !d->obsoleteMountpoints.isEmpty() )
-    {
-      QString path = d->obsoleteMountpoints.takeFirst();
-      
-      if ( path.startsWith( Smb4KSettings::mountPrefix().path() ) )
-      {
-        QDir dir( path );
-
-        if ( dir.rmdir( dir.canonicalPath() ) )
-        {
-          dir.cdUp();
-          dir.rmdir( dir.canonicalPath() );
-        }
-        else
-        {
-          // Do nothing
-        }
-      }
-      else
-      {
-        // Do nothing here. Do not remove any paths that are outside the
-        // mount prefix.
-      }
-    }
-  }
-  else
-  {
-    // Do nothing
-  }
+  }  
 }
 
 
@@ -1436,7 +1382,7 @@ void Smb4KMounter::timerEvent( QTimerEvent * )
 {
   // Import newly mounted shares. This is only done when imports
   // are allowed (no auth error occurred and a password dialog is
-  // op at the moment) and there are currently no subjobs.
+  // open at the moment) and there are currently no subjobs.
   if ( d->importTimeout >= Smb4KSettings::checkInterval() && d->importedShares.isEmpty() )
   {
     if ( d->importsAllowed && !hasSubjobs() )
@@ -1470,7 +1416,8 @@ void Smb4KMounter::timerEvent( QTimerEvent * )
   // before. Do this only if there are no subjobs, because we
   // do not want to get crashes because a share was invalidated
   // during processing the shares.
-  if ( Smb4KSettings::remountShares() && Smb4KSettings::remountAttempts() > d->remountAttempts )
+  if ( (Smb4KSettings::remountShares() || !Smb4KCustomOptionsManager::self()->sharesToRemount().isEmpty()) && 
+       Smb4KSettings::remountAttempts() > d->remountAttempts )
   {
     // Inhibit automatic sleeping.
     int cookie = Smb4KSolidInterface::self()->beginSleepSuppression(i18n("Remounting shares. Please wait."));
@@ -1528,9 +1475,6 @@ void Smb4KMounter::timerEvent( QTimerEvent * )
   {
     // Do nothing
   }
-  
-  // Finally, clean up the mount prefix.
-  cleanup();
 }
 
 
@@ -1542,41 +1486,39 @@ void Smb4KMounter::timerEvent( QTimerEvent * )
 void Smb4KMounter::slotStartJobs()
 {
   import( true );
-
-  if ( Smb4KSolidInterface::self()->networkStatus() == Smb4KSolidInterface::Connected ||
-       Smb4KSolidInterface::self()->networkStatus() == Smb4KSolidInterface::UnknownStatus )
-  {
-    d->hardwareReason = false;
-  }
-  else
-  {
-    // Do nothing and wait until the network becomes available.
-  }
-
-  startTimer( TIMEOUT );
+  d->timerId = startTimer( TIMEOUT );
 }
 
 
 void Smb4KMounter::slotAboutToQuit()
 {
-  // Tell the application it is about to quit.
-  d->aboutToQuit = true;
-
+  // Due to an internal reason some special things should
+  // be done, like unmounting synchronously.
+  d->internalReason = true;
+  
   // Abort any actions.
   abortAll();
 
-  // Save the shares that need to be remounted.
-  saveSharesForRemount();
+  // Check if the user wants to remount shares and save the
+  // shares for remount if so.
+  if (Smb4KSettings::remountShares())
+  {
+    saveSharesForRemount();
+  }
+  else
+  {
+    // Do nothing
+  }
 
   // Unmount the shares if the user chose to do so.
-  if ( Smb4KSettings::unmountSharesOnExit() )
+  if (Smb4KSettings::unmountSharesOnExit())
   {
     unmountAllShares();
     
     // Wait until done.
-    while ( hasSubjobs() )
+    while (hasSubjobs())
     {
-      QTest::qWait( TIMEOUT );
+      QTest::qWait(TIMEOUT);
     }
   }
   else
@@ -1586,43 +1528,33 @@ void Smb4KMounter::slotAboutToQuit()
 
   // Clean up the mount prefix.
   QDir dir;
-  dir.cd( Smb4KSettings::mountPrefix().path() );
-  QStringList dirs = dir.entryList( QDir::Dirs|QDir::NoDotAndDotDot, QDir::NoSort );
+  QStringList mountpoints;
+  dir.cd(Smb4KSettings::mountPrefix().path());
+  QStringList dirs = dir.entryList(QDir::Dirs|QDir::NoDotAndDotDot, QDir::NoSort);
 
   QList<Smb4KShare *> inaccessible = findInaccessibleShares();
 
   // Remove all directories from the list that belong to
   // inaccessible shares.
-  for ( int i = 0; i < inaccessible.size(); ++i )
+  for (int i = 0; i < inaccessible.size(); ++i)
   {
-    int index = dirs.indexOf( inaccessible.at( i )->hostName(), 0 );
+    int index = dirs.indexOf(inaccessible.at(i)->hostName(), 0);
 
-    if ( index != -1 )
+    if (index != -1)
     {
-      dirs.removeAt( index );
+      dirs.removeAt(index);
       continue;
     }
     else
     {
+      dir.cd(dirs.at(i));
+      mountpoints += dir.entryList(QDir::Dirs|QDir::NoDotAndDotDot, QDir::NoSort);
       continue;
     }
   }
-
-  // Now it is save to remove all empty directories.
-  for ( int i = 0; i < dirs.size(); ++i )
-  {
-    dir.cd( dirs.at( i ) );
-
-    QStringList subdirs = dir.entryList( QDir::Dirs|QDir::NoDotAndDotDot, QDir::NoSort );
-
-    for ( int k = 0; k < subdirs.size(); ++k )
-    {
-      dir.rmdir( subdirs.at( k ) );
-    }
-
-    dir.cdUp();
-    dir.rmdir( dirs.at( i ) );
-  }
+  
+  // Unset internal reason
+  d->internalReason = false;
 }
 
 
@@ -1687,55 +1619,62 @@ void Smb4KMounter::slotRetryMounting( Smb4KMountJob *job )
 }
 
 
-void Smb4KMounter::slotShareMounted( Smb4KShare *share )
+void Smb4KMounter::slotShareMounted(Smb4KShare *share)
 {
-  Q_ASSERT( share );
-
-  // Remove the share from the list of shares that are to be remounted.
-  QMutableListIterator<Smb4KShare *> s( d->remounts );
-
-  while ( s.hasNext() )
-  {
-    Smb4KShare *remount = s.next();
-
-    if ( !share->isForeign() && QString::compare( remount->unc(), share->unc(), Qt::CaseInsensitive ) == 0 )
-    {
-      s.remove();
-      break;
-    }
-    else
-    {
-      continue;
-    }
-  }
+  Q_ASSERT(share);
   
-  // Check that the share has not already been entered into the list.
-  Smb4KShare *known_share = findShareByPath( share->canonicalPath() );
-
-  if ( !known_share )
+  if (share)
   {
-    // Copy incoming share, because it will be deleted shortly
-    // after Smb4KMountJob::mounted() signal was emitted.
-    known_share = new Smb4KShare( *share );
+    // Remove the share from the list of shares that are to be remounted.
+    QMutableListIterator<Smb4KShare *> s(d->remounts);
 
-    // Check the usage, etc.
-    check( known_share );
-
-    // Add the share
-    addMountedShare( known_share );
-
-    if ( Smb4KSettings::remountShares() )
+    while (s.hasNext())
     {
-      Smb4KCustomOptionsManager::self()->removeRemount( known_share );
+      Smb4KShare *remount = s.next();
+
+      if (!share->isForeign() && QString::compare( remount->unc(), share->unc(), Qt::CaseInsensitive ) == 0)
+      {
+        s.remove();
+        break;
+      }
+      else
+      {
+        continue;
+      }
+    }
+    
+    // Check that the share has not already been entered into the list.
+    Smb4KShare *known_share = findShareByPath(share->canonicalPath());
+
+    if (!known_share)
+    {
+      // Copy incoming share, because it will be deleted shortly
+      // after Smb4KMountJob::mounted() signal was emitted.
+      known_share = new Smb4KShare(*share);
+
+      // Check the usage, etc.
+      check(known_share);
+
+      // Add the share
+      addMountedShare(known_share);
+
+      if (Smb4KSettings::remountShares())
+      {
+        Smb4KCustomOptionsManager::self()->removeRemount(known_share, false);
+      }
+      else
+      {
+        // Do nothing
+      }
+
+      // Emit the mounted() and mountedSharesListChanged() signals.
+      emit mounted(known_share);
+      emit mountedSharesListChanged();
     }
     else
     {
       // Do nothing
     }
-
-    // Emit the mounted() signal.
-    emit mounted( known_share );
-    emit mountedSharesListChanged();
   }
   else
   {
@@ -1744,30 +1683,34 @@ void Smb4KMounter::slotShareMounted( Smb4KShare *share )
 }
 
 
-void Smb4KMounter::slotShareUnmounted( Smb4KShare *share )
+void Smb4KMounter::slotShareUnmounted(Smb4KShare *share)
 {
-  Q_ASSERT( share );
+  Q_ASSERT(share);
   
-  // Get the share that was unmounted.
-  Smb4KShare *known_share = findShareByPath( share->canonicalPath() );
-
-  if ( known_share )
+  if (share)
   {
-    // Set the share as unmounted. Since the unmount job
-    // works with an internal copy, we have to set this here!
-    known_share->setIsMounted( false );
+    // Get the share that was unmounted.
+    Smb4KShare *known_share = findShareByPath(share->canonicalPath());
 
-    // Emit the unmounted() signal. We do it here, because if we do it
-    // after the mount prefix was cleaned up, Smb4KShare::canonicalPath()
-    // would return an empty string.
-    emit unmounted( known_share );
+    if (known_share)
+    {
+      // Set the share as unmounted. Since the unmount job
+      // works with an internal copy, we have to set this here!
+      known_share->setIsMounted(false);
 
-    // Schedule the obsolete mountpoint for removal.
-    d->obsoleteMountpoints << known_share->canonicalPath();
+      // Emit the unmounted() signal.
+      emit unmounted(known_share);
 
-    // Remove the share from the list of mounted shares.
-    removeMountedShare( known_share );
-    emit mountedSharesListChanged();
+      // Remove the share from the list of mounted shares.
+      removeMountedShare(known_share);
+      
+      // Emit the mountedSharesListChanged() signal.
+      emit mountedSharesListChanged();
+    }
+    else
+    {
+      // Do nothing
+    }
   }
   else
   {
@@ -1776,118 +1719,49 @@ void Smb4KMounter::slotShareUnmounted( Smb4KShare *share )
 }
 
 
-void Smb4KMounter::slotHardwareButtonPressed( Smb4KSolidInterface::ButtonType type )
+void Smb4KMounter::slotNetworkStatusChanged(Smb4KSolidInterface::ConnectionStatus status)
 {
   int cookie = Smb4KSolidInterface::self()->beginSleepSuppression(i18n("Unmounting shares. Please wait."));
-  d->hardwareReason = true;
   
-  switch ( type )
-  {
-    case Smb4KSolidInterface::SleepButton:
-    {
-      if (Smb4KSettings::unmountWhenSleepButtonPressed() && !mountedSharesList().isEmpty())
-      {
-        abortAll();
-        saveSharesForRemount();
-        unmountAllShares();
-
-        // Wait until done
-        while ( hasSubjobs() )
-        {
-          QTest::qWait( TIMEOUT );
-        }
-      }
-      else
-      {
-        // Do nothing
-      }
-      break;
-    }
-    case Smb4KSolidInterface::LidButton:
-    {
-      if (Smb4KSettings::unmountWhenLidButtonPressed() && !mountedSharesList().isEmpty())
-      {
-        abortAll();
-        saveSharesForRemount();
-        unmountAllShares();
-
-        // Wait until done
-        while ( hasSubjobs() )
-        {
-          QTest::qWait( TIMEOUT );
-        }
-      }
-      else
-      {
-        // Do nothing
-      }
-      break;
-    }
-    case Smb4KSolidInterface::PowerButton:
-    {
-      if (Smb4KSettings::unmountWhenPowerButtonPressed() && !mountedSharesList().isEmpty())
-      {
-        abortAll();
-        saveSharesForRemount();
-        unmountAllShares();
-
-        // Wait until done
-        while ( hasSubjobs() )
-        {
-          QTest::qWait( TIMEOUT );
-        }
-      }
-      else
-      {
-        // Do nothing
-      }
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
+  // Due to an internal reason some special things should
+  // be done, like unmounting synchronously.
+  d->internalReason = true;
   
-  d->hardwareReason = false;
-  Smb4KSolidInterface::self()->endSleepSuppression(cookie);
-  qDebug() << "Good night!";
-}
-
-
-void Smb4KMounter::slotComputerWokeUp()
-{
-}
-
-
-void Smb4KMounter::slotNetworkStatusChanged( Smb4KSolidInterface::ConnectionStatus status )
-{
-  switch ( status )
+  switch (status)
   {
+    case Smb4KSolidInterface::Disconnecting:
     case Smb4KSolidInterface::Disconnected:
     {
-      d->hardwareReason = true;
       abortAll();
+      // Save the shares for automatic remounting.
       saveSharesForRemount();
       unmountAllShares();
 
       // Wait until done
-      while ( hasSubjobs() )
+      while (hasSubjobs())
       {
-        QTest::qWait( TIMEOUT );
+        QTest::qWait(TIMEOUT);
       }
-      
-      d->hardwareReason = false;
+      break;
+    }
+    case Smb4KSolidInterface::Connected:
+    {
+      // Remount shares after the network became available
+      // again.
+      triggerRemounts(true);
       break;
     }
     default:
     {
-      d->hardwareReason = true;
-      triggerRemounts( true );
-      d->hardwareReason = false;
       break;
     }
   }
+  
+  // Unset internal reason.
+  d->internalReason = false;
+  
+  // End the sleep suppression.
+  Smb4KSolidInterface::self()->endSleepSuppression(cookie);
 }
 
 
@@ -1925,13 +1799,11 @@ void Smb4KMounter::slotFinishedMounting( const QList<Smb4KShare *> &shares )
   {
     if ( shares.size() > 1 )
     {
-      Smb4KNotification *notification = new Smb4KNotification( this );
-      notification->sharesMounted( shares.size(), (shares.size() - failed_mounts) );
+      Smb4KNotification::sharesMounted(shares.size(), (shares.size() - failed_mounts));
     }
     else
     {
-      Smb4KNotification *notification = new Smb4KNotification( this );
-      notification->shareMounted( shares.first() );
+      Smb4KNotification::shareMounted(shares.first());
     }
   }
   else
@@ -1974,13 +1846,11 @@ void Smb4KMounter::slotFinishedUnmounting( const QList<Smb4KShare *> &shares )
   {
     if ( shares.size() > 1 )
     {
-      Smb4KNotification *notification = new Smb4KNotification( this );
-      notification->allSharesUnmounted( shares.size(), (shares.size() - failed_unmounts) );
+      Smb4KNotification::sharesUnmounted(shares.size(), (shares.size() - failed_unmounts));
     }
     else
     {
-      Smb4KNotification *notification = new Smb4KNotification( this );
-      notification->shareUnmounted( shares.first() );
+      Smb4KNotification::shareUnmounted(shares.first());
     }
   }
   else
@@ -2032,7 +1902,7 @@ void Smb4KMounter::slotStatResult( KJob *job )
 
     // Is this a mount that was done by the user or by
     // someone else (or the system)?
-    if ( (share->uid() == getuid() && share->gid() == getgid()) ||
+    if ( (share->uid() == KUser(KUser::UseRealUserID).uid() && share->gid() == KUserGroup(KUser::UseRealUserID).gid()) ||
          (share->path().startsWith( Smb4KSettings::mountPrefix().path() ) || share->path().startsWith( QDir::homePath() )) ||
          (share->canonicalPath().startsWith( QDir( Smb4KSettings::mountPrefix().path() ).canonicalPath() ) ||
          share->canonicalPath().startsWith( QDir::home().canonicalPath() )) )
@@ -2133,7 +2003,7 @@ void Smb4KMounter::slotStatResult( KJob *job )
       // Now remove the obsolete share entry from the global list
       // of shares and add the stat'ed one. Emit the appropriate
       // signal when done.
-      if ( !share->isForeign() || Smb4KSettings::showAllShares() )
+      if ( !share->isForeign() || Smb4KSettings::detectAllShares() )
       {
         // This share was previouly mounted.
         removeMountedShare( mounted_share );
@@ -2185,7 +2055,7 @@ void Smb4KMounter::slotStatResult( KJob *job )
 
       // Now add the stat'ed share to the global list of shares.
       // Emit the appropriate signal when done.
-      if ( !share->isForeign() || Smb4KSettings::showAllShares() )
+      if ( !share->isForeign() || Smb4KSettings::detectAllShares() )
       {
         // This is a new share.
         Smb4KShare *new_share = new Smb4KShare( *share );
@@ -2217,4 +2087,76 @@ void Smb4KMounter::slotStatResult( KJob *job )
 }
 
 
+void Smb4KMounter::slotActiveProfileChanged(const QString &newProfile)
+{
+  if (QString::compare(d->activeProfile, newProfile) != 0)
+  {
+    // Stop the timer.
+    killTimer(d->timerId);
+
+    // Check if the user wants to remount shares and save the
+    // shares for remount if so.
+    if (Smb4KSettings::remountShares())
+    {
+      saveSharesForRemount();
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    abortAll();
+    
+    // Clear all remounts.
+    while (!d->remounts.isEmpty())
+    {
+      delete d->remounts.takeFirst();
+    }
+    
+    // Clear all retries.
+    while (!d->retries.isEmpty())
+    {
+      delete d->retries.takeFirst();
+    }
+    
+    // Unmount all shares and wait until done.
+    unmountAllShares();
+    
+    while (hasSubjobs())
+    {
+      QTest::qWait(TIMEOUT);
+    }
+    
+    // Reset some variables.
+    d->importTimeout = 0;
+    d->remountTimeout = 0;
+    d->remountAttempts = 0;
+    d->checks = 0;
+    d->firstImportDone = false;
+    d->importsAllowed = true;
+    d->activeProfile = newProfile;
+    
+    // Start the timer again.
+    d->timerId = startTimer(TIMEOUT);
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
+
+void Smb4KMounter::slotProfileMigrated(const QString& from, const QString& to)
+{
+  if (QString::compare(from, d->activeProfile, Qt::CaseSensitive) == 0)
+  {
+    d->activeProfile = to;
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
+
 #include "smb4kmounter.moc"
