@@ -2,7 +2,7 @@
     smb4kmounter.h  -  The core class that mounts the shares.
                              -------------------
     begin                : Die Jun 10 2003
-    copyright            : (C) 2003-2013 by Alexander Reinholdt
+    copyright            : (C) 2003-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -298,22 +298,6 @@ class KDE_EXPORT Smb4KMounter : public KCompositeJob
      */
     void slotShareUnmounted( Smb4KShare *share );
     
-    /**
-     * This slot is called by the Solid interface when a hardware button was
-     * pressed. It is used to unmount shares according to the wishes of the
-     * user.
-     *
-     * @param type            The button type
-     */
-    void slotHardwareButtonPressed( Smb4KSolidInterface::ButtonType type );
-
-    /**
-     * This slot is called by the Solid interface when the computer woke up
-     * from a sleep or stand-by state. It initializes the remounting of the
-     * previously unmounted shares.
-     */
-    void slotComputerWokeUp();
-
     /**
      * This slot is called by the Solid interface when the network status
      * changed. It is used to initialize network actions when the network
@@ -321,7 +305,7 @@ class KDE_EXPORT Smb4KMounter : public KCompositeJob
      *
      * @param status          The new network status
      */
-    void slotNetworkStatusChanged( Smb4KSolidInterface::ConnectionStatus status );
+    void slotNetworkStatusChanged(Smb4KSolidInterface::ConnectionStatus status);
 
     /**
      * Called when a mount job started. It just emits the 
@@ -365,7 +349,20 @@ class KDE_EXPORT Smb4KMounter : public KCompositeJob
      * @param job             The KIO::StatJob
      */
     void slotStatResult( KJob *job );
-
+    
+    /**
+     * This slot is called when the active profile changed.
+     */
+    void slotActiveProfileChanged(const QString &newProfile);
+    
+    /**
+     * This slot is called when a profile was migrated.
+     * 
+     * @param from            The old profile
+     * @param to              The new profile
+     */
+    void slotProfileMigrated(const QString &from, const QString &to);
+    
   private:
     /**
      * Trigger the remounting of shares. If the parameter @p fill_list is
@@ -400,11 +397,6 @@ class KDE_EXPORT Smb4KMounter : public KCompositeJob
      */
     void saveSharesForRemount();
     
-    /**
-     * Clean up the mount prefix.
-     */
-    void cleanup();
-
     /**
      * Pointer to the Smb4KMounterPrivate class.
      */
