@@ -3,7 +3,7 @@
     Smb4KMounter class.
                              -------------------
     begin                : Do Jul 19 2007
-    copyright            : (C) 2007-2012 by Alexander Reinholdt
+    copyright            : (C) 2007-2015 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -34,6 +34,7 @@
 // Qt includes
 #include <QtCore/QThread>
 #include <QtCore/QString>
+#include <QtCore/QVariant>
 #include <QtGui/QCheckBox>
 
 // KDE includes
@@ -167,8 +168,8 @@ class Smb4KMountJob : public KJob
     QList<Smb4KShare *> m_auth_errors;
     QList<Smb4KShare *> m_retries;
     QWidget *m_parent_widget;
-    bool createMountAction( Smb4KShare *share,
-                            Action *action );
+    bool createMountAction(Smb4KShare *share, Action *action);
+    bool fillArgs(Smb4KShare *share, QMap<QString, QVariant> &mountArgs);
     int m_processed;
 };
 
@@ -398,15 +399,15 @@ class Smb4KMounterPrivate
     int remountTimeout;
     int remountAttempts;
     int checks;
+    int timerId;
     Smb4KMountDialog *dialog;
     QList<Smb4KShare *> importedShares;
     QList<Smb4KShare *> retries;
     QList<Smb4KShare *> remounts;
-    QStringList obsoleteMountpoints;
-    bool aboutToQuit;
-    bool hardwareReason;
     bool firstImportDone;
     bool importsAllowed;
+    bool internalReason;
+    QString activeProfile;
 };
 
 
