@@ -2,7 +2,7 @@
     smb4kpreviewer_p  -  Private helper classes for Smb4KPreviewer class.
                              -------------------
     begin                : So Dez 21 2008
-    copyright            : (C) 2008-2013 by Alexander Reinholdt
+    copyright            : (C) 2008-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -51,10 +51,102 @@
 #include <ktoolbar.h>
 #include <kstatusbar.h>
 #include <kshell.h>
+#include <kguiitem.h>
 
 using namespace Smb4KGlobal;
 
 
+Smb4KPreviewFileItem::Smb4KPreviewFileItem()
+{
+  m_name = QString();
+}
+
+
+Smb4KPreviewFileItem::~Smb4KPreviewFileItem()
+{
+}
+
+
+void Smb4KPreviewFileItem::setItemName(const QString& name)
+{
+  m_name = name;
+}
+
+
+QString Smb4KPreviewFileItem::itemName() const
+{
+  return m_name;
+}
+
+
+void Smb4KPreviewFileItem::setDate(const QString& date)
+{
+  m_date = date;
+}
+
+
+QString Smb4KPreviewFileItem::date() const
+{
+  return m_date;
+}
+
+
+void Smb4KPreviewFileItem::setItemSize(const QString& size)
+{
+  m_size = size;
+}
+
+
+QString Smb4KPreviewFileItem::itemSize() const
+{
+  return m_size;
+}
+
+
+void Smb4KPreviewFileItem::setDir(bool dir)
+{
+  m_dir = dir;
+}
+
+
+bool Smb4KPreviewFileItem::isDir() const
+{
+  return m_dir;
+}
+
+
+bool Smb4KPreviewFileItem::isFile() const
+{
+  return !m_dir;
+}
+
+
+bool Smb4KPreviewFileItem::isHidden() const
+{
+  return (m_name.startsWith( '.' ) && QString::compare( m_name, "." ) != 0 && QString::compare( m_name, ".." ) != 0);
+}
+
+
+KIcon Smb4KPreviewFileItem::itemIcon() const
+{
+  KIcon icon;
+  
+  if ( m_dir )
+  {
+    icon = KIcon( "folder" );
+  }
+  else
+  {
+    KUrl url( m_name );
+    icon = KIcon( KMimeType::iconNameForUrl( url, 0 ) );
+  }
+  
+  return icon;
+}
+
+
+
+
 Smb4KPreviewJob::Smb4KPreviewJob( QObject *parent ) : KJob( parent ),
   m_started( false ), m_share( NULL ), m_parent_widget( NULL ), m_proc( NULL )
 {
@@ -104,8 +196,7 @@ void Smb4KPreviewJob::slotStartPreview()
 
   if ( smbclient.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "smbclient" );
+    Smb4KNotification::commandNotFound("smbclient");
     emitResult();
     return;
   }
@@ -341,7 +432,7 @@ void Smb4KPreviewJob::slotStartPreview()
   m_proc->setOutputChannelMode( KProcess::SeparateChannels );
   m_proc->setEnv( "PASSWD", m_share->password(), true );
   m_proc->setShellCommand( arguments.join( " " ) );
-
+  
   connect( m_proc, SIGNAL(readyReadStandardOutput()), SLOT(slotReadStandardOutput()) );
   connect( m_proc, SIGNAL(readyReadStandardError()),  SLOT(slotReadStandardError()) );
   connect( m_proc, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(slotProcessFinished(int,QProcess::ExitStatus)) );
@@ -352,10 +443,10 @@ void Smb4KPreviewJob::slotStartPreview()
 
 void Smb4KPreviewJob::slotReadStandardOutput()
 {
-  QString stdout = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 );
+  QString stdOut = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 );
   
-  if ( stdout.contains( "NT_STATUS_ACCESS_DENIED", Qt::CaseSensitive ) || 
-       stdout.contains( "NT_STATUS_LOGON_FAILURE", Qt::CaseSensitive ) )
+  if ( stdOut.contains( "NT_STATUS_ACCESS_DENIED", Qt::CaseSensitive ) || 
+       stdOut.contains( "NT_STATUS_LOGON_FAILURE", Qt::CaseSensitive ) )
   {
     // This might happen if a directory cannot be accessed due to missing
     // read permissions.
@@ -363,8 +454,8 @@ void Smb4KPreviewJob::slotReadStandardOutput()
   }
   else
   {
-    QStringList list = stdout.split( '\n', QString::SkipEmptyParts );
-    QList<Item> items;
+    QStringList list = stdOut.split( '\n', QString::SkipEmptyParts );
+    QList<Smb4KPreviewFileItem> items;
 
     foreach ( const QString &line, list )
     {
@@ -399,44 +490,42 @@ void Smb4KPreviewJob::slotReadStandardOutput()
         }
 
         QString date = QDateTime::fromString( right.section( QString( " %1 " ).arg( size ), 1, 1 ).trimmed() ).toString();
-
-        if ( !name.isEmpty() )
+        
+        if (!name.isEmpty())
         {
-          Item item;
-
-          if ( is_dir )
+          if (!name.startsWith('.'))
           {
-            if ( name.startsWith( '.' ) &&
-                 (QString::compare( name, "." ) != 0 && QString::compare( name, ".." ) != 0) )
+            Smb4KPreviewFileItem item;
+            item.setItemName(name);
+            item.setDir(is_dir);
+            item.setDate(date);
+            item.setItemSize(size);
+            items << item;
+          }
+          else if (name.startsWith('.') && QString::compare(name, ".") != 0 && QString::compare(name, "..") != 0)
+          {
+            if (Smb4KSettings::previewHiddenItems())
             {
-              item.first = HiddenDirectoryItem;
+              Smb4KPreviewFileItem item;
+              item.setItemName(name);
+              item.setDir(is_dir);
+              item.setDate(date);
+              item.setItemSize(size);
+              items << item;
             }
             else
             {
-              item.first = DirectoryItem;
+              // Do nothing
             }
           }
           else
           {
-            if ( name.startsWith( '.' ) )
-            {
-              item.first = HiddenFileItem;
-            }
-            else
-            {
-              item.first = FileItem;
-            }
+            // Do nothing
           }
-
-          item.second["name"] = name;
-          item.second["size"] = size;
-          item.second["date"] = date;
-
-          items << item;
         }
         else
         {
-          continue;
+          // Do nothing
         }
       }
     }
@@ -448,30 +537,49 @@ void Smb4KPreviewJob::slotReadStandardOutput()
 
 void Smb4KPreviewJob::slotReadStandardError()
 {
-  QString stderr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
+  QString stdErr = QString::fromUtf8(m_proc->readAllStandardError(), -1).trimmed();
 
-  // Remove DEBUG messages and the additional information
-  // that smbclient unfortunately reports to stderr.
-  QStringList err_msg = stderr.split( '\n', QString::SkipEmptyParts );
+  // Remove DEBUG messages, the additional information
+  // that smbclient unfortunately reports to stderr and
+  // error messages due to a missing smb.conf file.
+  QStringList err_msg = stdErr.split('\n', QString::SkipEmptyParts);
+  QMutableStringListIterator it(err_msg);
+  bool delete_next = false;
 
-  QMutableStringListIterator it( err_msg );
-
-  while ( it.hasNext() )
+  while (it.hasNext())
   {
     QString line = it.next();
 
-    if ( line.contains( "DEBUG" ) )
+    if (line.contains( "DEBUG" ))
     {
       it.remove();
     }
-    else if ( line.trimmed().startsWith( QLatin1String( "Domain" ) ) || line.trimmed().startsWith( QLatin1String( "OS" ) ) )
+    else if (line.trimmed().startsWith(QLatin1String("Domain")) || line.trimmed().startsWith(QLatin1String("OS")))
     {
       it.remove();
     }
-    else if ( line.trimmed().startsWith( QLatin1String( "Ignoring unknown parameter" ) ) )
+    else if (line.trimmed().startsWith(QLatin1String("Ignoring unknown parameter")))
     {
       it.remove();
     }
+    else if (line.contains(QLatin1String("smb.conf")) && line.contains(QLatin1String("Can't load")))
+    {
+      // smb.conf is missing.
+      // Output from smbclient (1 line)
+      it.remove();
+    }
+    else if (line.contains(QLatin1String("smb.conf")) && line.contains(QLatin1String("Unable to open configuration file")))
+    {
+      // smb.conf is missing.
+      // Output by param.c (2 lines)
+      it.remove();
+      delete_next = true;
+    }
+    else if (delete_next && line.contains(QLatin1String("No such file or directory")))
+    {
+      it.remove();
+      delete_next = false;
+    }
     else
     {
       // Do nothing
@@ -484,8 +592,8 @@ void Smb4KPreviewJob::slotReadStandardError()
   {
     m_proc->abort();
 
-    if ( stderr.contains( "NT_STATUS_LOGON_FAILURE" ) ||
-         stderr.contains( "NT_STATUS_ACCESS_DENIED" ) )
+    if ( stdErr.contains( "NT_STATUS_LOGON_FAILURE" ) ||
+         stdErr.contains( "NT_STATUS_ACCESS_DENIED" ) )
     {
       // Authentication error
       emit authError( this );
@@ -494,8 +602,7 @@ void Smb4KPreviewJob::slotReadStandardError()
     {
       if ( !err_msg.isEmpty() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->retrievingPreviewFailed( m_share, err_msg.join( "\n" ) );
+        Smb4KNotification::retrievingPreviewFailed(m_share, err_msg.join( "\n" ));
       }
       else
       {
@@ -518,8 +625,7 @@ void Smb4KPreviewJob::slotProcessFinished( int /*exitCode*/, QProcess::ExitStatu
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );;
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -553,7 +659,7 @@ Smb4KPreviewDialog::Smb4KPreviewDialog( Smb4KShare *share, QWidget *parent )
   
   setAttribute( Qt::WA_DeleteOnClose, true );
 
-  setCaption( i18n( "Preview" ) );
+  setCaption( i18n( "Preview of %1" ).arg( share->unc() ) );
   setButtons( Close );
   setDefaultButton( Close );
 
@@ -570,8 +676,8 @@ Smb4KPreviewDialog::Smb4KPreviewDialog( Smb4KShare *share, QWidget *parent )
 
   setupView();
 
-  connect( this,                   SIGNAL(closeClicked()),
-           this,                   SLOT(slotCloseClicked()) );
+  connect( this, SIGNAL(closeClicked()),
+           this, SLOT(slotCloseClicked()) );
 
   setMinimumWidth( sizeHint().width() > 350 ? sizeHint().width() : 350 );
 
@@ -606,12 +712,15 @@ void Smb4KPreviewDialog::setupView()
   m_view->setIconSize( QSize( icon_size, icon_size ) );
 
   KToolBar *toolbar = new KToolBar( main_widget, true, false );
-
-  m_reload  = new KAction( KIcon( "view-refresh" ), i18n( "Reload" ), toolbar );
-  m_reload->setEnabled( false );
   
-  m_abort   = new KAction( KIcon( "process-stop" ), i18n( "Abort" ), toolbar );
-  m_abort->setEnabled( false );
+  m_reload_abort = new KDualAction(toolbar);
+  KGuiItem reload_item(i18n( "Reload" ), KIcon( "view-refresh" ));
+  KGuiItem abort_item(i18n( "Abort" ), KIcon( "process-stop" ));
+  m_reload_abort->setActiveGuiItem(reload_item);
+  m_reload_abort->setInactiveGuiItem(abort_item);
+  m_reload_abort->setActive(true);
+  m_reload_abort->setAutoToggle(false);
+//   m_reload_abort->setEnabled(false);
   
   m_back    = new KAction( KIcon( "go-previous" ), i18n( "Back" ), toolbar );
   m_back->setEnabled( false );
@@ -629,8 +738,7 @@ void Smb4KPreviewDialog::setupView()
     "the previously visited locations from the drop-down menu that will then be displayed in the "
     "view above." ) );
 
-  toolbar->addAction( m_reload );
-  toolbar->addAction( m_abort );
+  toolbar->addAction( m_reload_abort );
   toolbar->addAction( m_back );
   toolbar->addAction( m_forward );
   toolbar->addAction( m_up );
@@ -639,15 +747,18 @@ void Smb4KPreviewDialog::setupView()
   layout->addWidget( m_view, 0, 0, 0 );
   layout->addWidget( toolbar, 1, 0, 0 );
 
-  connect( toolbar, SIGNAL(actionTriggered(QAction*)),
-           this,      SLOT(slotActionTriggered(QAction*)) );
-
-  connect( m_combo,   SIGNAL(activated(QString)),
-           this,      SLOT(slotItemActivated(QString)) );
-
-  connect( m_view,    SIGNAL(executed(QListWidgetItem*)),
-           this,      SLOT(slotItemExecuted(QListWidgetItem*)) );
-
+  connect(m_reload_abort, SIGNAL(triggered(bool)), 
+          this,           SLOT(slotReloadAbortActionTriggered(bool)));
+  connect(m_back,         SIGNAL(triggered(bool)),
+          this,           SLOT(slotBackActionTriggered(bool)));
+  connect(m_forward,      SIGNAL(triggered(bool)),
+          this,           SLOT(slotForwardActionTriggered(bool)));
+  connect(m_up,           SIGNAL(triggered(bool)),
+          this,           SLOT(slotUpActionTriggered(bool)));
+  connect(m_combo,        SIGNAL(activated(QString)),
+          this,           SLOT(slotItemActivated(QString)) );
+  connect(m_view,         SIGNAL(executed(QListWidgetItem*)),
+          this,           SLOT(slotItemExecuted(QListWidgetItem*)) );
   connect( KGlobalSettings::self(), SIGNAL(iconChanged(int)),
            this,                    SLOT(slotIconSizeChanged(int)) );
 }
@@ -657,174 +768,139 @@ void Smb4KPreviewDialog::setupView()
 // SLOT IMPLEMENTATIONS
 /////////////////////////////////////////////////////////////////////////////
 
-void Smb4KPreviewDialog::slotActionTriggered( QAction *action )
+void Smb4KPreviewDialog::slotReloadAbortActionTriggered(bool /*t*/)
 {
-  KAction *kaction = static_cast<KAction *>( action );
-
-  if ( kaction )
+  if ( m_reload_abort->isActive() )
   {
-    if ( kaction == m_reload )
-    {
-      // Clear the history
-      m_history.clear();
+    // Clear the history
+    m_history.clear();
       
-     // Request the preview
-      slotRequestPreview();
-    }
-    else if ( kaction == m_abort )
-    {
-      // Emit signal to kill the preview job.
-      emit abortPreview( m_share );
-    }
-    else if ( kaction == m_back )
-    {
-      // Get the current history if necessary,
-      // shift one item back and request a preview.
-      if ( m_history.isEmpty() )
-      {
-        m_history = m_combo->historyItems();
-        m_iterator = QStringListIterator( m_history );
-      }
-      else
-      {
-        // Do nothing
-      }
+    // Request the preview
+    slotRequestPreview();
+  }
+  else
+  {
+    // Emit signal to kill the preview job.
+    emit abortPreview( m_share );
+  }
+}
 
+
+void Smb4KPreviewDialog::slotBackActionTriggered(bool /*t*/)
+{
+  // Get the current history if necessary,
+  // shift one item back and request a preview.
+  if ( m_history.isEmpty() )
+  {
+    m_history = m_combo->historyItems();
+    m_iterator = QStringListIterator( m_history );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  if ( m_iterator.hasNext() )
+  {
+    // Jump behind the current item.
+    QString location = m_iterator.next();
+
+    if ( QString::compare( location, m_combo->currentText(), Qt::CaseInsensitive ) == 0 )
+    {
       if ( m_iterator.hasNext() )
       {
-        // Jump behind the current item.
-        QString location = m_iterator.next();
+        location = m_iterator.next();
+        QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
 
-        if ( QString::compare( location, m_combo->currentText(), Qt::CaseInsensitive ) == 0 )
+        if ( !path.isEmpty() )
         {
-          if ( m_iterator.hasNext() )
-          {
-            location = m_iterator.next();
-            QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
-
-            if ( !path.isEmpty() )
-            {
-              m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
-            }
-            else
-            {
-              m_url.setPath( m_share->shareName() );
-            }
-
-            // Request the preview.
-            slotRequestPreview();
-          }
-          else
-          {
-            m_back->setEnabled( false );
-          }
+          m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
         }
         else
         {
-          QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
-
-          if ( !path.isEmpty() )
-          {
-            m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
-          }
-          else
-          {
-            m_url.setPath( m_share->shareName() );
-          }
-
-          // Request the preview.
-          slotRequestPreview();
+          m_url.setPath( m_share->shareName() );
         }
+
+        // Request the preview.
+        slotRequestPreview();
       }
       else
       {
-        // Do nothing
+        m_back->setEnabled( false );
       }
     }
-    else if ( kaction == m_forward )
+    else
     {
-      // Shift one item forward an request a preview.
-      if ( !m_history.isEmpty() && m_iterator.hasPrevious() )
+      QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
+
+      if ( !path.isEmpty() )
       {
-        // Jump in front of the current item
-        QString location = m_iterator.previous();
+        m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
+      }
+      else
+      {
+        m_url.setPath( m_share->shareName() );
+      }
 
-        if ( QString::compare( location, m_combo->currentText(), Qt::CaseInsensitive ) == 0 )
+      // Request the preview.
+      slotRequestPreview();
+    }
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
+
+void Smb4KPreviewDialog::slotForwardActionTriggered(bool /*t*/)
+{
+  // Shift one item forward an request a preview.
+  if ( !m_history.isEmpty() && m_iterator.hasPrevious() )
+  {
+    // Jump in front of the current item
+    QString location = m_iterator.previous();
+
+    if ( QString::compare( location, m_combo->currentText(), Qt::CaseInsensitive ) == 0 )
+    {
+      // Now get the next location.
+      if ( m_iterator.hasPrevious() )
+      {
+        location = m_iterator.previous();
+        QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
+
+        if ( !path.isEmpty() )
         {
-          // Now get the next location.
-          if ( m_iterator.hasPrevious() )
-          {
-            location = m_iterator.previous();
-            QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
-
-            if ( !path.isEmpty() )
-            {
-              m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
-            }
-            else
-            {
-              m_url.setPath( m_share->shareName() );
-            }
-
-            // Request the preview
-            slotRequestPreview();
-          }
-          else
-          {
-            m_forward->setEnabled( false );
-          }
+          m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
         }
         else
         {
-          QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
-
-          if ( !path.isEmpty() )
-          {
-            m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
-          }
-          else
-          {
-            m_url.setPath( m_share->shareName() );
-          }
-
-          // Request the preview
-          slotRequestPreview();
+          m_url.setPath( m_share->shareName() );
         }
-      }
-      else
-      {
-        // Do nothing
-      }
-    }
-    else if ( kaction == m_up )
-    {
-      QString test = QString("//%1%2%3")
-                     .arg(m_url.host())
-                     .arg(m_url.path().startsWith('/') ? "" : "/")
-                     .arg(m_url.path(KUrl::RemoveTrailingSlash));
-      
-      if ( QString::compare( m_share->unc(), test, Qt::CaseInsensitive ) != 0 )
-      {
-        // Clear the history
-        m_history.clear();
-
-        // Adjust the path
-        // FIXME: Use KUrl::upUrl() here when we have found to adjust 
-        // the path
-        QString path = m_url.path();
-        m_url.setPath( path.section( '/', 0, -2 ) );
 
         // Request the preview
         slotRequestPreview();
       }
       else
       {
-        m_up->setEnabled( false );
+        m_forward->setEnabled( false );
       }
     }
     else
     {
-      // Do nothing
+      QString path = location.remove( m_share->unc(), Qt::CaseInsensitive );
+
+      if ( !path.isEmpty() )
+      {
+        m_url.setPath( QString( "%1%2" ).arg( m_share->shareName() ).arg( path ) );
+      }
+      else
+      {
+        m_url.setPath( m_share->shareName() );
+      }
+
+      // Request the preview
+      slotRequestPreview();
     }
   }
   else
@@ -834,6 +910,34 @@ void Smb4KPreviewDialog::slotActionTriggered( QAction *action )
 }
 
 
+void Smb4KPreviewDialog::slotUpActionTriggered(bool /*t*/)
+{
+  QString test = QString("//%1%2%3")
+                 .arg(m_url.host())
+                 .arg(m_url.path().startsWith('/') ? "" : "/")
+                 .arg(m_url.path(KUrl::RemoveTrailingSlash));
+      
+  if ( QString::compare( m_share->unc(), test, Qt::CaseInsensitive ) != 0 )
+  {
+    // Clear the history
+    m_history.clear();
+
+    // Adjust the path
+    // FIXME: Use KUrl::upUrl() here when we have found to adjust 
+    // the path
+    QString path = m_url.path();
+    m_url.setPath( path.section( '/', 0, -2 ) );
+
+    // Request the preview
+    slotRequestPreview();
+  }
+  else
+  {
+    m_up->setEnabled( false );
+  }
+}
+
+
 void Smb4KPreviewDialog::slotRequestPreview()
 {
   // Set the current item
@@ -870,7 +974,7 @@ void Smb4KPreviewDialog::slotRequestPreview()
 }
 
 
-void Smb4KPreviewDialog::slotDisplayPreview( const KUrl &url, const QList<Item> &contents )
+void Smb4KPreviewDialog::slotDisplayPreview( const KUrl &url, const QList<Smb4KPreviewFileItem> &contents )
 {
   if ( m_url != url )
   {
@@ -884,65 +988,11 @@ void Smb4KPreviewDialog::slotDisplayPreview( const KUrl &url, const QList<Item>
   // Display the preview
   for ( int i = 0; i < contents.size(); ++i )
   {
-    switch ( contents.at( i ).first )
-    {
-      case HiddenDirectoryItem:
-      {
-        // Honor the user's setting about hidden items. And do not show the '.' and '..' directories.
-        if ( Smb4KSettings::previewHiddenItems() &&
-             QString::compare( contents.at( i ).second.value( "name" ), "." ) != 0 &&
-             QString::compare( contents.at( i ).second.value( "name" ), ".." ) != 0 )
-        {
-          QListWidgetItem *listItem = new QListWidgetItem( KIcon( "folder" ), contents.at( i ).second.value( "name" ), m_view, Directory );
-          listItem->setData( Qt::UserRole, contents.at( i ).second.value( "name" ) );
-        }
-        else
-        {
-          // Do nothing
-        }
-        break;
-      }
-      case DirectoryItem:
-      {
-        // Do not show the '.' and '..' directories.
-        if ( QString::compare( contents.at( i ).second.value( "name" ), "." ) != 0 &&
-             QString::compare( contents.at( i ).second.value( "name" ), ".." ) != 0 )
-        {
-          QListWidgetItem *listItem = new QListWidgetItem( KIcon( "folder" ), contents.at( i ).second.value( "name" ), m_view, Directory );
-          listItem->setData( Qt::UserRole, contents.at( i ).second.value( "name" ) );
-        }
-        else
-        {
-          // Do nothing
-        }
-        break;
-      }
-      case HiddenFileItem:
-      {
-        if ( Smb4KSettings::previewHiddenItems() )
-        {
-          KUrl url( contents.at( i ).second.value( "name" ) );
-          QListWidgetItem *listItem = new QListWidgetItem( KIcon( KMimeType::iconNameForUrl( url, 0 ) ), contents.at( i ).second.value( "name" ), m_view, File );
-          listItem->setData( Qt::UserRole, contents.at( i ).second.value( "name" ) );
-        }
-        else
-        {
-          // Do nothing
-        }
-        break;
-      }
-      case FileItem:
-      {
-        KUrl url( contents.at( i ).second.value( "name" ) );
-        QListWidgetItem *listItem = new QListWidgetItem( KIcon( KMimeType::iconNameForUrl( url, 0 ) ), contents.at( i ).second.value( "name" ), m_view, File );
-        listItem->setData( Qt::UserRole, contents.at( i ).second.value( "name" ) );
-        break;
-      }
-      default:
-      {
-        break;
-      }
-    }
+    QListWidgetItem *listItem = new QListWidgetItem( contents.at(i).itemIcon(), 
+                                                     contents.at(i).itemName(), 
+                                                     m_view,
+                                                     (contents.at(i).isDir() ? Directory : File) );
+    listItem->setData( Qt::UserRole, contents.at(i).itemName() );
   }
 
   // Enable/disable the back action.
@@ -968,8 +1018,7 @@ void Smb4KPreviewDialog::slotAboutToStart( Smb4KShare *share, const KUrl &url )
 {
   if ( share == m_share && url == m_url )
   {
-    m_reload->setEnabled( false );
-    m_abort->setEnabled( true );
+    m_reload_abort->setActive(false);
   }
   else
   {
@@ -982,8 +1031,7 @@ void Smb4KPreviewDialog::slotFinished( Smb4KShare *share, const KUrl &url )
 {
   if ( share == m_share && url == m_url )
   {
-    m_reload->setEnabled( true );
-    m_abort->setEnabled( false );
+    m_reload_abort->setActive(true);
   }
   else
   {
