@@ -130,7 +130,7 @@ bool Smb4KScanner::isRunning( Smb4KGlobal::Process process, Smb4KBasicNetworkIte
     }
     case LookupDomainMembers:
     {
-      if ( item && item->type() == Smb4KBasicNetworkItem::Workgroup )
+      if ( item && item->type() == Workgroup )
       {
         // Only return TRUE if a job for the passed workgroup is running.
         Smb4KWorkgroup *workgroup = static_cast<Smb4KWorkgroup *>( item );
@@ -177,7 +177,7 @@ bool Smb4KScanner::isRunning( Smb4KGlobal::Process process, Smb4KBasicNetworkIte
     }
     case LookupShares:
     {
-      if ( item && item->type() == Smb4KBasicNetworkItem::Host )
+      if ( item && item->type() == Host )
       {
         // Only return TRUE if a job for the passed host is running.
         Smb4KHost *host = static_cast<Smb4KHost *>( item );
@@ -224,7 +224,7 @@ bool Smb4KScanner::isRunning( Smb4KGlobal::Process process, Smb4KBasicNetworkIte
     }
     case LookupInfo:
     {
-      if ( item && item->type() == Smb4KBasicNetworkItem::Host )
+      if ( item && item->type() == Host )
       {
         // Only return TRUE if a job for the passed host is running.
         Smb4KHost *host = static_cast<Smb4KHost *>( item );
@@ -314,7 +314,7 @@ void Smb4KScanner::abort( Smb4KGlobal::Process process, Smb4KBasicNetworkItem *i
     }
     case LookupDomainMembers:
     {
-      if ( item && item->type() == Smb4KBasicNetworkItem::Workgroup )
+      if ( item && item->type() == Workgroup )
       {
         // Only kill a job if the workgroup matches.
         Smb4KWorkgroup *workgroup = static_cast<Smb4KWorkgroup *>( item );
@@ -361,7 +361,7 @@ void Smb4KScanner::abort( Smb4KGlobal::Process process, Smb4KBasicNetworkItem *i
     }
     case LookupShares:
     {
-      if ( item && item->type() == Smb4KBasicNetworkItem::Host )
+      if ( item && item->type() == Host )
       {
         // Only kill a job if the host matches
         Smb4KHost *host = static_cast<Smb4KHost *>( item );
@@ -409,7 +409,7 @@ void Smb4KScanner::abort( Smb4KGlobal::Process process, Smb4KBasicNetworkItem *i
     }
     case LookupInfo:
     {
-      if ( item && item->type() == Smb4KBasicNetworkItem::Host )
+      if ( item && item->type() == Host )
       {
         // Only return TRUE if a job for the passed host is running.
         Smb4KHost *host = static_cast<Smb4KHost *>( item );
@@ -606,8 +606,7 @@ void Smb4KScanner::lookupDomains( QWidget *parent )
     // browser instead.
     if ( Smb4KSettings::customMasterBrowser().isEmpty() )
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->emptyCustomMasterBrowser();
+      Smb4KNotification::emptyCustomMasterBrowser();
     }
     else
     {
@@ -666,8 +665,7 @@ void Smb4KScanner::lookupDomains( QWidget *parent )
     }
     else
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->emptyBroadcastAreas();
+      Smb4KNotification::emptyBroadcastAreas();
     }
   }
   else
@@ -960,7 +958,7 @@ void Smb4KScanner::slotJobFinished( KJob *job )
 }
 
 
-void Smb4KScanner::slotAuthError( Smb4KQueryMasterJob *job )
+void Smb4KScanner::slotAuthError(Smb4KQueryMasterJob *job)
 {
   // Do not allow periodic scanning when an authentication
   // error occurred. We do not want to operate on a network
@@ -969,49 +967,49 @@ void Smb4KScanner::slotAuthError( Smb4KQueryMasterJob *job )
   
   Smb4KHost master_browser;
   
-  if ( !job->masterBrowser().isEmpty() )
+  if (!job->masterBrowser().isEmpty())
   {
-    master_browser.setIsMasterBrowser( true );
+    master_browser.setIsMasterBrowser(true);
 
-    if ( QHostAddress( job->masterBrowser() ).protocol() == QAbstractSocket::UnknownNetworkLayerProtocol )
+    if (QHostAddress( job->masterBrowser() ).protocol() == QAbstractSocket::UnknownNetworkLayerProtocol)
     {
-      master_browser.setHostName( job->masterBrowser() );
+      master_browser.setHostName(job->masterBrowser());
     }
     else
     {
-      master_browser.setIP( job->masterBrowser() );
+      master_browser.setIP(job->masterBrowser());
     }
     
-    emit authError ( &master_browser, LookupDomains );
+    emit authError (&master_browser, LookupDomains);
   }
   else
   {
     // Do nothing
   }
 
-  if ( Smb4KWalletManager::self()->showPasswordDialog( &master_browser, job->parentWidget() ) )
+  if (Smb4KWalletManager::self()->showPasswordDialog(&master_browser, job->parentWidget()))
   {
     // Start a query job with the returned master browser.
-    Smb4KQueryMasterJob *job = new Smb4KQueryMasterJob( this );
-    job->setObjectName( "LookupDomainsJob" );
-    job->setupLookup( job->masterBrowser(), job->parentWidget() );
+    Smb4KQueryMasterJob *job = new Smb4KQueryMasterJob(this);
+    job->setObjectName("LookupDomainsJob");
+    job->setupLookup(master_browser.hostName().isEmpty() ? master_browser.ip() : master_browser.hostName(), job->parentWidget());
 
-    connect( job, SIGNAL(result(KJob*)), SLOT(slotJobFinished(KJob*)) );
-    connect( job, SIGNAL(aboutToStart()), SLOT(slotAboutToStartDomainsLookup()) );
-    connect( job, SIGNAL(finished()), SLOT(slotDomainsLookupFinished()) );
-    connect( job, SIGNAL(workgroups(QList<Smb4KWorkgroup*>)), SLOT(slotWorkgroups(QList<Smb4KWorkgroup*>)) );
-    connect( job, SIGNAL(authError(Smb4KQueryMasterJob*)), SLOT(slotAuthError(Smb4KQueryMasterJob*)) );
+    connect(job, SIGNAL(result(KJob*)), SLOT(slotJobFinished(KJob*)));
+    connect(job, SIGNAL(aboutToStart()), SLOT(slotAboutToStartDomainsLookup()));
+    connect(job, SIGNAL(finished()), SLOT(slotDomainsLookupFinished()));
+    connect(job, SIGNAL(workgroups(QList<Smb4KWorkgroup*>)), SLOT(slotWorkgroups(QList<Smb4KWorkgroup*>)));
+    connect(job, SIGNAL(authError(Smb4KQueryMasterJob*)), SLOT(slotAuthError(Smb4KQueryMasterJob*)));
 
-    if ( !hasSubjobs() && modifyCursor() )
+    if (!hasSubjobs() && modifyCursor())
     {
-      QApplication::setOverrideCursor( Qt::BusyCursor );
+      QApplication::setOverrideCursor(Qt::BusyCursor);
     }
     else
     {
       // Do nothing
     }
 
-    addSubjob( job );
+    addSubjob(job);
 
     job->start();
   }
@@ -1024,22 +1022,23 @@ void Smb4KScanner::slotAuthError( Smb4KQueryMasterJob *job )
 }
 
 
-void Smb4KScanner::slotAuthError( Smb4KLookupDomainMembersJob *job )
+void Smb4KScanner::slotAuthError(Smb4KLookupDomainMembersJob *job)
 {
   // Do not allow periodic scanning when an authentication
   // error occurred. We do not want to operate on a network
   // item that might get invalidated during periodic scanning.
   d->scanningAllowed = false;
 
-  Smb4KHost *master_browser = findHost( job->workgroup()->masterBrowserName(), job->workgroup()->workgroupName() );
+  Smb4KWorkgroup *workgroup = findWorkgroup(job->workgroup()->workgroupName());
+  Smb4KHost *master_browser = findHost(job->workgroup()->masterBrowserName(), job->workgroup()->workgroupName());
   
-  if ( master_browser )
+  if (workgroup && master_browser)
   {
-    emit authError( master_browser, LookupDomainMembers );
+    emit authError(master_browser, LookupDomainMembers);
     
-    if ( Smb4KWalletManager::self()->showPasswordDialog( master_browser, job->parentWidget() ) )
+    if (Smb4KWalletManager::self()->showPasswordDialog(master_browser, job->parentWidget()))
     {
-      lookupDomainMembers( job->workgroup(), job->parentWidget() );
+      lookupDomainMembers(workgroup, job->parentWidget());
     }
     else
     {
@@ -1055,22 +1054,22 @@ void Smb4KScanner::slotAuthError( Smb4KLookupDomainMembersJob *job )
 }
 
 
-void Smb4KScanner::slotAuthError( Smb4KLookupSharesJob *job )
+void Smb4KScanner::slotAuthError(Smb4KLookupSharesJob *job)
 {
   // Do not allow periodic scanning when an authentication
   // error occurred. We do not want to operate on a network
   // item that might get invalidated during periodic scanning.
   d->scanningAllowed = false;
   
-  Smb4KHost *host = findHost( job->host()->hostName(), job->host()->workgroupName() );
+  Smb4KHost *host = findHost(job->host()->hostName(), job->host()->workgroupName());
   
-  if ( host )
+  if (host)
   {
-    emit authError( host, LookupShares );
+    emit authError(host, LookupShares);
     
-    if ( Smb4KWalletManager::self()->showPasswordDialog( host, job->parentWidget() ) )
+    if (Smb4KWalletManager::self()->showPasswordDialog(host, job->parentWidget()))
     {
-      lookupShares( host, job->parentWidget() );
+      lookupShares(host, job->parentWidget());
     }
     else
     {
@@ -1415,21 +1414,36 @@ void Smb4KScanner::slotHosts( Smb4KWorkgroup *workgroup, const QList<Smb4KHost *
 
 void Smb4KScanner::slotShares( Smb4KHost *host, const QList<Smb4KShare *> &shares_list )
 {
-  Q_ASSERT( host );
+  Q_ASSERT(host);
   
-  if ( !shares_list.isEmpty() )
+  if (host && !shares_list.isEmpty())
   {
-    // Copy some information before processing the shares further. 
-    // Note, that the IP address and other information stemming from
-    // the host were already entered by the lookup job.
-    for ( int i = 0; i < shares_list.size(); ++i )
+    // Process list of shares:
+    QList<Smb4KShare *> new_shares;
+    
+    // Copy all shares that the user wishes to retrieve and set the mount data
+    // as well as the IP address (if not already present).
+    for (int i = 0; i < shares_list.size(); ++i)
     {
+      if (shares_list.at(i)->isPrinter() && !Smb4KSettings::detectPrinterShares())
+      {
+        continue;
+      }
+      else if (shares_list.at(i)->isHidden() && !Smb4KSettings::detectHiddenShares())
+      {
+        continue;
+      }
+      else
+      {
+        // Do nothing
+      }
+      
       // Check if the share has already been mounted.
       QList<Smb4KShare *> mounted_shares = findShareByUNC( shares_list.at( i )->unc() );
       
       if ( !mounted_shares.isEmpty() )
       {
-        // FIXME: We cannot honor Smb4KSettings::showAllShares() here, because 
+        // FIXME: We cannot honor Smb4KSettings::detectAllShares() here, because 
         // in case the setting is changed, there will be no automatic rescan
         // (in case of an automatic or periodical rescan that would be the 
         // favorable method...
@@ -1456,68 +1470,71 @@ void Smb4KScanner::slotShares( Smb4KHost *host, const QList<Smb4KShare *> &share
       else
       {
         // Do nothing
-      }
-      
-      // Now set some information that might have been collected
-      // since the lookup started...
-      Smb4KShare *share = findShare( shares_list.at( i )->shareName(), shares_list.at( i )->hostName(), shares_list.at( i )->workgroupName() );
-        
-      if ( share )
+      }      
+ 
+      // Now set the IP address, if none could be retrieved by the
+      // lookup job. 
+      if (!shares_list.at(i)->hasHostIP())
       {
-        if ( !shares_list.at( i )->hasHostIP() && share->hasHostIP() )
+        Smb4KShare *s = findShare(shares_list.at(i)->shareName(),
+                                  shares_list.at(i)->hostName(),
+                                  shares_list.at(i)->workgroupName());
+        
+        if (s)
         {
-          shares_list[i]->setHostIP( share->hostIP() );
+          shares_list[i]->setHostIP(s->hostIP());
         }
         else
         {
           // Do nothing
         }
-          
-        removeShare( share );
       }
       else
       {
         // Do nothing
       }
+      
+      new_shares << shares_list[i];
     }
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  // Copy authentication information
-  Smb4KHost *known_host = findHost( host->hostName(), host->workgroupName() );
-  
-  if ( known_host )
-  {
-    known_host->setLogin( host->login() );
-    known_host->setPassword( host->password() );
-  }
-  else
-  {
-    // Do nothing
-  }
-  
-  // Now remove all (obsolete) shares of the scanned host from
-  // the global list.
-  QList<Smb4KShare *> obsolete_shares = sharedResources( host );
-  QListIterator<Smb4KShare *> s( obsolete_shares );
     
-  while ( s.hasNext() )
-  {
-    Smb4KShare *share = s.next();
-    removeShare( share );
+    // Process the host:
+    // Copy the authentication information.
+    Smb4KHost *known_host = findHost(host->hostName(), host->workgroupName());
+    
+    if(known_host)
+    {
+      known_host->setLogin(host->login());
+      known_host->setPassword(host->password());
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Now remove all shares of this host from the global list.
+    QList<Smb4KShare *> known_shares = sharedResources(known_host);
+    QListIterator<Smb4KShare *> it(known_shares);
+    
+    while(it.hasNext())
+    {
+      Smb4KShare *s = it.next();
+      removeShare(s);
+    }
+    
+    // Add a copy of all desired shares to the global list.
+    for (int i = 0; i < new_shares.size(); ++i)
+    {
+      addShare(new Smb4KShare(*new_shares[i]));
+    }
+    
+    // Now emit the list of shared resources.
+    QList<Smb4KShare *> shared_resources = sharedResources(host);
+    emit shares(host, shared_resources);
   }
-  
-  // Add a copy of all shares to the global list.
-  for ( int i = 0; i < shares_list.size(); ++i )
+  else
   {
-    addShare( new Smb4KShare( *shares_list.at( i ) ) );
+    // Do nothing
   }
-  
-  QList<Smb4KShare *> shared_resources = sharedResources( host );
-  emit shares( host, shared_resources );
 }
 
 
