@@ -2,7 +2,7 @@
     smb4kscanner_p  -  Private helper classes for the scanner
                              -------------------
     begin                : So Mai 22 2011
-    copyright            : (C) 2011-2013 by Alexander Reinholdt
+    copyright            : (C) 2011-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -98,13 +98,13 @@ bool Smb4KLookupDomainsJob::doKill()
 
 void Smb4KLookupDomainsJob::processWorkgroups()
 {
-  QStringList stdout = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
+  QStringList stdOut = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
 
-  if ( !stdout.isEmpty() )
+  if ( !stdOut.isEmpty() )
   {
     Smb4KWorkgroup *workgroup = new Smb4KWorkgroup();
 
-    foreach ( const QString &line, stdout )
+    foreach ( const QString &line, stdOut )
     {
       if ( line.startsWith( QLatin1String( "Looking up status of" ) ) )
       {
@@ -205,8 +205,7 @@ void Smb4KLookupDomainsJob::slotStartLookup()
 
   if ( nmblookup.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "nmblookup" );
+    Smb4KNotification::commandNotFound("nmblookup");
     emitResult();
     return;
   }
@@ -220,8 +219,7 @@ void Smb4KLookupDomainsJob::slotStartLookup()
 
   if ( grep.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "grep" );
+    Smb4KNotification::commandNotFound("grep");
     emitResult();
     return;
   }
@@ -235,8 +233,7 @@ void Smb4KLookupDomainsJob::slotStartLookup()
 
   if ( awk.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "awk" );
+    Smb4KNotification::commandNotFound("awk");
     emitResult();
     return;
   }
@@ -250,8 +247,7 @@ void Smb4KLookupDomainsJob::slotStartLookup()
 
   if ( xargs.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "xargs" );
+    Smb4KNotification::commandNotFound("xargs");
     emitResult();
     return;
   }
@@ -434,20 +430,19 @@ void Smb4KLookupDomainsJob::slotStartLookup()
 void Smb4KLookupDomainsJob::slotReadStandardError()
 {
   // Read from stderr and decide what to do.
-  QString stderr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
+  QString stdErr = QString::fromUtf8(m_proc->readAllStandardError(), -1).trimmed();
 
   // Remove unimportant warnings
-  if ( stderr.contains( "Ignoring unknown parameter" ) )
+  if (stdErr.contains("Ignoring unknown parameter"))
   {
-    QStringList tmp = stderr.split( '\n' );
+    QStringList list = stdErr.split('\n');
+    QMutableStringListIterator it(list);
 
-    QMutableStringListIterator it( tmp );
-
-    while ( it.hasNext() )
+    while (it.hasNext())
     {
       QString test = it.next();
 
-      if ( test.trimmed().startsWith( QLatin1String( "Ignoring unknown parameter" ) ) )
+      if (test.trimmed().startsWith(QLatin1String("Ignoring unknown parameter")))
       {
         it.remove();
       }
@@ -457,17 +452,39 @@ void Smb4KLookupDomainsJob::slotReadStandardError()
       }
     }
 
-    stderr = tmp.join( "\n" );
+    stdErr = list.join("\n");
+  }
+  else if (stdErr.contains("smb.conf"))
+  {
+    QStringList list = stdErr.split('\n');
+    QMutableStringListIterator it(list);
+    
+    while (it.hasNext())
+    {
+      QString test = it.next();
+      
+      if (test.contains(QLatin1String("smb.conf")) && test.contains(QLatin1String("Can't load")))
+      {
+        // smb.conf is missing.
+        // Output from nmblookup (1 line)
+        it.remove();
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    
+    stdErr = list.join("\n");
   }
   else
   {
     // Do nothing
   }
   
-  if ( !stderr.isEmpty() )
+  if ( !stdErr.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->retrievingDomainsFailed( stderr );
+    Smb4KNotification::retrievingDomainsFailed(stdErr);
   }
   else
   {
@@ -484,8 +501,7 @@ void Smb4KLookupDomainsJob::slotProcessFinished( int /*exitCode*/, QProcess::Exi
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -552,13 +568,13 @@ bool Smb4KQueryMasterJob::doKill()
 
 void Smb4KQueryMasterJob::processWorkgroups()
 {
-  QStringList stdout = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
+  QStringList stdOut = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
 
-  if ( !stdout.isEmpty() )
+  if ( !stdOut.isEmpty() )
   {
     Smb4KWorkgroup *workgroup = new Smb4KWorkgroup();
 
-    foreach ( const QString &line, stdout )
+    foreach ( const QString &line, stdOut )
     {
       if ( line.trimmed().startsWith( QLatin1String( "Enumerating" ) ) )
       {
@@ -609,8 +625,7 @@ void Smb4KQueryMasterJob::slotStartLookup()
 
   if ( net.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "net" );
+    Smb4KNotification::commandNotFound("net");
     emitResult();
     return;
   }
@@ -624,8 +639,7 @@ void Smb4KQueryMasterJob::slotStartLookup()
 
   if ( xargs.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "xargs" );
+    Smb4KNotification::commandNotFound("xargs");
     emitResult();
     return;
   }
@@ -736,6 +750,71 @@ void Smb4KQueryMasterJob::slotStartLookup()
   {
     // Do nothing
   }
+  
+  // Encrypt SMB transport
+  if (Smb4KSettings::encryptSMBTransport())
+  {
+    arguments << "-e";
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Use Kerberos
+  if (options)
+  {
+    switch (options->useKerberos())
+    {
+      case Smb4KCustomOptions::UseKerberos:
+      {
+        arguments << "-k";
+        break;
+      }
+      case Smb4KCustomOptions::NoKerberos:
+      {
+        // No kerberos 
+        break;
+      }
+      case Smb4KCustomOptions::UndefinedKerberos:
+      {
+        if (Smb4KSettings::useKerberos())
+        {
+          arguments << "-k";
+        }
+        else
+        {
+          // Do nothing
+        }
+        break;
+      }
+      default:
+      {
+        break;
+      }
+    }
+  }
+  else
+  {
+    if (Smb4KSettings::useKerberos())
+    {
+      arguments << "-k";
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  // Use Winbind's ccache
+  if (Smb4KSettings::useWinbindCCache())
+  {
+    arguments << "--use-ccache";
+  }
+  else
+  {
+    // Do nothing
+  }
 
   // Port
   if ( options && options->smbPort() != Smb4KSettings::remoteSMBPort() )
@@ -801,6 +880,71 @@ void Smb4KQueryMasterJob::slotStartLookup()
   {
     // Do nothing
   }
+  
+  // Encrypt SMB transport
+  if (Smb4KSettings::encryptSMBTransport())
+  {
+    arguments << "-e";
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Use Kerberos
+  if (options)
+  {
+    switch (options->useKerberos())
+    {
+      case Smb4KCustomOptions::UseKerberos:
+      {
+        arguments << "-k";
+        break;
+      }
+      case Smb4KCustomOptions::NoKerberos:
+      {
+        // No kerberos 
+        break;
+      }
+      case Smb4KCustomOptions::UndefinedKerberos:
+      {
+        if (Smb4KSettings::useKerberos())
+        {
+          arguments << "-k";
+        }
+        else
+        {
+          // Do nothing
+        }
+        break;
+      }
+      default:
+      {
+        break;
+      }
+    }
+  }
+  else
+  {
+    if (Smb4KSettings::useKerberos())
+    {
+      arguments << "-k";
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  // Use Winbind's ccache
+  if (Smb4KSettings::useWinbindCCache())
+  {
+    arguments << "--use-ccache";
+  }
+  else
+  {
+    // Do nothing
+  }
 
   // Port
   if ( options && options->smbPort() != Smb4KSettings::remoteSMBPort() )
@@ -823,7 +967,7 @@ void Smb4KQueryMasterJob::slotStartLookup()
     arguments << "-U %";
   }
 
-  // IP address (discovered by by previous net command)
+  // IP address (discovered by previous net command)
   arguments << "-I ip";
 
   // Server name if available
@@ -865,12 +1009,12 @@ void Smb4KQueryMasterJob::slotStartLookup()
 void Smb4KQueryMasterJob::slotReadStandardError()
 {
   // Read from stderr and decide what to do.
-  QString stderr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
+  QString stdErr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
 
   // Remove unimportant warnings
-  if ( stderr.contains( "Ignoring unknown parameter" ) )
+  if ( stdErr.contains( "Ignoring unknown parameter" ) )
   {
-    QStringList tmp = stderr.split( '\n' );
+    QStringList tmp = stdErr.split( '\n' );
 
     QMutableStringListIterator it( tmp );
 
@@ -888,26 +1032,53 @@ void Smb4KQueryMasterJob::slotReadStandardError()
       }
     }
 
-    stderr = tmp.join( "\n" );
+    stdErr = tmp.join( "\n" );
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Remove irrelevant error messages
+  if (stdErr.contains("messaging_tdb_init failed:"))
+  {
+    QStringList tmp = stdErr.split('\n');
+    QMutableStringListIterator it(tmp);
+
+    while (it.hasNext())
+    {
+      QString test = it.next();
+
+      if (test.trimmed().startsWith(QLatin1String("messaging_tdb_init failed:")))
+      {
+        it.remove();
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+
+    stdErr = tmp.join("\n");
   }
   else
   {
     // Do nothing
   }
 
-  if ( !stderr.isEmpty() )
+  if ( !stdErr.isEmpty() )
   {
-    if ( stderr.contains( "The username or password was not correct." ) ||
-         stderr.contains( "NT_STATUS_ACCOUNT_DISABLED" ) /* AD error */ ||
-         stderr.contains( "NT_STATUS_ACCESS_DENIED" ) ||
-         stderr.contains( "NT_STATUS_LOGON_FAILURE" ) )
+    if ( stdErr.contains( "The username or password was not correct." ) ||
+         stdErr.contains( "NT_STATUS_ACCOUNT_DISABLED" ) /* AD error */ ||
+         stdErr.contains( "NT_STATUS_ACCESS_DENIED" ) ||
+         stdErr.contains( "NT_STATUS_LOGON_FAILURE" ) )
     {
       if ( m_master_browser.isEmpty() )
       {
         // Figure out the current master browser's IP address.
-        QStringList stderr_list = stderr.split( '\n', QString::SkipEmptyParts );
+        QStringList stdErr_list = stdErr.split( '\n', QString::SkipEmptyParts );
 
-        foreach ( const QString &line, stderr_list )
+        foreach ( const QString &line, stdErr_list )
         {
           if ( line.contains( "Connecting to host=" ) )
           {
@@ -930,10 +1101,9 @@ void Smb4KQueryMasterJob::slotReadStandardError()
     else
     {
       // Avoid reporting the debug output as error.
-      if ( stderr.contains( "NT_STATUS" ) )
+      if ( stdErr.contains( "NT_STATUS" ) )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->retrievingDomainsFailed( stderr );
+        Smb4KNotification::retrievingDomainsFailed(stdErr);
       }
       else
       {
@@ -956,8 +1126,7 @@ void Smb4KQueryMasterJob::slotProcessFinished( int /*exitCode*/, QProcess::ExitS
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -1028,15 +1197,15 @@ bool Smb4KScanBAreasJob::doKill()
 
 void Smb4KScanBAreasJob::processScan()
 {
-  QStringList stdout = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).trimmed().split( '\n', QString::SkipEmptyParts );
+  QStringList stdOut = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).trimmed().split( '\n', QString::SkipEmptyParts );
 
-  if ( !stdout.isEmpty() )
+  if ( !stdOut.isEmpty() )
   {
     Smb4KWorkgroup *workgroup = new Smb4KWorkgroup();
     Smb4KHost *host = new Smb4KHost();
     bool skip = false;
 
-    foreach ( const QString &line, stdout )
+    foreach ( const QString &line, stdOut )
     {
       // Check if we have to skip this host entry.
       // A host entry is skipped if the IP address is invalid, i.e.
@@ -1192,8 +1361,7 @@ void Smb4KScanBAreasJob::slotStartScan()
 
   if ( nmblookup.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "nmblookup" );
+    Smb4KNotification::commandNotFound("nmblookup");
     emitResult();
     return;
   }
@@ -1207,8 +1375,7 @@ void Smb4KScanBAreasJob::slotStartScan()
 
   if ( awk.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "awk" );
+    Smb4KNotification::commandNotFound("awk");
     emitResult();
     return;
   }
@@ -1222,8 +1389,7 @@ void Smb4KScanBAreasJob::slotStartScan()
 
   if ( sed.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "sed" );
+    Smb4KNotification::commandNotFound("sed");
     emitResult();
     return;
   }
@@ -1237,8 +1403,7 @@ void Smb4KScanBAreasJob::slotStartScan()
 
   if ( xargs.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "xargs" );
+    Smb4KNotification::commandNotFound("xargs");
     emitResult();
     return;
   }
@@ -1332,6 +1497,7 @@ void Smb4KScanBAreasJob::slotStartScan()
     arguments << QString( "-B %1" ).arg( addresses.at( i ) );
     arguments << "--";
     arguments << QString( "%1" ).arg( KShell::quoteArg( "*" ) );
+    arguments << "|";
     arguments << sed;
     arguments << "-e /querying/d";
     arguments << "|";
@@ -1437,20 +1603,19 @@ void Smb4KScanBAreasJob::slotStartScan()
 
 void Smb4KScanBAreasJob::slotReadStandardError()
 {
-  QString stderr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
+  QString stdErr = QString::fromUtf8(m_proc->readAllStandardError(), -1).trimmed();
 
   // Remove unimportant warnings
-  if ( stderr.contains( "Ignoring unknown parameter" ) )
+  if ( stdErr.contains("Ignoring unknown parameter"))
   {
-    QStringList tmp = stderr.split( '\n' );
+    QStringList list = stdErr.split('\n');
+    QMutableStringListIterator it(list);
 
-    QMutableStringListIterator it( tmp );
-
-    while ( it.hasNext() )
+    while (it.hasNext())
     {
       QString test = it.next();
 
-      if ( test.trimmed().startsWith( QLatin1String( "Ignoring unknown parameter" ) ) )
+      if (test.trimmed().startsWith(QLatin1String("Ignoring unknown parameter")))
       {
         it.remove();
       }
@@ -1460,17 +1625,39 @@ void Smb4KScanBAreasJob::slotReadStandardError()
       }
     }
 
-    stderr = tmp.join( "\n" );
+    stdErr = list.join("\n");
+  }
+  else if (stdErr.contains("smb.conf"))
+  {
+    QStringList list = stdErr.split('\n');
+    QMutableStringListIterator it(list);
+    
+    while (it.hasNext())
+    {
+      QString test = it.next();
+      
+      if (test.contains(QLatin1String("smb.conf")) && test.contains(QLatin1String("Can't load")))
+      {
+        // smb.conf is missing.
+        // Output from nmblookup (1 line)
+        it.remove();
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    
+    stdErr = list.join("\n");
   }
   else
   {
     // Do nothing
   }
 
-  if ( !stderr.isEmpty() )
+  if ( !stdErr.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->scanningBroadcastAreaFailed( stderr );
+    Smb4KNotification::scanningBroadcastAreaFailed(stdErr);
   }
   else
   {
@@ -1487,8 +1674,7 @@ void Smb4KScanBAreasJob::slotProcessFinished( int /*exitCode*/, QProcess::ExitSt
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -1561,13 +1747,13 @@ bool Smb4KLookupDomainMembersJob::doKill()
 
 void Smb4KLookupDomainMembersJob::processHosts()
 {
-  QStringList stdout = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
+  QStringList stdOut = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
 
-  if ( !stdout.isEmpty() )
+  if ( !stdOut.isEmpty() )
   {
     Smb4KHost *host = new Smb4KHost();
 
-    foreach ( const QString &line, stdout )
+    foreach ( const QString &line, stdOut )
     {
       if ( line.trimmed().startsWith( QLatin1String( "Enumerating" ) ) )
       {
@@ -1643,8 +1829,7 @@ void Smb4KLookupDomainMembersJob::slotStartLookup()
 
   if ( net.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "net" );
+    Smb4KNotification::commandNotFound("net");
     emitResult();
     return;
   }
@@ -1720,6 +1905,71 @@ void Smb4KLookupDomainMembersJob::slotStartLookup()
     {
       // Do nothing
     }
+    
+    // Encrypt SMB transport
+    if (Smb4KSettings::encryptSMBTransport())
+    {
+      arguments << "-e";
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Use Kerberos
+    if (options)
+    {
+      switch (options->useKerberos())
+      {
+        case Smb4KCustomOptions::UseKerberos:
+        {
+          arguments << "-k";
+          break;
+        }
+        case Smb4KCustomOptions::NoKerberos:
+        {
+          // No kerberos 
+          break;
+        }
+        case Smb4KCustomOptions::UndefinedKerberos:
+        {
+          if (Smb4KSettings::useKerberos())
+          {
+            arguments << "-k";
+          }
+          else
+          {
+            // Do nothing
+          }
+          break;
+        }
+        default:
+        {
+          break;
+        }
+      }
+    }
+    else
+    {
+      if (Smb4KSettings::useKerberos())
+      {
+        arguments << "-k";
+      }
+      else
+      {
+        // Do nothing
+      }
+    }    
+    
+    // Use Winbind's ccache
+    if (Smb4KSettings::useWinbindCCache())
+    {
+      arguments << "--use-ccache";
+    }
+    else
+    {
+      // Do nothing
+    }
 
     // Remote SMB port
     if ( options && options->smbPort() != Smb4KSettings::remoteSMBPort() )
@@ -1791,12 +2041,12 @@ void Smb4KLookupDomainMembersJob::slotStartLookup()
 
 void Smb4KLookupDomainMembersJob::slotReadStandardError()
 {
-  QString stderr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
+  QString stdErr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
 
   // Remove unimportant warnings
-  if ( stderr.contains( "Ignoring unknown parameter" ) )
+  if ( stdErr.contains( "Ignoring unknown parameter" ) )
   {
-    QStringList tmp = stderr.split( '\n' );
+    QStringList tmp = stdErr.split( '\n' );
 
     QMutableStringListIterator it( tmp );
 
@@ -1814,33 +2064,32 @@ void Smb4KLookupDomainMembersJob::slotReadStandardError()
       }
     }
 
-    stderr = tmp.join( "\n" );
+    stdErr = tmp.join( "\n" );
   }
   else
   {
     // Do nothing
   }
 
-  if ( !stderr.isEmpty() )
+  if ( !stdErr.isEmpty() )
   {
-    if ( stderr.contains( "The username or password was not correct." ) ||
-         stderr.contains( "NT_STATUS_ACCOUNT_DISABLED" ) /* AD error */ ||
-         stderr.contains( "NT_STATUS_ACCESS_DENIED" ) ||
-         stderr.contains( "NT_STATUS_LOGON_FAILURE" ) )
+    if ( stdErr.contains( "The username or password was not correct." ) ||
+         stdErr.contains( "NT_STATUS_ACCOUNT_DISABLED" ) /* AD error */ ||
+         stdErr.contains( "NT_STATUS_ACCESS_DENIED" ) ||
+         stdErr.contains( "NT_STATUS_LOGON_FAILURE" ) )
     {
       emit authError( this );
     }
-    else if ( stderr.contains( "tdb_transaction_recover:" ) ||
-              stderr.contains( "tdb_log" ) )
+    else if ( stdErr.contains( "tdb_transaction_recover:" ) ||
+              stdErr.contains( "tdb_log" ) )
     {
       // Suppress debug output/information send to stderr
-      qDebug() << stderr;
+      qDebug() << stdErr;
     }
     else
     {
       // Notify the user that an error occurred.
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->retrievingServersFailed( m_workgroup, stderr );
+      Smb4KNotification::retrievingHostsFailed(m_workgroup, stdErr);
     }
   }
   else
@@ -1858,8 +2107,7 @@ void Smb4KLookupDomainMembersJob::slotProcessFinished( int /*exitCode*/, QProces
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -1929,13 +2177,31 @@ bool Smb4KLookupSharesJob::doKill()
 
 void Smb4KLookupSharesJob::processShares()
 {
-  QStringList stdout = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
-  
-  if ( !stdout.isEmpty() )
+  QStringList stdOut = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).split( '\n', QString::SkipEmptyParts );
+
+  if ( !stdOut.isEmpty() )
   {
+    // For some strange reason, Samba sometimes reports
+    // some NT_STATUS error messages via stdout (e.g. Kubuntu 13.10,
+    // Samba 3.6.18). So, we need to check stdout for authentication
+    // errors as well...
+    if ( stdOut.contains( "The username or password was not correct." ) ||
+         stdOut.contains( "NT_STATUS_ACCOUNT_DISABLED" ) /* AD error */ ||
+         stdOut.contains( "NT_STATUS_ACCESS_DENIED" ) ||
+         stdOut.contains( "NT_STATUS_LOGON_FAILURE" ) )
+    {
+      emit authError( this );
+      return;
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Now process the shares.
     Smb4KShare *share = new Smb4KShare();
     
-    foreach ( const QString &line, stdout )
+    foreach ( const QString &line, stdOut )
     {
       if ( line.trimmed().startsWith( QLatin1String( "Enumerating" ) ) )
       {
@@ -2084,8 +2350,7 @@ void Smb4KLookupSharesJob::slotStartLookup()
 
   if ( net.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "net" );
+    Smb4KNotification::commandNotFound("net");
     emitResult();
     return;
   }
@@ -2193,6 +2458,26 @@ void Smb4KLookupSharesJob::slotStartLookup()
     // Do nothing
   }
   
+  // Encrypt SMB transport
+  if (Smb4KSettings::encryptSMBTransport())
+  {
+    arguments << "-e";
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Use Winbind's ccache
+  if (Smb4KSettings::useWinbindCCache())
+  {
+    arguments << "--use-ccache";
+  }
+  else
+  {
+    // Do nothing
+  }
+  
   // Port
   // If a port was defined for the host via Smb4KHost::port(), it will 
   // overwrite the other options.
@@ -2253,7 +2538,7 @@ void Smb4KLookupSharesJob::slotStartLookup()
   {
     m_proc->unsetEnv( "PASSWD" );
   }
-
+  
   connect( m_proc, SIGNAL(readyReadStandardError()), this, SLOT(slotReadStandardError()) );
   connect( m_proc, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(slotProcessFinished(int,QProcess::ExitStatus)) );
 
@@ -2265,12 +2550,12 @@ void Smb4KLookupSharesJob::slotStartLookup()
 
 void Smb4KLookupSharesJob::slotReadStandardError()
 {
-  QString stderr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
+  QString stdErr = QString::fromUtf8( m_proc->readAllStandardError(), -1 ).trimmed();
 
   // Remove unimportant warnings
-  if ( stderr.contains( "Ignoring unknown parameter" ) )
+  if ( stdErr.contains( "Ignoring unknown parameter" ) )
   {
-    QStringList tmp = stderr.split( '\n' );
+    QStringList tmp = stdErr.split( '\n' );
 
     QMutableStringListIterator it( tmp );
 
@@ -2292,31 +2577,30 @@ void Smb4KLookupSharesJob::slotReadStandardError()
       }
     }
 
-    stderr = tmp.join( "\n" );
+    stdErr = tmp.join( "\n" );
   }
   else
   {
     // Do nothing
   }
 
-  if ( !stderr.isEmpty() )
+  if ( !stdErr.isEmpty() )
   {
-    if ( stderr.contains( "The username or password was not correct." ) ||
-         stderr.contains( "NT_STATUS_ACCOUNT_DISABLED" ) /* AD error */ ||
-         stderr.contains( "NT_STATUS_ACCESS_DENIED" ) ||
-         stderr.contains( "NT_STATUS_LOGON_FAILURE" ) )
+    if ( stdErr.contains( "The username or password was not correct." ) ||
+         stdErr.contains( "NT_STATUS_ACCOUNT_DISABLED" ) /* AD error */ ||
+         stdErr.contains( "NT_STATUS_ACCESS_DENIED" ) ||
+         stdErr.contains( "NT_STATUS_LOGON_FAILURE" ) )
     {
       emit authError( this );
     }
-    else if ( stderr.contains( "could not obtain sid for domain", Qt::CaseSensitive ) )
+    else if ( stdErr.contains( "could not obtain sid for domain", Qt::CaseSensitive ) )
     {
       // FIXME
       qDebug() << "FIXME: Wrong protocol used for host " << m_host->hostName();
     }
     else
     {
-      Smb4KNotification *notification = new Smb4KNotification();
-      notification->retrievingSharesFailed( m_host, stderr );
+      Smb4KNotification::retrievingSharesFailed(m_host, stdErr);
     }
   }
   else
@@ -2334,8 +2618,7 @@ void Smb4KLookupSharesJob::slotProcessFinished( int /*exitCode*/, QProcess::Exit
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -2402,13 +2685,13 @@ void Smb4KLookupInfoJob::processInfo()
 {
   // First evaluate stdout and if we cannot find the appropriate 
   // information also evaluate stderr.
-  QString stdout = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 );
+  QString stdOut = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 );
   
-  if ( stdout.contains( "OS=" ) || stdout.contains( "Server=" ) )
+  if ( stdOut.contains( "OS=" ) || stdOut.contains( "Server=" ) )
   {
-    QStringList stdout_list = stdout.split( '\n', QString::SkipEmptyParts );
+    QStringList stdOut_list = stdOut.split( '\n', QString::SkipEmptyParts );
     
-    foreach ( const QString &line, stdout_list )
+    foreach ( const QString &line, stdOut_list )
     {
       if ( line.contains( "OS=" ) || line.contains( "Server=" ) )
       {
@@ -2426,13 +2709,13 @@ void Smb4KLookupInfoJob::processInfo()
   }
   else
   {
-    QString stderr = QString::fromUtf8( m_proc->readAllStandardError(), -1 );
+    QString stdErr = QString::fromUtf8( m_proc->readAllStandardError(), -1 );
     
-    if ( stderr.contains( "OS=" ) || stderr.contains( "Server=" ) )
+    if ( stdErr.contains( "OS=" ) || stdErr.contains( "Server=" ) )
     {
-      QStringList stderr_list = stderr.split( '\n', QString::SkipEmptyParts );
+      QStringList stdErr_list = stdErr.split( '\n', QString::SkipEmptyParts );
       
-      foreach ( const QString &line, stderr_list )
+      foreach ( const QString &line, stdErr_list )
       {
         if ( line.contains( "OS=" ) || line.contains( "Server=" ) )
         {
@@ -2459,8 +2742,7 @@ void Smb4KLookupInfoJob::slotStartLookup()
 
   if ( smbclient.isEmpty() )
   {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "smbclient" );
+    Smb4KNotification::commandNotFound("smbclient");
     emitResult();
     return;
   }
@@ -2557,7 +2839,7 @@ void Smb4KLookupInfoJob::slotStartLookup()
     }
   }
   
-  // Kerberos
+  // Use Kerberos
   if ( options )
   {
     switch ( options->useKerberos() )
@@ -2686,8 +2968,7 @@ void Smb4KLookupInfoJob::slotProcessFinished( int /*exitCode*/, QProcess::ExitSt
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -2750,7 +3031,240 @@ bool Smb4KLookupIPAddressJob::doKill()
 }
 
 
-void Smb4KLookupIPAddressJob::processIPAddress()
+void Smb4KLookupIPAddressJob::useNmblookup(QStringList &args)
+{
+  // Find nmblookup program.
+  QString nmblookup = KStandardDirs::findExe( "nmblookup" );
+
+  if ( nmblookup.isEmpty() )
+  {
+    Smb4KNotification::commandNotFound("nmblookup");
+    emitResult();
+    return;
+  }
+  else
+  {
+    // Go ahead
+  }
+
+  // Global Samba options
+  QMap<QString,QString> samba_options = globalSambaOptions();
+
+  // Compile the arguments
+  args << nmblookup;
+
+  // Domain
+  if ( !Smb4KSettings::domainName().isEmpty() &&
+       QString::compare( Smb4KSettings::domainName(), samba_options["workgroup"] ) != 0 )
+  {
+    args << QString( "-W %1" ).arg( KShell::quoteArg( Smb4KSettings::domainName() ) );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // NetBIOS name
+  if ( !Smb4KSettings::netBIOSName().isEmpty() &&
+       QString::compare( Smb4KSettings::netBIOSName(), samba_options["netbios name"] ) != 0 )
+  {
+    args << QString( "-n %1" ).arg( KShell::quoteArg( Smb4KSettings::netBIOSName() ) );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // NetBIOS scope
+  if ( !Smb4KSettings::netBIOSScope().isEmpty() &&
+       QString::compare( Smb4KSettings::netBIOSScope(), samba_options["netbios scope"] ) != 0 )
+  {
+    args << QString( "-i %1" ).arg( KShell::quoteArg( Smb4KSettings::netBIOSScope() ) );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // Socket options
+  if ( !Smb4KSettings::socketOptions().isEmpty() &&
+       QString::compare( Smb4KSettings::socketOptions(), samba_options["socket options"] ) != 0 )
+  {
+    args << QString( "-O %1" ).arg( KShell::quoteArg( Smb4KSettings::socketOptions() ) );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // Port 137
+  if ( Smb4KSettings::usePort137() )
+  {
+    args << "-r";
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // Broadcast address
+  QHostAddress address( Smb4KSettings::broadcastAddress() );
+
+  if ( !Smb4KSettings::broadcastAddress().isEmpty() &&
+       address.protocol() != QAbstractSocket::UnknownNetworkLayerProtocol )
+  {
+    args << QString( "-B %1" ).arg( Smb4KSettings::broadcastAddress() );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // We do not use the WINS server here, because it emerged that using
+  // the WINS server is sometimes not very reliable. 
+  
+  
+  if ( !winsServer().isEmpty() )
+  {
+    args << "-R";
+    args << QString( "-U %1" ).arg( KShell::quoteArg( winsServer() ) );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  args << "--";
+  args << KShell::quoteArg( m_host->hostName() );
+}
+
+
+void Smb4KLookupIPAddressJob::useNet(QStringList &args)
+{
+  // Find net program
+  QString net = KStandardDirs::findExe("net");
+
+  if ( net.isEmpty() )
+  {
+    Smb4KNotification::commandNotFound("net");
+    emitResult();
+    return;
+  }
+  else
+  {
+    // Go ahead
+  }
+  
+  // Global Samba options
+  QMap<QString,QString> samba_options = globalSambaOptions();
+  Smb4KCustomOptions *options = Smb4KCustomOptionsManager::self()->findOptions(m_host);
+  
+  // Compile the arguments
+  args << net;
+  args << "lookup";
+  args << "host";
+  args << KShell::quoteArg( m_host->hostName() );
+  
+  // The user's workgroup/domain name
+  if ( !Smb4KSettings::domainName().isEmpty() &&
+       QString::compare( Smb4KSettings::domainName(), samba_options["workgroup"] ) != 0 )
+  {
+    args << QString( "-W %1" ).arg( KShell::quoteArg( Smb4KSettings::domainName() ) );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // The user's NetBIOS name
+  if ( !Smb4KSettings::netBIOSName().isEmpty() &&
+       QString::compare( Smb4KSettings::netBIOSName(), samba_options["netbios name"] ) != 0 )
+  {
+    args << QString( "-n %1" ).arg( KShell::quoteArg( Smb4KSettings::netBIOSName() ) );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // Machine account
+  if ( Smb4KSettings::machineAccount() )
+  {
+    args << "-P";
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Encrypt SMB transport
+  if (Smb4KSettings::encryptSMBTransport())
+  {
+    args << "-e";
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Use Kerberos
+  if (options)
+  {
+    switch (options->useKerberos())
+    {
+      case Smb4KCustomOptions::UseKerberos:
+      {
+        args << "-k";
+        break;
+      }
+      case Smb4KCustomOptions::NoKerberos:
+      {
+        // No kerberos 
+        break;
+      }
+      case Smb4KCustomOptions::UndefinedKerberos:
+      {
+        if (Smb4KSettings::useKerberos())
+        {
+          args << "-k";
+        }
+        else
+        {
+          // Do nothing
+        }
+        break;
+      }
+      default:
+      {
+        break;
+      }
+    }
+  }
+  else
+  {
+    if (Smb4KSettings::useKerberos())
+    {
+      args << "-k";
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  
+  // Winbind's ccache
+  if (Smb4KSettings::useWinbindCCache())
+  {
+    args << "--use-ccache";
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
+
+void Smb4KLookupIPAddressJob::processNmblookupOutput()
 {
   // Normally, there should only be one IP address. However, there might
   // be more than one. So, split the incoming data and use the first entry
@@ -2776,121 +3290,51 @@ void Smb4KLookupIPAddressJob::processIPAddress()
 }
 
 
+void Smb4KLookupIPAddressJob::processNetOutput()
+{
+  // There is only one IP address reported:
+  QString output = QString::fromUtf8( m_proc->readAllStandardOutput(), -1 ).trimmed();
+  m_host->setIP(output);
+  emit ipAddress(m_host);
+}
+
+
 void Smb4KLookupIPAddressJob::slotStartLookup()
 {
-  // Find nmblookup program.
-  QString nmblookup = KStandardDirs::findExe( "nmblookup" );
-
-  if ( nmblookup.isEmpty() )
-  {
-    Smb4KNotification *notification = new Smb4KNotification();
-    notification->commandNotFound( "nmblookup" );
-    emitResult();
-    return;
-  }
-  else
-  {
-    // Go ahead
-  }
-
-  // Global Samba options
-  QMap<QString,QString> samba_options = globalSambaOptions();
-
-  // Compile the arguments
   QStringList arguments;
-  arguments << nmblookup;
-
-  // Domain
-  if ( !Smb4KSettings::domainName().isEmpty() &&
-       QString::compare( Smb4KSettings::domainName(), samba_options["workgroup"] ) != 0 )
-  {
-    arguments << QString( "-W %1" ).arg( KShell::quoteArg( Smb4KSettings::domainName() ) );
-  }
-  else
-  {
-    // Do nothing
-  }
-
-  // NetBIOS name
-  if ( !Smb4KSettings::netBIOSName().isEmpty() &&
-       QString::compare( Smb4KSettings::netBIOSName(), samba_options["netbios name"] ) != 0 )
-  {
-    arguments << QString( "-n %1" ).arg( KShell::quoteArg( Smb4KSettings::netBIOSName() ) );
-  }
-  else
-  {
-    // Do nothing
-  }
-
-  // NetBIOS scope
-  if ( !Smb4KSettings::netBIOSScope().isEmpty() &&
-       QString::compare( Smb4KSettings::netBIOSScope(), samba_options["netbios scope"] ) != 0 )
-  {
-    arguments << QString( "-i %1" ).arg( KShell::quoteArg( Smb4KSettings::netBIOSScope() ) );
-  }
-  else
-  {
-    // Do nothing
-  }
-
-  // Socket options
-  if ( !Smb4KSettings::socketOptions().isEmpty() &&
-       QString::compare( Smb4KSettings::socketOptions(), samba_options["socket options"] ) != 0 )
-  {
-    arguments << QString( "-O %1" ).arg( KShell::quoteArg( Smb4KSettings::socketOptions() ) );
-  }
-  else
-  {
-    // Do nothing
-  }
-
-  // Port 137
-  if ( Smb4KSettings::usePort137() )
-  {
-    arguments << "-r";
-  }
-  else
-  {
-    // Do nothing
-  }
-
-  // Broadcast address
-  QHostAddress address( Smb4KSettings::broadcastAddress() );
-
-  if ( !Smb4KSettings::broadcastAddress().isEmpty() &&
-       address.protocol() != QAbstractSocket::UnknownNetworkLayerProtocol )
-  {
-    arguments << QString( "-B %1" ).arg( Smb4KSettings::broadcastAddress() );
-  }
-  else
-  {
-    // Do nothing
-  }
-
-  // We do not use the WINS server here, because it emerged that using
-  // the WINS server is sometimes not very reliable. 
   
-  
-  if ( !winsServer().isEmpty() )
+  switch (Smb4KSettings::lookupIPs())
   {
-    arguments << "-R";
-    arguments << QString( "-U %1" ).arg( KShell::quoteArg( winsServer() ) );
+    case Smb4KSettings::EnumLookupIPs::nmblookup:
+    {
+      useNmblookup(arguments);
+      break;
+    }
+    case Smb4KSettings::EnumLookupIPs::net:
+    {
+      useNet(arguments);
+      break;
+    }
+    default:
+    {
+      break;
+    }
+  }
+  
+  if (!arguments.isEmpty())
+  {
+    m_proc = new Smb4KProcess(this);
+    m_proc->setOutputChannelMode(KProcess::SeparateChannels);
+    m_proc->setShellCommand(arguments.join(" "));
+    
+    connect(m_proc, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(slotProcessFinished(int,QProcess::ExitStatus)));
+    
+    m_proc->start();
   }
   else
   {
     // Do nothing
   }
-
-  arguments << "--";
-  arguments << KShell::quoteArg( m_host->hostName() );
-  
-  m_proc = new Smb4KProcess( this );
-  m_proc->setOutputChannelMode( KProcess::SeparateChannels );
-  m_proc->setShellCommand( arguments.join( " " ) );
-
-  connect( m_proc, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(slotProcessFinished(int,QProcess::ExitStatus)) );
-
-  m_proc->start();
 }
 
 
@@ -2902,8 +3346,7 @@ void Smb4KLookupIPAddressJob::slotProcessFinished( int /*exitCode*/, QProcess::E
     {
       if ( !m_proc->isAborted() )
       {
-        Smb4KNotification *notification = new Smb4KNotification();
-        notification->processError( m_proc->error() );
+        Smb4KNotification::processError(m_proc->error());
       }
       else
       {
@@ -2913,7 +3356,23 @@ void Smb4KLookupIPAddressJob::slotProcessFinished( int /*exitCode*/, QProcess::E
     }
     default:
     {
-      processIPAddress();
+      switch (Smb4KSettings::lookupIPs())
+      {
+        case Smb4KSettings::EnumLookupIPs::nmblookup:
+        {
+          processNmblookupOutput();
+          break;
+        }
+        case Smb4KSettings::EnumLookupIPs::net:
+        {
+          processNetOutput();
+          break;
+        }
+        default:
+        {
+          break;
+        }
+      }
       break;
     }
   }
