@@ -2,7 +2,7 @@
     smb4kshare  -  Smb4K's container class for information about a share.
                              -------------------
     begin                : Mo Jan 28 2008
-    copyright            : (C) 2008-2012 by Alexander Reinholdt
+    copyright            : (C) 2008-2015 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -40,10 +40,8 @@
 #include <kdebug.h>
 #include <kicon.h>
 #include <kiconloader.h>
+#include <kio/global.h>
 
-// system includes
-#include <unistd.h>
-#include <math.h>
 
 class Smb4KSharePrivate
 {
@@ -66,50 +64,50 @@ class Smb4KSharePrivate
 };
 
 
-Smb4KShare::Smb4KShare( const QString &host, const QString &name )
-: Smb4KBasicNetworkItem( Share ), d( new Smb4KSharePrivate )
+Smb4KShare::Smb4KShare(const QString &host, const QString &name)
+: Smb4KBasicNetworkItem(Share), d(new Smb4KSharePrivate)
 {
   d->typeString   = "Disk";
   d->inaccessible = false;
   d->foreign      = false;
   d->filesystem   = Unknown;
-  d->user         = KUser( getuid() );
-  d->group        = KUserGroup( getgid() );
+  d->user         = KUser(KUser::UseRealUserID);
+  d->group        = KUserGroup(KUser::UseRealUserID);
   d->totalSpace   = -1;
   d->freeSpace    = -1;
   d->usedSpace    = -1;
   d->mounted      = false;
-  setHostName( host );
-  setShareName( name );
+  setHostName(host);
+  setShareName(name);
   setShareIcon();
 }
 
 
-Smb4KShare::Smb4KShare( const QString &unc )
-: Smb4KBasicNetworkItem( Share ), d( new Smb4KSharePrivate )
+Smb4KShare::Smb4KShare(const QString &unc)
+: Smb4KBasicNetworkItem(Share), d(new Smb4KSharePrivate)
 {
   d->typeString   = "Disk";
   d->inaccessible = false;
   d->foreign      = false;
   d->filesystem   = Unknown;
-  d->user         = KUser( getuid() );
-  d->group        = KUserGroup( getgid() );
+  d->user         = KUser(KUser::UseRealUserID);
+  d->group        = KUserGroup(KUser::UseRealUserID);
   d->totalSpace   = -1;
   d->freeSpace    = -1;
   d->usedSpace    = -1;
   d->mounted      = false;
-  d->url.setUrl( unc, KUrl::TolerantMode );
-  d->url.setProtocol( "smb" );
+  d->url.setUrl(unc, KUrl::TolerantMode);
+  d->url.setProtocol("smb");
   setShareIcon();
 }
 
 
-Smb4KShare::Smb4KShare( const Smb4KShare &s )
-: Smb4KBasicNetworkItem( Share ), d( new Smb4KSharePrivate )
+Smb4KShare::Smb4KShare(const Smb4KShare &s)
+: Smb4KBasicNetworkItem(Share), d(new Smb4KSharePrivate)
 {
   *d = *s.d;
 
-  if ( icon().isNull() )
+  if (icon().isNull())
   {
     setShareIcon();
   }
@@ -122,19 +120,19 @@ Smb4KShare::Smb4KShare( const Smb4KShare &s )
 
 
 Smb4KShare::Smb4KShare()
-: Smb4KBasicNetworkItem( Share ), d( new Smb4KSharePrivate )
+: Smb4KBasicNetworkItem(Share), d(new Smb4KSharePrivate)
 {
   d->typeString   = "Disk";
   d->inaccessible = false;
   d->foreign      = false;
   d->filesystem   = Unknown;
-  d->user         = KUser( getuid() );
-  d->group        = KUserGroup( getgid() );
+  d->user         = KUser(KUser::UseRealUserID);
+  d->group        = KUserGroup(KUser::UseRealUserID);
   d->totalSpace   = -1;
   d->freeSpace    = -1;
   d->usedSpace    = -1;
   d->mounted      = false;
-  d->url.setProtocol( "smb" );
+  d->url.setProtocol("smb");
 }
 
 
@@ -143,31 +141,31 @@ Smb4KShare::~Smb4KShare()
 }
 
 
-void Smb4KShare::setShareName( const QString &name )
+void Smb4KShare::setShareName(const QString &name)
 {
-  if ( name.startsWith( '/' ) )
+  if (name.startsWith('/'))
   {
-    d->url.setPath( name.trimmed() );
+    d->url.setPath(name.trimmed());
   }
   else
   {
-    d->url.setPath( '/'+name.trimmed() );
+    d->url.setPath('/'+name.trimmed());
   }
   
-  d->url.setProtocol( "smb" );
+  d->url.setProtocol("smb");
 }
 
 
 QString Smb4KShare::shareName() const
 {
   // Since users might come up with very weird share names,
-  // we are careful and do not use QString::remove( "/" ), but
+  // we are careful and do not use QString::remove("/"), but
   // only remove preceding and trailing slashes.
   QString share_name = d->url.path(KUrl::RemoveTrailingSlash);
   
-  if ( share_name.startsWith( '/' ) )
+  if (share_name.startsWith('/'))
   {
-    share_name = share_name.remove( 0, 1 );
+    share_name = share_name.remove(0, 1);
   }
   else
   {
@@ -178,10 +176,10 @@ QString Smb4KShare::shareName() const
 }
 
 
-void Smb4KShare::setHostName( const QString &hostName )
+void Smb4KShare::setHostName(const QString &hostName)
 {
-  d->url.setHost( hostName.trimmed() );
-  d->url.setProtocol( "smb" );
+  d->url.setHost(hostName.trimmed());
+  d->url.setProtocol("smb");
 }
 
 
@@ -195,9 +193,9 @@ QString Smb4KShare::unc() const
 {
   QString unc;
   
-  if ( !hostName().isEmpty() && !shareName().isEmpty() )
+  if (!hostName().isEmpty() && !shareName().isEmpty())
   {
-    unc = QString( "//%1/%2" ).arg( hostName() ).arg( shareName() );
+    unc = QString("//%1/%2").arg(hostName()).arg(shareName());
   }
   else
   {
@@ -212,9 +210,9 @@ QString Smb4KShare::homeUNC() const
 {
   QString unc;
 
-  if ( isHomesShare() && !hostName().isEmpty() && !d->url.userName().isEmpty() )
+  if (isHomesShare() && !hostName().isEmpty() && !d->url.userName().isEmpty())
   {
-    unc = QString( "//%1/%2" ).arg( hostName() ).arg( d->url.userName() );
+    unc = QString("//%1/%2").arg(hostName()).arg(d->url.userName());
   }
   else
   {
@@ -225,10 +223,10 @@ QString Smb4KShare::homeUNC() const
 }
 
 
-void Smb4KShare::setURL( const KUrl &url )
+void Smb4KShare::setURL(const KUrl &url)
 {
   // Check validity.
-  if ( !url.isValid() )
+  if (!url.isValid())
   {
     return;
   }
@@ -238,7 +236,7 @@ void Smb4KShare::setURL( const KUrl &url )
   }
 
   // Check protocol
-  if ( url.protocol().isEmpty() || QString::compare( url.protocol(), "smb" ) == 0 )
+  if (url.protocol().isEmpty() || QString::compare(url.protocol(), "smb") == 0)
   {
     // Do nothing
   }
@@ -248,7 +246,7 @@ void Smb4KShare::setURL( const KUrl &url )
   }
   
   // Check that the share name is present
-  if ( !url.hasPath() || url.path(KUrl::RemoveTrailingSlash).endsWith('/') )
+  if (!url.hasPath() || url.path(KUrl::RemoveTrailingSlash).endsWith('/'))
   {
     return;
   }
@@ -261,16 +259,16 @@ void Smb4KShare::setURL( const KUrl &url )
   d->url = url;
   
   // Force the protocol
-  d->url.setProtocol( "smb" );
+  d->url.setProtocol("smb");
 }
 
 
-void Smb4KShare::setURL( const QString &url )
+void Smb4KShare::setURL(const QString &url)
 {
   KUrl u;
-  u.setUrl( url, KUrl::TolerantMode );
+  u.setUrl(url, KUrl::TolerantMode);
 
-  setURL( u );
+  setURL(u);
 }
 
 
@@ -284,10 +282,10 @@ KUrl Smb4KShare::homeURL() const
 {
   KUrl url;
   
-  if ( isHomesShare() && !d->url.userName().isEmpty() )
+  if (isHomesShare() && !d->url.userName().isEmpty())
   {
     url = d->url;
-    url.setPath( d->url.userName() );
+    url.setPath(d->url.userName());
   }
   else
   {
@@ -302,9 +300,9 @@ QString Smb4KShare::hostUNC() const
 {
   QString unc;
   
-  if ( !hostName().isEmpty() )
+  if (!hostName().isEmpty())
   {
-    unc = QString( "//%1" ).arg( hostName() );
+    unc = QString("//%1").arg(hostName());
   }
   else
   {
@@ -315,7 +313,7 @@ QString Smb4KShare::hostUNC() const
 }
 
 
-void Smb4KShare::setWorkgroupName( const QString &workgroup )
+void Smb4KShare::setWorkgroupName(const QString &workgroup)
 {
   d->workgroup = workgroup;
 }
@@ -327,7 +325,7 @@ QString Smb4KShare::workgroupName() const
 }
 
 
-void Smb4KShare::setTypeString( const QString &typeString )
+void Smb4KShare::setTypeString(const QString &typeString)
 {
   d->typeString = typeString;
   setShareIcon();
@@ -342,14 +340,14 @@ QString Smb4KShare::typeString() const
 
 QString Smb4KShare::translatedTypeString() const
 {
-  if ( QString::compare( d->typeString, "Disk" ) == 0 )
+  if (QString::compare(d->typeString, "Disk") == 0)
   {
-    return i18n( "Disk" );
+    return i18n("Disk");
   }
-  else if ( QString::compare( d->typeString, "Print" ) == 0 ||
-            QString::compare( d->typeString, "Printer" ) == 0 )
+  else if (QString::compare(d->typeString, "Print") == 0 ||
+            QString::compare(d->typeString, "Printer") == 0)
   {
-    return i18n( "Printer" );
+    return i18n("Printer");
   }
   else
   {
@@ -360,7 +358,7 @@ QString Smb4KShare::translatedTypeString() const
 }
 
 
-void Smb4KShare::setComment( const QString &comment )
+void Smb4KShare::setComment(const QString &comment)
 {
   d->comment = comment;
 }
@@ -372,9 +370,9 @@ QString Smb4KShare::comment() const
 }
 
 
-void Smb4KShare::setHostIP( const QString &ip )
+void Smb4KShare::setHostIP(const QString &ip)
 {
-  d->ip.setAddress( ip );
+  d->ip.setAddress(ip);
 }
 
 
@@ -386,13 +384,13 @@ QString Smb4KShare::hostIP() const
 
 bool Smb4KShare::isHidden() const
 {
-  return d->url.path().endsWith( '$' );
+  return d->url.path().endsWith('$');
 }
 
 
 bool Smb4KShare::isPrinter() const
 {
-  if ( d->inaccessible || d->typeString.isEmpty() )
+  if (d->inaccessible || d->typeString.isEmpty())
   {
     return false;
   }
@@ -401,24 +399,12 @@ bool Smb4KShare::isPrinter() const
     // Do nothing
   }
   
-  return (QString::compare( d->typeString, "Print" ) == 0 ||
-          QString::compare( d->typeString, "Printer" ) == 0);
+  return (QString::compare(d->typeString, "Print") == 0 ||
+          QString::compare(d->typeString, "Printer") == 0);
 }
 
 
-bool Smb4KShare::isIPC() const
-{
-  return (QString::compare( d->url.path(), "IPC$" ) == 0);
-}
-
-
-bool Smb4KShare::isADMIN() const
-{
-  return (QString::compare( d->url.path(), "ADMIN$" ) == 0);
-}
-
-
-void Smb4KShare::setPath( const QString &mountpoint )
+void Smb4KShare::setPath(const QString &mountpoint)
 {
   d->path = mountpoint;
 }
@@ -432,11 +418,11 @@ QString Smb4KShare::path() const
 
 QString Smb4KShare::canonicalPath() const
 {
-  return (d->inaccessible ? d->path : QDir( d->path ).canonicalPath());
+  return (d->inaccessible ? d->path : QDir(d->path).canonicalPath());
 }
 
 
-void Smb4KShare::setInaccessible( bool in )
+void Smb4KShare::setInaccessible(bool in)
 {
   d->inaccessible = in;
   setShareIcon();
@@ -449,7 +435,7 @@ bool Smb4KShare::isInaccessible() const
 }
 
 
-void Smb4KShare::setForeign( bool foreign )
+void Smb4KShare::setForeign(bool foreign)
 {
   d->foreign = foreign;
   setShareIcon();
@@ -462,7 +448,7 @@ bool Smb4KShare::isForeign() const
 }
 
 
-void Smb4KShare::setFileSystem( FileSystem filesystem )
+void Smb4KShare::setFileSystem(FileSystem filesystem)
 {
   d->filesystem = filesystem;
 }
@@ -470,13 +456,13 @@ void Smb4KShare::setFileSystem( FileSystem filesystem )
 
 Smb4KShare::FileSystem Smb4KShare::fileSystem() const
 {
-  return static_cast<FileSystem>( d->filesystem );
+  return static_cast<FileSystem>(d->filesystem);
 }
 
 
 QString Smb4KShare::fileSystemString() const
 {
-  switch ( d->filesystem )
+  switch (d->filesystem)
   {
     case CIFS:
     {
@@ -496,9 +482,9 @@ QString Smb4KShare::fileSystemString() const
 }
 
 
-void Smb4KShare::setUID( K_UID uid )
+void Smb4KShare::setUID(K_UID uid)
 {
-  d->user = KUser( uid );
+  d->user = KUser(uid);
 }
 
 
@@ -514,9 +500,9 @@ QString Smb4KShare::owner() const
 }
 
 
-void Smb4KShare::setGID( K_GID gid )
+void Smb4KShare::setGID(K_GID gid)
 {
-  d->group = KUserGroup( gid );
+  d->group = KUserGroup(gid);
 }
 
 
@@ -532,9 +518,9 @@ QString Smb4KShare::group() const
 }
 
 
-void Smb4KShare::setIsMounted( bool mounted )
+void Smb4KShare::setIsMounted(bool mounted)
 {
-  if ( !isPrinter() )
+  if (!isPrinter())
   {
     d->mounted = mounted;
     setShareIcon();
@@ -552,7 +538,7 @@ bool Smb4KShare::isMounted() const
 }
 
 
-void Smb4KShare::setTotalDiskSpace( qulonglong size )
+void Smb4KShare::setTotalDiskSpace(qulonglong size)
 {
   d->totalSpace = size;
 }
@@ -566,56 +552,11 @@ qulonglong Smb4KShare::totalDiskSpace() const
 
 QString Smb4KShare::totalDiskSpaceString() const
 {
-  QString total, total_dim;
-
-  int exponent = 0;
-  double tmp_factor = 0;
-  qulonglong factor = 0;
-
-  (void) frexp( d->totalSpace * 1024, &exponent );
-  (void) modf( (exponent - 10) / 10, &tmp_factor );
-  factor = tmp_factor;
-  qreal tmp_total = d->totalSpace / pow( 1024, factor );
-  total = QString( "%1" ).arg( tmp_total, 0, 'f', 1 );
-
-  switch ( factor )
-  {
-    case 0:
-    {
-      total_dim = 'B';
-      break;
-    }
-    case 1:
-    {
-      total_dim = "KiB";
-      break;
-    }
-    case 2:
-    {
-      total_dim = "MiB";
-      break;
-    }
-    case 3:
-    {
-      total_dim = "GiB";
-      break;
-    }
-    case 4:
-    {
-      total_dim = "TiB";
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
-
-  return total+' '+total_dim;
+  return KIO::convertSize(d->totalSpace);
 }
 
 
-void Smb4KShare::setFreeDiskSpace( qulonglong size )
+void Smb4KShare::setFreeDiskSpace(qulonglong size)
 {
   d->freeSpace = size;
 }
@@ -629,56 +570,11 @@ qulonglong Smb4KShare::freeDiskSpace() const
 
 QString Smb4KShare::freeDiskSpaceString() const
 {
-  QString free, free_dim;
-
-  int exponent = 0;
-  double tmp_factor = 0;
-  qulonglong factor = 0;
-
-  (void) frexp( d->freeSpace * 1024, &exponent );
-  (void) modf( (exponent - 10) / 10, &tmp_factor );
-  factor = tmp_factor;
-  qreal tmp_free = d->freeSpace / pow( 1024, factor );
-  free = QString( "%1" ).arg( tmp_free, 0, 'f', 1 );
-
-  switch ( factor )
-  {
-    case 0:
-    {
-      free_dim = 'B';
-      break;
-    }
-    case 1:
-    {
-      free_dim = "KiB";
-      break;
-    }
-    case 2:
-    {
-      free_dim = "MiB";
-      break;
-    }
-    case 3:
-    {
-      free_dim = "GiB";
-      break;
-    }
-    case 4:
-    {
-      free_dim = "TiB";
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
-
-  return free+' '+free_dim;
+  return KIO::convertSize(d->freeSpace);
 }
 
 
-void Smb4KShare::setUsedDiskSpace( qulonglong size )
+void Smb4KShare::setUsedDiskSpace(qulonglong size)
 {
   d->usedSpace = size;
 }
@@ -692,61 +588,16 @@ qulonglong Smb4KShare::usedDiskSpace() const
 
 QString Smb4KShare::usedDiskSpaceString() const
 {
-  QString used, used_dim;
-
-  int exponent = 0;
-  double tmp_factor = 0;
-  qulonglong factor = 0;
-
-  (void) frexp( d->usedSpace * 1024, &exponent );
-  (void) modf( (exponent - 10) / 10, &tmp_factor );
-  factor = tmp_factor;
-  qreal tmp_used = d->usedSpace / pow( 1024, factor );
-  used = QString( "%1" ).arg( tmp_used, 0, 'f', 1 );
-
-  switch ( factor )
-  {
-    case 0:
-    {
-      used_dim = 'B';
-      break;
-    }
-    case 1:
-    {
-      used_dim = "KiB";
-      break;
-    }
-    case 2:
-    {
-      used_dim = "MiB";
-      break;
-    }
-    case 3:
-    {
-      used_dim = "GiB";
-      break;
-    }
-    case 4:
-    {
-      used_dim = "TiB";
-      break;
-    }
-    default:
-    {
-      break;
-    }
-  }
-
-  return used+' '+used_dim;
+  return KIO::convertSize(d->usedSpace);
 }
 
 
 qreal Smb4KShare::diskUsage() const
 {
-  qreal used( usedDiskSpace() );
-  qreal total( totalDiskSpace() );
+  qreal used(usedDiskSpace());
+  qreal total(totalDiskSpace());
 
-  if ( total > 0 )
+  if (total > 0)
   {
     return used * 100 / total;
   }
@@ -757,20 +608,20 @@ qreal Smb4KShare::diskUsage() const
 
 QString Smb4KShare::diskUsageString() const
 {
-  return QString( "%1 %" ).arg( diskUsage(), 0, 'f', 1 );
+  return QString("%1 %").arg(diskUsage(), 0, 'f', 1);
 }
 
 
-bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
+bool Smb4KShare::equals(Smb4KShare *share, CheckFlags flag) const
 {
-  Q_ASSERT( share );
+  Q_ASSERT(share);
 
-  switch ( flag )
+  switch (flag)
   {
     case Full:
     {
       // UNC
-      if ( QString::compare( unc(), share->unc(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(unc(), share->unc(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -780,7 +631,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Login
-      if ( QString::compare( login(), share->login() ) != 0 )
+      if (QString::compare(login(), share->login()) != 0)
       {
         return false;
       }
@@ -790,7 +641,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Workgroup name
-      if ( QString::compare( workgroupName(), share->workgroupName(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(workgroupName(), share->workgroupName(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -800,7 +651,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Type
-      if ( QString::compare( typeString(), share->typeString() ) != 0 )
+      if (QString::compare(typeString(), share->typeString()) != 0)
       {
         return false;
       }
@@ -810,7 +661,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Comment
-      if ( QString::compare( comment(), share->comment() ) != 0 )
+      if (QString::compare(comment(), share->comment()) != 0)
       {
         return false;
       }
@@ -820,7 +671,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // IP address
-      if ( QString::compare( hostIP(), share->hostIP() ) != 0 )
+      if (QString::compare(hostIP(), share->hostIP()) != 0)
       {
         return false;
       }
@@ -830,7 +681,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Path
-      if ( QString::compare( path(), share->path() ) != 0 )
+      if (QString::compare(path(), share->path()) != 0)
       {
         return false;
       }
@@ -840,7 +691,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Accessibility?
-      if ( isInaccessible() != share->isInaccessible() )
+      if (isInaccessible() != share->isInaccessible())
       {
         return false;
       }
@@ -850,7 +701,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Foreignness
-      if ( isForeign() != share->isForeign() )
+      if (isForeign() != share->isForeign())
       {
         return false;
       }
@@ -860,7 +711,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // File system
-      if ( fileSystem() != share->fileSystem() )
+      if (fileSystem() != share->fileSystem())
       {
         return false;
       }
@@ -870,7 +721,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // UID
-      if ( uid() != share->uid() )
+      if (uid() != share->uid())
       {
         return false;
       }
@@ -880,7 +731,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // GID
-      if ( gid() != share->gid() )
+      if (gid() != share->gid())
       {
         return false;
       }
@@ -890,7 +741,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Total space
-      if ( totalDiskSpace() != share->totalDiskSpace() )
+      if (totalDiskSpace() != share->totalDiskSpace())
       {
         return false;
       }
@@ -900,7 +751,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Used space
-      if ( usedDiskSpace() != share->usedDiskSpace() )
+      if (usedDiskSpace() != share->usedDiskSpace())
       {
         return false;
       }
@@ -910,7 +761,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Mounted
-      if ( isMounted() != share->isMounted() )
+      if (isMounted() != share->isMounted())
       {
         return false;
       }
@@ -924,7 +775,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
     case NetworkOnly:
     {
       // UNC
-      if ( QString::compare( unc(), share->unc(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(unc(), share->unc(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -934,7 +785,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Login
-      if ( QString::compare( login(), share->login() ) != 0 )
+      if (QString::compare(login(), share->login()) != 0)
       {
         return false;
       }
@@ -944,7 +795,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Workgroup name
-      if ( QString::compare( workgroupName(), share->workgroupName(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(workgroupName(), share->workgroupName(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -954,7 +805,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Type
-      if ( QString::compare( typeString(), share->typeString() ) != 0 )
+      if (QString::compare(typeString(), share->typeString()) != 0)
       {
         return false;
       }
@@ -964,7 +815,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Comment
-      if ( QString::compare( comment(), share->comment() ) != 0 )
+      if (QString::compare(comment(), share->comment()) != 0)
       {
         return false;
       }
@@ -974,7 +825,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // IP address
-      if ( QString::compare( hostIP(), share->hostIP() ) != 0 )
+      if (QString::compare(hostIP(), share->hostIP()) != 0)
       {
         return false;
       }
@@ -988,7 +839,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
     case MinimalNetworkOnly:
     {
       // UNC
-      if ( QString::compare( unc(), share->unc(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(unc(), share->unc(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -998,7 +849,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Workgroup name
-      if ( QString::compare( workgroupName(), share->workgroupName(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(workgroupName(), share->workgroupName(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -1008,7 +859,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Type
-      if ( QString::compare( typeString(), share->typeString() ) != 0 )
+      if (QString::compare(typeString(), share->typeString()) != 0)
       {
         return false;
       }
@@ -1022,7 +873,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
     case LocalOnly:
     {
       // UNC
-      if ( QString::compare( unc(), share->unc(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(unc(), share->unc(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -1032,7 +883,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Login
-      if ( QString::compare( login(), share->login() ) != 0 )
+      if (QString::compare(login(), share->login()) != 0)
       {
         return false;
       }
@@ -1042,7 +893,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
       
       // Path
-      if ( QString::compare( path(), share->path() ) != 0 )
+      if (QString::compare(path(), share->path()) != 0)
       {
         return false;
       }
@@ -1052,7 +903,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Accessibility?
-      if ( isInaccessible() != share->isInaccessible() )
+      if (isInaccessible() != share->isInaccessible())
       {
         return false;
       }
@@ -1062,7 +913,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Foreignness
-      if ( isForeign() != share->isForeign() )
+      if (isForeign() != share->isForeign())
       {
         return false;
       }
@@ -1072,7 +923,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // File system
-      if ( fileSystem() != share->fileSystem() )
+      if (fileSystem() != share->fileSystem())
       {
         return false;
       }
@@ -1082,7 +933,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // UID
-      if ( uid() != share->uid() )
+      if (uid() != share->uid())
       {
         return false;
       }
@@ -1092,7 +943,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // GID
-      if ( gid() != share->gid() )
+      if (gid() != share->gid())
       {
         return false;
       }
@@ -1102,7 +953,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Total space
-      if ( totalDiskSpace() != share->totalDiskSpace() )
+      if (totalDiskSpace() != share->totalDiskSpace())
       {
         return false;
       }
@@ -1112,7 +963,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Used space
-      if ( usedDiskSpace() != share->usedDiskSpace() )
+      if (usedDiskSpace() != share->usedDiskSpace())
       {
         return false;
       }
@@ -1122,7 +973,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Mounted
-      if ( isMounted() != share->isMounted() )
+      if (isMounted() != share->isMounted())
       {
         return false;
       }
@@ -1136,7 +987,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
     case MinimalLocalOnly:
     {
       // UNC
-      if ( QString::compare( unc(), share->unc(), Qt::CaseInsensitive ) != 0 )
+      if (QString::compare(unc(), share->unc(), Qt::CaseInsensitive) != 0)
       {
         return false;
       }
@@ -1146,7 +997,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Path
-      if ( QString::compare( path(), share->path() ) != 0 )
+      if (QString::compare(path(), share->path()) != 0)
       {
         return false;
       }
@@ -1156,7 +1007,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // File system
-      if ( fileSystem() != share->fileSystem() )
+      if (fileSystem() != share->fileSystem())
       {
         return false;
       }
@@ -1166,7 +1017,7 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
       }
 
       // Mounted
-      if ( isMounted() != share->isMounted() )
+      if (isMounted() != share->isMounted())
       {
         return false;
       }
@@ -1187,58 +1038,58 @@ bool Smb4KShare::equals( Smb4KShare *share, CheckFlags flag ) const
 }
 
 
-bool Smb4KShare::isEmpty( CheckFlags flag ) const
+bool Smb4KShare::isEmpty(CheckFlags flag) const
 {
-  switch ( flag )
+  switch (flag)
   {
     case Full:
     {
-      if ( !d->url.isEmpty() )
+      if (!d->url.isEmpty())
       {
         return false;
       }
 
-      if ( !d->workgroup.isEmpty() )
+      if (!d->workgroup.isEmpty())
       {
         return false;
       }
 
-      if ( !d->typeString.isEmpty() )
+      if (!d->typeString.isEmpty())
       {
         return false;
       }
 
-      if ( !d->comment.isEmpty() )
+      if (!d->comment.isEmpty())
       {
         return false;
       }
 
-      if ( !d->ip.isNull() )
+      if (!d->ip.isNull())
       {
         return false;
       }
 
-      if ( !d->path.isEmpty() )
+      if (!d->path.isEmpty())
       {
         return false;
       }
 
-      if ( d->filesystem != Unknown )
+      if (d->filesystem != Unknown)
       {
         return false;
       }
 
-      if ( d->totalSpace > 0 )
+      if (d->totalSpace > 0)
       {
         return false;
       }
 
-      if ( d->freeSpace > 0 )
+      if (d->freeSpace > 0)
       {
         return false;
       }
       
-      if ( d->usedSpace > 0 )
+      if (d->usedSpace > 0)
       {
         return false;
       }
@@ -1247,27 +1098,27 @@ bool Smb4KShare::isEmpty( CheckFlags flag ) const
     }
     case NetworkOnly:
     {
-      if ( !d->url.isEmpty() )
+      if (!d->url.isEmpty())
       {
         return false;
       }
 
-      if ( !d->workgroup.isEmpty() )
+      if (!d->workgroup.isEmpty())
       {
         return false;
       }
 
-      if ( !d->typeString.isEmpty() )
+      if (!d->typeString.isEmpty())
       {
         return false;
       }
 
-      if ( !d->comment.isEmpty() )
+      if (!d->comment.isEmpty())
       {
         return false;
       }
 
-      if ( !d->ip.isNull() )
+      if (!d->ip.isNull())
       {
         return false;
       }
@@ -1276,27 +1127,27 @@ bool Smb4KShare::isEmpty( CheckFlags flag ) const
     }
     case LocalOnly:
     {
-      if ( !d->path.isEmpty() )
+      if (!d->path.isEmpty())
       {
         return false;
       }
 
-      if ( d->filesystem != Unknown )
+      if (d->filesystem != Unknown)
       {
         return false;
       }
 
-      if ( d->totalSpace > 0 )
+      if (d->totalSpace > 0)
       {
         return false;
       }
 
-      if ( d->freeSpace > 0 )
+      if (d->freeSpace > 0)
       {
         return false;
       }
       
-      if ( d->usedSpace > 0 )
+      if (d->usedSpace > 0)
       {
         return false;
       }
@@ -1313,18 +1164,18 @@ bool Smb4KShare::isEmpty( CheckFlags flag ) const
 }
 
 
-void Smb4KShare::setMountData( Smb4KShare *share )
+void Smb4KShare::setMountData(Smb4KShare *share)
 {
-  Q_ASSERT( share );
+  Q_ASSERT(share);
 
-  if ( equals( share, MinimalNetworkOnly ) )
+  if (equals(share, MinimalNetworkOnly))
   {
     d->path         = share->path();
     d->inaccessible = share->isInaccessible();
     d->foreign      = share->isForeign();
     d->filesystem   = share->fileSystem();
-    d->user         = KUser( share->uid() );
-    d->group        = KUserGroup( share->gid() );
+    d->user         = KUser(share->uid());
+    d->group        = KUserGroup(share->gid());
     d->totalSpace   = share->totalDiskSpace();
     d->freeSpace    = share->freeDiskSpace();
     d->usedSpace    = share->usedDiskSpace();
@@ -1347,8 +1198,8 @@ void Smb4KShare::resetMountData()
   d->inaccessible = false;
   d->foreign      = false;
   d->filesystem   = Unknown;
-  d->user         = KUser( getuid() );
-  d->group        = KUserGroup( getgid() );
+  d->user         = KUser(KUser::UseRealUserID);
+  d->group        = KUserGroup(KUser::UseRealUserID);
   d->totalSpace   = -1;
   d->freeSpace    = -1;
   d->usedSpace    = -1;
@@ -1360,13 +1211,13 @@ void Smb4KShare::resetMountData()
 
 bool Smb4KShare::isHomesShare() const
 {
-  return d->url.path().endsWith( QLatin1String( "homes" ) );
+  return d->url.path().endsWith(QLatin1String("homes"));
 }
 
 
-void Smb4KShare::setPort( int port )
+void Smb4KShare::setPort(int port)
 {
-  d->url.setPort( port );
+  d->url.setPort(port);
 }
 
 
@@ -1376,14 +1227,14 @@ int Smb4KShare::port() const
 }
 
 
-void Smb4KShare::setAuthInfo( Smb4KAuthInfo *authInfo )
+void Smb4KShare::setAuthInfo(Smb4KAuthInfo *authInfo)
 {
   // Avoid that the login is overwritten with an empty 
   // string if we have a homes share.
-  if ( !isHomesShare() || !authInfo->login().isEmpty() )
+  if (!isHomesShare() || !authInfo->userName().isEmpty())
   {
-    d->url.setUserName( authInfo->login() );
-    d->url.setPassword( authInfo->password() );
+    d->url.setUserName(authInfo->userName());
+    d->url.setPassword(authInfo->password());
   }
   else
   {
@@ -1392,13 +1243,13 @@ void Smb4KShare::setAuthInfo( Smb4KAuthInfo *authInfo )
 }
 
 
-void Smb4KShare::setLogin( const QString &login )
+void Smb4KShare::setLogin(const QString &login)
 {
   // Avoid that the login is overwritten with an empty 
   // string if we have a homes share.
-  if ( !isHomesShare() || !login.isEmpty() )
+  if (!isHomesShare() || !login.isEmpty())
   {
-    d->url.setUserName( login );
+    d->url.setUserName(login);
   }
   else
   {
@@ -1413,13 +1264,13 @@ QString Smb4KShare::login() const
 }
 
 
-void Smb4KShare::setPassword( const QString &passwd )
+void Smb4KShare::setPassword(const QString &passwd)
 {
   // Avoid that the password is overwritten with an empty 
   // string if we have a homes share.
-  if ( !isHomesShare() || !passwd.isEmpty() )
+  if (!isHomesShare() || !passwd.isEmpty())
   {
-    d->url.setPassword( passwd );
+    d->url.setPassword(passwd);
   }
   else
   {
@@ -1444,12 +1295,12 @@ void Smb4KShare::setShareIcon()
 {
   // We have three base icons: The remote folder, the locked folder
   // and the printer icon.
-  if ( !isPrinter() )
+  if (!isPrinter())
   {
     // Overlays
     QStringList overlays;
     
-    if ( isMounted() )
+    if (isMounted())
     {
       overlays << "emblem-mounted";
     }
@@ -1458,33 +1309,47 @@ void Smb4KShare::setShareIcon()
       overlays << "";
     }
 
-    if ( isForeign() )
+    if (isForeign())
     {
       overlays << "";
       overlays << "flag-red";
+      // FIXME (Smb4K 2.0): Use 'view-media-artist' instead of 'flag-red'.
+//       overlays << "view-media-artist";
     }
     else
     {
       // Do nothing
     }
+
+    // The icon
+    KIcon icon;
     
-    // Icon name
-    QString icon_name;
-    
-    if ( !isInaccessible() )
+    if (!isInaccessible())
     {
-      icon_name = "folder-remote";
+      icon = KIcon("folder-remote", KIconLoader::global(), overlays);
+      
+      // FIXME (Smb4K 2.0): Use 'folder-network' instead of 'folder-remote'.
+//       icon = KIcon("folder-network", KIconLoader::global(), overlays);
+//       
+//       if (icon.isNull())
+//       {
+//         icon = KIcon("folder-remote", KIconLoader::global(), overlays);
+//       }
+//       else
+//       {
+//         // Nichts tun
+//       }
     }
     else
     {
-      icon_name = "folder-locked";
+      icon = KIcon("folder-locked", KIconLoader::global(), overlays);
     }
     
-    setIcon( KIcon( icon_name, KIconLoader::global(), overlays ) );
+    setIcon(icon);
   }
   else
   {
-    setIcon( KIcon( "printer" ) );
+    setIcon(KIcon("printer"));
   }
 }
 
