@@ -2,7 +2,7 @@
     smb4kmounthelper  -  The helper that mounts and unmounts shares.
                              -------------------
     begin                : Sa Okt 16 2010
-    copyright            : (C) 2010-2013 by Alexander Reinholdt
+    copyright            : (C) 2010-2015 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -46,71 +46,76 @@
 KDE4_AUTH_HELPER_MAIN( "net.sourceforge.smb4k.mounthelper", Smb4KMountHelper )
 
 
-ActionReply Smb4KMountHelper::mount( const QVariantMap &args )
+ActionReply Smb4KMountHelper::mount(const QVariantMap &args)
 {
   ActionReply reply;
-  reply.addData( "url", args["url"] );
-  reply.addData( "workgroup", args["workgroup"] );
-  reply.addData( "comment", args["comment"] );
-  reply.addData( "ip", args["ip"] );
-  reply.addData( "mountpoint", args["mountpoint"] );
-  reply.addData( "key", args["key"] );
+  // The mountpoint is a unique and can be used to
+  // find the share.
+  reply.addData("mh_mountpoint", args["mh_mountpoint"]);
   
-  KProcess proc( this );
-  proc.setOutputChannelMode( KProcess::SeparateChannels );
-  proc.setProcessEnvironment( QProcessEnvironment::systemEnvironment() );
-#ifndef Q_OS_FREEBSD
-  proc.setEnv( "PASSWD", args["url"].toUrl().password(), true );
-#else
+  KProcess proc(this);
+  proc.setOutputChannelMode(KProcess::SeparateChannels);
+  proc.setProcessEnvironment(QProcessEnvironment::systemEnvironment());
+#if defined(Q_OS_LINUX)
+  proc.setEnv("PASSWD", args["mh_url"].toUrl().password(), true);
+#endif
   // We need this to avoid a translated password prompt.
   proc.setEnv("LANG", "C");
-#endif
-  // The $HOME environment variable is needed under FreeBSD to
-  // point the mount process to the right ~/.nsmbrc file. Under
-  // Linux it is not needed.
-  proc.setEnv( "HOME", args["home_dir"].toString() );
   // If the location of a Kerberos ticket is passed, it needs to
   // be passed to the process environment here.
-  if ( args.contains( "krb_ticket" ) )
+  if (args.contains("mh_krb5ticket"))
   {
-    proc.setEnv( "KRB5CCNAME", args["krb5_ticket"].toString() );
+    proc.setEnv("KRB5CCNAME", args["mh_krb5ticket"].toString());
   }
   else
   {
     // Do nothing
   }
-  // Set the program.
-  proc.setProgram( args["command"].toStringList() );
+  
+  // Set the mount command here.
+  QStringList command;
+#if defined(Q_OS_LINUX)
+  command << args["mh_command"].toString();
+  command << args["mh_unc"].toString();
+  command << args["mh_mountpoint"].toString();
+  command << args["mh_options"].toStringList();
+#elif defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD)
+  command << args["mh_command"].toString();
+  command << args["mh_options"].toStringList();
+  command << args["mh_unc"].toString();
+  command << args["mh_mountpoint"].toString();
+#else
+  // Do nothing.
+#endif
+  proc.setProgram(command);
 
   // Run the mount process.
   proc.start();
 
-  if ( proc.waitForStarted( -1 ) )
+  if (proc.waitForStarted(-1))
   {
     bool user_kill = false;
 
-    while ( !proc.waitForFinished( 10 ) )
+    while (!proc.waitForFinished(10))
     {
-#ifdef Q_OS_FREEBSD
       // Check if there is a password prompt. If there is one, pass
       // the password to it.
       QByteArray out = proc.readAllStandardError();
       
-      if ( out.startsWith(QLatin1String("Password:")) )
+      if (out.startsWith("Password:"))
       {
-        proc.write(args["url"].toUrl().password().toUtf8());
+        proc.write(args["mh_url"].toUrl().password().toUtf8());
         proc.write("\r");
       }
       else
       {
         // Do nothing
       }
-#endif
 
       // We want to be able to terminate the process from outside.
       // Thus, we implement a loop that checks periodically, if we
       // need to kill the process.
-      if ( HelperSupport::isStopped() )
+      if (HelperSupport::isStopped())
       {
         proc.kill();
         user_kill = true;
@@ -122,12 +127,12 @@ ActionReply Smb4KMountHelper::mount( const QVariantMap &args )
       }
     }
 
-    if ( proc.exitStatus() == KProcess::CrashExit )
+    if (proc.exitStatus() == KProcess::CrashExit)
     {
-      if ( !user_kill )
+      if (!user_kill)
       {
-        reply.setErrorCode( ActionReply::HelperError );
-        reply.setErrorDescription( i18n( "The mount process crashed." ) );
+        reply.setErrorCode(ActionReply::HelperError);
+        reply.setErrorDescription(i18n("The mount process crashed."));
         return reply;
       }
       else
@@ -138,18 +143,14 @@ ActionReply Smb4KMountHelper::mount( const QVariantMap &args )
     else
     {
       // Check if there is output on stderr.
-      QString stderr = QString::fromUtf8( proc.readAllStandardError() );
-      reply.addData( "stderr", stderr.trimmed() );
-
-      // Check if there is output on stdout.
-      QString stdout = QString::fromUtf8( proc.readAllStandardOutput() );
-      reply.addData( "stdout", stdout.trimmed() );
+      QString stdErr = QString::fromUtf8(proc.readAllStandardError());
+      reply.addData("mh_error_message", stdErr.trimmed());
     }
   }
   else
   {
-    reply.setErrorCode( ActionReply::HelperError );
-    reply.setErrorDescription( i18n( "The mount process could not be started." ) );
+    reply.setErrorCode(ActionReply::HelperError);
+    reply.setErrorDescription(i18n("The mount process could not be started."));
   }
 
   return reply;
@@ -157,28 +158,28 @@ ActionReply Smb4KMountHelper::mount( const QVariantMap &args )
 
 
 
-ActionReply Smb4KMountHelper::unmount( const QVariantMap &args )
+ActionReply Smb4KMountHelper::unmount(const QVariantMap &args)
 {
   ActionReply reply;
-  reply.addData( "url", args["url"] );
-  reply.addData( "mountpoint", args["mountpoint"] );
-  reply.addData( "key", args["key"] );
+  // The mountpoint is a unique and can be used to
+  // find the share.
+  reply.addData("mh_mountpoint", args["mh_mountpoint"]);
 
   // Check if the mountpoint is valid and the file system is
   // also correct.
   bool mountpoint_ok = false;
-  KMountPoint::List mountpoints = KMountPoint::currentMountPoints( KMountPoint::BasicInfoNeeded|KMountPoint::NeedMountOptions );
+  KMountPoint::List mountpoints = KMountPoint::currentMountPoints(KMountPoint::BasicInfoNeeded|KMountPoint::NeedMountOptions);
 
-  for( int j = 0; j < mountpoints.size(); j++ )
+  for(int j = 0; j < mountpoints.size(); j++)
   {
-#ifndef Q_OS_FREEBSD
-    if ( QString::compare( args["mountpoint"].toString(),
-                           mountpoints.at( j )->mountPoint(), Qt::CaseInsensitive ) == 0 &&
-         QString::compare( mountpoints.at( j )->mountType(), "cifs", Qt::CaseInsensitive ) == 0 )
+#ifdef Q_OS_LINUX
+    if (QString::compare(args["mh_mountpoint"].toString(),
+                         mountpoints.at(j)->mountPoint(), Qt::CaseInsensitive) == 0 &&
+        QString::compare(mountpoints.at(j)->mountType(), "cifs", Qt::CaseInsensitive) == 0)
 #else
-    if ( QString::compare( args["mountpoint"].toString(),
-                           mountpoints.at( j )->mountPoint(), Qt::CaseInsensitive ) == 0 &&
-         QString::compare( mountpoints.at( j )->mountType(), "smbfs", Qt::CaseInsensitive ) == 0 )
+    if (QString::compare(args["mh_mountpoint"].toString(),
+                         mountpoints.at(j)->mountPoint(), Qt::CaseInsensitive) == 0 &&
+        QString::compare(mountpoints.at(j)->mountType(), "smbfs", Qt::CaseInsensitive) == 0)
 #endif
     {
       mountpoint_ok = true;
@@ -190,10 +191,10 @@ ActionReply Smb4KMountHelper::unmount( const QVariantMap &args )
     }
   }
 
-  if ( !mountpoint_ok )
+  if (!mountpoint_ok)
   {
-    reply.setErrorCode( ActionReply::HelperError );
-    reply.setErrorDescription( i18n( "The mountpoint is invalid." ) );
+    reply.setErrorCode(ActionReply::HelperError);
+    reply.setErrorDescription(i18n("The mountpoint is invalid."));
     return reply;
   }
   else
@@ -201,24 +202,31 @@ ActionReply Smb4KMountHelper::unmount( const QVariantMap &args )
     // Do nothing
   }
 
-  KProcess proc( this );
-  proc.setOutputChannelMode( KProcess::SeparateChannels );
-  proc.setProcessEnvironment( QProcessEnvironment::systemEnvironment() );
-  proc.setProgram( args["command"].toStringList() );
+  KProcess proc(this);
+  proc.setOutputChannelMode(KProcess::SeparateChannels);
+  proc.setProcessEnvironment(QProcessEnvironment::systemEnvironment());
+  
+  // Set the umount command here.
+  QStringList command;
+  command << args["mh_command"].toString();
+  command << args["mh_options"].toStringList();
+  command << args["mh_mountpoint"].toString();
+
+  proc.setProgram(command);
 
   // Run the unmount process.
   proc.start();
 
-  if ( proc.waitForStarted( -1 ) )
+  if (proc.waitForStarted(-1))
   {
     // We want to be able to terminate the process from outside.
     // Thus, we implement a loop that checks periodically, if we
     // need to kill the process.
     bool user_kill = false;
 
-    while ( !proc.waitForFinished( 10 ) )
+    while (!proc.waitForFinished(10))
     {
-      if ( HelperSupport::isStopped() )
+      if (HelperSupport::isStopped())
       {
         proc.kill();
         user_kill = true;
@@ -230,12 +238,12 @@ ActionReply Smb4KMountHelper::unmount( const QVariantMap &args )
       }
     }
 
-    if ( proc.exitStatus() == KProcess::CrashExit )
+    if (proc.exitStatus() == KProcess::CrashExit)
     {
-      if ( !user_kill )
+      if (!user_kill)
       {
-        reply.setErrorCode( ActionReply::HelperError );
-        reply.setErrorDescription( i18n( "The unmount process crashed." ) );
+        reply.setErrorCode(ActionReply::HelperError);
+        reply.setErrorDescription(i18n("The unmount process crashed."));
         return reply;
       }
       else
@@ -246,18 +254,14 @@ ActionReply Smb4KMountHelper::unmount( const QVariantMap &args )
     else
     {
       // Check if there is output on stderr.
-      QString stderr = QString::fromUtf8( proc.readAllStandardError() );
-      reply.addData( "stderr", stderr.trimmed() );
-
-      // Check if there is output on stdout.
-      QString stdout = QString::fromUtf8( proc.readAllStandardOutput() );
-      reply.addData( "stdout", stdout.trimmed() );
+      QString stdErr = QString::fromUtf8(proc.readAllStandardError());
+      reply.addData("mh_error_message", stdErr.trimmed());
     }
   }
   else
   {
-    reply.setErrorCode( ActionReply::HelperError );
-    reply.setErrorDescription( i18n( "The unmount process could not be started." ) );
+    reply.setErrorCode(ActionReply::HelperError);
+    reply.setErrorDescription(i18n("The unmount process could not be started."));
   }
 
   return reply;
