@@ -3,7 +3,7 @@
     browser of Smb4K.
                              -------------------
     begin                : Fr Jan 5 2007
-    copyright            : (C) 2007-2013 by Alexander Reinholdt
+    copyright            : (C) 2007-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -204,10 +204,7 @@ void Smb4KNetworkBrowserPart::setupActions()
   KGuiItem abort_item( i18n( "&Abort" ), KIcon( "process-stop" ) );
   rescan_abort_action->setActiveGuiItem( rescan_item );
   rescan_abort_action->setInactiveGuiItem( abort_item );
-  QList<QKeySequence> rescan_shortcuts;
-  rescan_shortcuts += QKeySequence::Refresh;
-  rescan_shortcuts += QKeySequence( Qt::CTRL+Qt::Key_R );
-  rescan_abort_action->setShortcuts( rescan_shortcuts );
+  rescan_abort_action->setShortcut(QKeySequence::Refresh);
   rescan_abort_action->setActive( true );
   rescan_abort_action->setAutoToggle( false );
   connect( rescan_abort_action, SIGNAL(triggered(bool)), this, SLOT(slotRescanAbortActionTriggered(bool)) );
@@ -441,6 +438,8 @@ void Smb4KNetworkBrowserPart::slotContextMenuRequested( const QPoint &pos )
 
 void Smb4KNetworkBrowserPart::slotItemSelectionChanged()
 {
+  kDebug() << "Item selection changed...";
+  
   // Get the selected item.
   QList<QTreeWidgetItem *> items = m_widget->selectedItems();
 
@@ -452,7 +451,7 @@ void Smb4KNetworkBrowserPart::slotItemSelectionChanged()
     {
       switch ( browser_item->type() )
       {
-        case Smb4KNetworkBrowserItem::Host:
+        case Host:
         {
           // Change the text of the rescan action:
           KGuiItem rescan_item( i18n( "Scan Compute&r" ), KIcon( "view-refresh" ) );
@@ -466,10 +465,9 @@ void Smb4KNetworkBrowserPart::slotItemSelectionChanged()
           static_cast<KDualAction *>( actionCollection()->action( "mount_action" ) )->setActive( true );
           actionCollection()->action( "print_action" )->setEnabled( false );
           actionCollection()->action( "custom_action" )->setEnabled( true );
-          
           break;
         }
-        case Smb4KNetworkBrowserItem::Share:
+        case Share:
         {
           // Change the text of the rescan action:
           KGuiItem rescan_item( i18n( "Scan Compute&r" ), KIcon( "view-refresh" ) );
@@ -535,19 +533,43 @@ void Smb4KNetworkBrowserPart::slotItemSelectionChanged()
       // Do nothing. This is managed elsewhere.
     }
   }
-  else
+  else if ( items.size() > 1 )
   {
-    // Do nothing
+    // In this case there are only shares selected, because all other items
+    // are automatically deselected in extended selection mode.
+    
+    // For deciding which function the mount action should have, we use
+    // the number of unmounted shares. If that is identical with the items.size(),
+    // it will mount the items, otherwise it will unmount them.
+    int unmounted_shares = items.size();
+    
+    for ( int i = 0; i < items.size(); ++i )
+    {
+      Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( items.at( i ) );
+      
+      if ( item && item->shareItem()->isMounted() )
+      {
+        unmounted_shares--;
+      }
+      else
+      {
+        // Do nothing
+      }
+    }
+    
+    // Adjust the actions.
+    KGuiItem rescan_item( i18n( "Scan Netwo&rk" ), KIcon( "view-refresh" ) );
+    static_cast<KDualAction *>( actionCollection()->action( "rescan_abort_action" ) )->setActiveGuiItem( rescan_item );
+    actionCollection()->action( "rescan_abort_action" )->setEnabled( false );
+    actionCollection()->action( "bookmark_action" )->setEnabled( true );
+    actionCollection()->action( "authentication_action" )->setEnabled( false );
+    actionCollection()->action( "preview_action" )->setEnabled( true );
+    actionCollection()->action( "mount_action" )->setEnabled( true );
+    static_cast<KDualAction *>( actionCollection()->action( "mount_action" ) )->setActive( (unmounted_shares == items.size()) );
+    actionCollection()->action( "print_action" )->setEnabled( false );
+    actionCollection()->action( "custom_action" )->setEnabled( false );    
   }
-}
-
-
-void Smb4KNetworkBrowserPart::slotItemPressed( QTreeWidgetItem *item, int /*column*/ )
-{
-  // Enable/disable the actions.
-  Smb4KNetworkBrowserItem *browser_item = static_cast<Smb4KNetworkBrowserItem *>( item );
-
-  if ( !browser_item && m_widget->selectedItems().size() == 0 )
+  else
   {
     KGuiItem rescan_item( i18n( "Scan Netwo&rk" ), KIcon( "view-refresh" ) );
     static_cast<KDualAction *>( actionCollection()->action( "rescan_abort_action" ) )->setActiveGuiItem( rescan_item );    
@@ -559,13 +581,37 @@ void Smb4KNetworkBrowserPart::slotItemPressed( QTreeWidgetItem *item, int /*colu
     actionCollection()->action( "print_action" )->setEnabled( false );
     actionCollection()->action( "custom_action" )->setEnabled( false );
   }
-  else
+  
+  kDebug() << "Smb4KNetworkBrowserPart: Item selection changed ...";
+}
+
+
+void Smb4KNetworkBrowserPart::slotItemPressed( QTreeWidgetItem *item, int /*column*/ )
+{
+  // FIXME: Check if this slot is still necessary...
+  
+  if ( QApplication::keyboardModifiers() == Qt::NoModifier )
   {
-    if ( browser_item )
+    // Enable/disable the actions.
+    Smb4KNetworkBrowserItem *browser_item = static_cast<Smb4KNetworkBrowserItem *>( item );
+    
+    if ( !browser_item && m_widget->selectedItems().size() == 0 )
+    {
+      KGuiItem rescan_item( i18n( "Scan Netwo&rk" ), KIcon( "view-refresh" ) );
+      static_cast<KDualAction *>( actionCollection()->action( "rescan_abort_action" ) )->setActiveGuiItem( rescan_item );    
+      actionCollection()->action( "bookmark_action" )->setEnabled( false );
+      actionCollection()->action( "authentication_action" )->setEnabled( false );
+      actionCollection()->action( "preview_action" )->setEnabled( false );
+      actionCollection()->action( "mount_action" )->setEnabled( false );
+      static_cast<KDualAction *>( actionCollection()->action( "mount_action" ) )->setActive( true );
+      actionCollection()->action( "print_action" )->setEnabled( false );
+      actionCollection()->action( "custom_action" )->setEnabled( false );
+    }
+    else if ( browser_item )
     {
       switch ( browser_item->type() )
       {
-        case Smb4KNetworkBrowserItem::Share:
+        case Share:
         {
           if ( browser_item->shareItem()->isPrinter() )
           {
@@ -606,58 +652,71 @@ void Smb4KNetworkBrowserPart::slotItemPressed( QTreeWidgetItem *item, int /*colu
       // Do nothing
     }
   }
+  else
+  {
+    // Do nothing
+  }
+  
+  kDebug() << "Smb4KNetworkBrowserPart: Item pressed...";
 }
 
 
 void Smb4KNetworkBrowserPart::slotItemExecuted( QTreeWidgetItem *item, int /*column*/ )
 {
-  Smb4KNetworkBrowserItem *browserItem = static_cast<Smb4KNetworkBrowserItem *>( item );
-
-  if ( browserItem )
+  if ( QApplication::keyboardModifiers() == Qt::NoModifier && m_widget->selectedItems().size() == 1 )
   {
-    switch ( browserItem->type() )
+    Smb4KNetworkBrowserItem *browserItem = static_cast<Smb4KNetworkBrowserItem *>( item );
+
+    if ( browserItem )
     {
-      case Smb4KNetworkBrowserItem::Workgroup:
+      switch ( browserItem->type() )
       {
-        if ( browserItem->isExpanded() )
+        case Workgroup:
         {
-          Smb4KScanner::self()->lookupDomainMembers( browserItem->workgroupItem(), m_widget );
+          if ( browserItem->isExpanded() )
+          {
+            Smb4KScanner::self()->lookupDomainMembers( browserItem->workgroupItem(), m_widget );
+          }
+          else
+          {
+            // Do nothing
+          }
+          break;
         }
-        else
+        case Host:
         {
-          // Do nothing
+          if ( browserItem->isExpanded() )
+          {
+            Smb4KScanner::self()->lookupShares( browserItem->hostItem(), m_widget );
+          }
+          else
+          {
+            // Do nothing
+          }
+          break;
         }
-        break;
-      }
-      case Smb4KNetworkBrowserItem::Host:
-      {
-        if ( browserItem->isExpanded() )
+        case Share:
         {
-          Smb4KScanner::self()->lookupShares( browserItem->hostItem(), m_widget );
+          if ( !browserItem->shareItem()->isPrinter() )
+          {
+            slotMountActionTriggered( false );  // boolean is ignored
+          }
+          else
+          {
+            slotPrint( false );  // boolean is ignored
+          }
+          break;
         }
-        else
+        default:
         {
-          // Do nothing
+          break;
         }
-        break;
-      }
-      case Smb4KNetworkBrowserItem::Share:
-      {
-        if ( !browserItem->shareItem()->isPrinter() )
-        {
-          slotMountActionTriggered( false );  // boolean is ignored
-        }
-        else
-        {
-          slotPrint( false );  // boolean is ignored
-        }
-        break;
-      }
-      default:
-      {
-        break;
       }
     }
+    else
+    {
+      // Do nothing
+    }    
   }
   else
   {
@@ -672,7 +731,7 @@ void Smb4KNetworkBrowserPart::slotAboutToShowToolTip( Smb4KNetworkBrowserItem *i
   {
     switch ( item->type() )
     {
-      case Smb4KNetworkBrowserItem::Host:
+      case Host:
       {
         if ( !item->hostItem()->hasInfo() )
         {
@@ -704,7 +763,7 @@ void Smb4KNetworkBrowserPart::slotAboutToHideToolTip( Smb4KNetworkBrowserItem *i
   {
     switch ( item->type() )
     {
-      case Smb4KNetworkBrowserItem::Host:
+      case Host:
       {
         // Kill the lookup process for the additional information.
         Smb4KScanner::self()->abort( LookupInfo, item->hostItem() );
@@ -1013,7 +1072,7 @@ void Smb4KNetworkBrowserPart::slotWorkgroupMembers( Smb4KWorkgroup *workgroup, c
         
         switch ( item->type() )
         {
-          case Smb4KNetworkBrowserItem::Host:
+          case Host:
           {
             bool found = false;
             
@@ -1068,7 +1127,7 @@ void Smb4KNetworkBrowserPart::slotWorkgroupMembers( Smb4KWorkgroup *workgroup, c
           {
             Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( hosts.at( i ) );
             
-            if ( item->type() == Smb4KNetworkBrowserItem::Host &&
+            if ( item->type() == Host &&
                  QString::compare( list.at( i )->workgroupName(), item->hostItem()->workgroupName(), Qt::CaseInsensitive ) == 0 &&
                  QString::compare( list.at( i )->unc(), item->hostItem()->unc(), Qt::CaseInsensitive ) == 0 )
             {
@@ -1237,217 +1296,72 @@ void Smb4KNetworkBrowserPart::slotWorkgroupMembers( Smb4KWorkgroup *workgroup, c
 
 void Smb4KNetworkBrowserPart::slotShares( Smb4KHost *host, const QList<Smb4KShare *> &list )
 {
-  if ( host )
+  if (host)
   {
     QList<QTreeWidgetItem *> network_items = m_widget->findItems( host->hostName(), Qt::MatchFixedString|Qt::MatchRecursive, Smb4KNetworkBrowser::Network );
-
-    // There might be several entries in the list that have the same.
-    // So, loop through the list, get the host(s) and process it/them.
-    for ( int i = 0; i < network_items.size(); ++i )
+    
+    // Find the host and process it.
+    for (int i = 0; i < network_items.size(); ++i)
     {
-      Smb4KNetworkBrowserItem *network_item = static_cast<Smb4KNetworkBrowserItem *>( network_items.at( i ) );
-
-      if ( network_item->type() == Smb4KNetworkBrowserItem::Host )
+      Smb4KNetworkBrowserItem *network_item = static_cast<Smb4KNetworkBrowserItem *>(network_items[i]);
+      
+      if (network_item && network_item->type() == Host &&
+          QString::compare(network_item->hostItem()->workgroupName(), host->workgroupName(), Qt::CaseInsensitive) == 0)
       {
-        if ( !list.isEmpty() )
+        QStringList selected_items;
+        
+        // Delete all shares of the host.
+        while ( network_item->childCount() != 0 )
         {
-          if ( Smb4KSettings::autoExpandNetworkItems() && !network_item->isExpanded() )
+          if (network_item->child(0)->isSelected())
           {
-            m_widget->expandItem( network_item );
+            // Add item to the list of selected items.
+            selected_items << static_cast<Smb4KNetworkBrowserItem *>(network_item->child(0))->shareItem()->unc();
           }
           else
           {
             // Do nothing
           }
-
-          // Update and add the shares.
-          for ( int j = 0; j < list.size(); ++j )
+          delete network_item->child( 0 );
+        }
+        
+        // Add the newly discovered shares to the host.
+        if (!list.isEmpty())
+        {
+          // Expand the list of shares.
+          if (Smb4KSettings::autoExpandNetworkItems() && !network_item->isExpanded())
           {
-            bool found_item = false;
-
-            if ( QString::compare( network_item->hostItem()->hostName(), list.at( j )->hostName() ) == 0 )
-            {
-              for ( int k = 0; k < network_item->childCount(); ++k )
-              {
-                // In case the list also carries entries for other hosts, check that
-                // the host name of the entry in the list and of the current host in
-                // the tree widget are the same.
-                Smb4KNetworkBrowserItem *share_item = static_cast<Smb4KNetworkBrowserItem *>( network_item->child( k ) );
-
-                if ( QString::compare( list.at( j )->shareName(), share_item->shareItem()->shareName() ) == 0 )
-                {
-                  // We found the share. Now process it.
-                  if ( !share_item->shareItem()->isHidden() )
-                  {
-                    if ( !share_item->shareItem()->isPrinter() || Smb4KSettings::showPrinterShares() )
-                    {
-                      share_item->update( list.at( j ) );
-                    }
-                    else
-                    {
-                      // Do nothing. The item will be deleted below.
-                    }
-
-                    found_item = true;
-                    break;
-                  }
-                  else
-                  {
-                    if ( Smb4KSettings::showHiddenShares() )
-                    {
-                      if ( (share_item->shareItem()->isPrinter() && Smb4KSettings::showPrinterShares()) ||
-                           (share_item->shareItem()->isIPC() && Smb4KSettings::showHiddenIPCShares()) ||
-                           (share_item->shareItem()->isADMIN() && Smb4KSettings::showHiddenADMINShares()) )
-                      {
-                        share_item->update( list.at( j ) );
-                      }
-                      else
-                      {
-                        // Do nothing. The item will be deleted below.
-                      }
-                    }
-                    else
-                    {
-                      // Do nothing. The item will be deleted below.
-                    }
-
-                    found_item = true;
-                    break;
-                  }
-                }
-                else
-                {
-                  continue;
-                }
-              }
-
-              if ( !found_item )
-              {
-                if ( !list.at( j )->isHidden() )
-                {
-                  if ( !list.at( j )->isPrinter() || Smb4KSettings::showPrinterShares() )
-                  {
-                    (void) new Smb4KNetworkBrowserItem( network_item, list.at( j ) );
-                  }
-                  else
-                  {
-                    // Do nothing
-                  }
-
-                  continue;
-                }
-                else
-                {
-                  if ( Smb4KSettings::showHiddenShares() )
-                  {
-                    if ( (!list.at( j )->isPrinter() && !list.at( j )->isIPC() && !list.at( j )->isADMIN()) ||
-                         (list.at( j )->isPrinter() && Smb4KSettings::showPrinterShares()) ||
-                         (list.at( j )->isIPC() && Smb4KSettings::showHiddenIPCShares()) ||
-                         (list.at( j )->isADMIN() && Smb4KSettings::showHiddenADMINShares()) )
-                    {
-                      (void) new Smb4KNetworkBrowserItem( network_item, list.at( j ) );
-                    }
-                    else
-                    {
-                      // Do nothing
-                    }
-                  }
-                  else
-                  {
-                    // Do nothing
-                  }
-
-                  continue;
-                }
-              }
-              else
-              {
-                continue;
-              }
-            }
+            m_widget->expandItem(network_item);
           }
-
-          // Delete obsolete shares.
-          for ( int j = 0; j < network_item->childCount(); ++j )
+          else
           {
-            Smb4KNetworkBrowserItem *share_item = static_cast<Smb4KNetworkBrowserItem *>( network_item->child( j ) );
-            bool found_share = false;
-
-            for ( int k = 0; k < list.size(); ++k )
-            {
-              if ( QString::compare( share_item->shareItem()->hostName(), list.at( k )->hostName() ) == 0 &&
-                   QString::compare( share_item->shareItem()->shareName(), list.at( k )->shareName() ) == 0 )
-              {
-                // Test if the share needs to be deleted. If this is the case, we
-                // won't set found_share to TRUE.
-
-                // (a) Printer shares
-                if ( !Smb4KSettings::showPrinterShares() && share_item->shareItem()->isPrinter() )
-                {
-                  break;
-                }
-                else
-                {
-                  // Do nothing.
-                }
-
-                // (b) Hidden shares
-                if ( !Smb4KSettings::showHiddenShares() && share_item->shareItem()->isHidden() )
-                {
-                  break;
-                }
-                else
-                {
-                  if ( (!Smb4KSettings::showHiddenIPCShares() && share_item->shareItem()->isIPC()) ||
-                       (!Smb4KSettings::showHiddenADMINShares() && share_item->shareItem()->isADMIN()) )
-                  {
-                    break;
-                  }
-                  else
-                  {
-                    // Do nothing
-                  }
-                }
-
-                found_share = true;
-              }
-              else
-              {
-                continue;
-              }
-            }
-
-            if ( !found_share )
-            {
-              delete share_item;
-              continue;
-            }
-            else
-            {
-              continue;
-            }
+            // Do nothing
+          }
+          
+          // Add the shares to the host.
+          for (int j = 0; j < list.size(); ++j)
+          {
+            Smb4KNetworkBrowserItem *item = new Smb4KNetworkBrowserItem(network_item, list.at(j));
+            item->setSelected(selected_items.contains(list.at(j)->unc()));
           }
         }
         else
         {
-          // Collapse the item.
-          m_widget->collapseItem( network_item );
-
-          // Delete all share items from this host:
-          while ( network_item->childCount() != 0 )
-          {
-            delete network_item->child( 0 );
-          }
+          // Collapse the list of shares.
+          m_widget->collapseItem(network_item);
         }
 
-        // Sort the items.
-        m_widget->sortItems( Smb4KNetworkBrowser::Network, Qt::AscendingOrder );
+        // Stop the loop.
+        break;
       }
       else
       {
-        continue;
+        // Do nothing
       }
     }
+    
+    // Sort the items.
+    m_widget->sortItems( Smb4KNetworkBrowser::Network, Qt::AscendingOrder );
   }
   else
   {
@@ -1470,7 +1384,7 @@ void Smb4KNetworkBrowserPart::slotAddIPAddress( Smb4KHost *host )
     
     if ( item )
     {
-      if ( item->type() == Smb4KNetworkBrowserItem::Host )
+      if ( item->type() == Host )
       {
         if ( QString::compare( host->unc(), item->hostItem()->unc(), Qt::CaseInsensitive ) == 0 &&
              QString::compare( host->workgroupName(), item->hostItem()->workgroupName(), Qt::CaseInsensitive ) == 0 )
@@ -1625,7 +1539,7 @@ void Smb4KNetworkBrowserPart::slotAuthError( Smb4KHost *host, int process )
         {
           Smb4KNetworkBrowserItem *workgroup = static_cast<Smb4KNetworkBrowserItem *>( m_widget->topLevelItem( i ) );
           
-          if ( workgroup && workgroup->type() == Smb4KNetworkBrowserItem::Workgroup &&
+          if ( workgroup && workgroup->type() == Workgroup &&
                QString::compare( host->workgroupName(), workgroup->workgroupItem()->workgroupName(), Qt::CaseInsensitive ) == 0 )
           {
             QList<QTreeWidgetItem *> hosts = workgroup->takeChildren();
@@ -1657,7 +1571,7 @@ void Smb4KNetworkBrowserPart::slotAuthError( Smb4KHost *host, int process )
       {
         Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( *it );
         
-        if ( item && item->type() == Smb4KNetworkBrowserItem::Host )
+        if ( item && item->type() == Host )
         {
           if ( QString::compare( host->hostName(), item->hostItem()->hostName(), Qt::CaseInsensitive ) == 0 &&
                QString::compare( host->workgroupName(), item->hostItem()->workgroupName(), Qt::CaseInsensitive ) == 0 )
@@ -1706,17 +1620,17 @@ void Smb4KNetworkBrowserPart::slotRescanAbortActionTriggered( bool /* checked */
 
         switch ( item->type() )
         {
-          case Smb4KNetworkBrowserItem::Workgroup:
+          case Workgroup:
           {
             Smb4KScanner::self()->lookupDomainMembers( item->workgroupItem(), m_widget );
             break;
           }
-          case Smb4KNetworkBrowserItem::Host:
+          case Host:
           {
             Smb4KScanner::self()->lookupShares( item->hostItem(), m_widget );
             break;
           }
-          case Smb4KNetworkBrowserItem::Share:
+          case Share:
           {
             Smb4KNetworkBrowserItem *parentItem = static_cast<Smb4KNetworkBrowserItem *>( item->parent() );
             Smb4KScanner::self()->lookupShares( parentItem->hostItem(), m_widget );
@@ -1766,12 +1680,12 @@ void Smb4KNetworkBrowserPart::slotAuthentication( bool /*checked*/ )
   {
     switch ( item->type() )
     {
-      case Smb4KNetworkBrowserItem::Host:
+      case Host:
       {
         Smb4KWalletManager::self()->showPasswordDialog( item->hostItem(), m_widget );
         break;
       }
-      case Smb4KNetworkBrowserItem::Share:
+      case Share:
       {
         Smb4KWalletManager::self()->showPasswordDialog( item->shareItem(), m_widget );
         break;
@@ -1797,12 +1711,12 @@ void Smb4KNetworkBrowserPart::slotCustomOptions( bool /*checked*/ )
   {
     switch ( item->type() )
     {
-      case Smb4KNetworkBrowserItem::Host:
+      case Host:
       {
         Smb4KCustomOptionsManager::self()->openCustomOptionsDialog( item->hostItem(), m_widget );
         break;
       }
-      case Smb4KNetworkBrowserItem::Share:
+      case Share:
       {
         Smb4KCustomOptionsManager::self()->openCustomOptionsDialog( item->shareItem(), m_widget );
         break;
@@ -1822,16 +1736,16 @@ void Smb4KNetworkBrowserPart::slotCustomOptions( bool /*checked*/ )
 
 void Smb4KNetworkBrowserPart::slotAddBookmark( bool /*checked*/ )
 {
-  QList<QTreeWidgetItem *> selected_items = m_widget->selectedItems();
+  QList<QTreeWidgetItem *> items = m_widget->selectedItems();
   QList<Smb4KShare *> shares;
 
-  if ( !selected_items.isEmpty() )
+  if ( !items.isEmpty() )
   {
-    for ( int i = 0; i < selected_items.size(); ++i )
+    for ( int i = 0; i < items.size(); ++i )
     {
-      Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( selected_items.at( i ) );
+      Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( items.at( i ) );
 
-      if ( item->type() == Smb4KNetworkBrowserItem::Share && !item->shareItem()->isPrinter() )
+      if ( item && item->type() == Share && !item->shareItem()->isPrinter() )
       {
         shares << item->shareItem();
         continue;
@@ -1861,13 +1775,24 @@ void Smb4KNetworkBrowserPart::slotAddBookmark( bool /*checked*/ )
 
 void Smb4KNetworkBrowserPart::slotPreview( bool /*checked*/ )
 {
-  // Get the current item and pass the encapsulated Smb4KShare item
-  // to the preview job.
-  Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( m_widget->currentItem() );
-
-  if ( item && !item->shareItem()->isPrinter() )
+  QList<QTreeWidgetItem *> items = m_widget->selectedItems();
+  
+  if ( !items.isEmpty() )
   {
-    Smb4KPreviewer::self()->preview( item->shareItem(), m_widget );
+    for ( int i = 0; i < items.size(); ++i )
+    {
+      Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( items.at( i ) );
+
+      if ( item && item->type() == Share && !item->shareItem()->isPrinter() )
+      {
+        Smb4KPreviewer::self()->preview( item->shareItem(), m_widget );
+        continue;
+      }
+      else
+      {
+        continue;
+      }
+    }
   }
   else
   {
@@ -1880,7 +1805,7 @@ void Smb4KNetworkBrowserPart::slotPrint( bool /*checked*/ )
 {
   Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( m_widget->currentItem() );
   
-  if ( item->shareItem()->isPrinter() )
+  if ( item && item->shareItem()->isPrinter() )
   {
     Smb4KPrint::self()->print( item->shareItem(), m_widget );
   }
@@ -1893,33 +1818,80 @@ void Smb4KNetworkBrowserPart::slotPrint( bool /*checked*/ )
 
 void Smb4KNetworkBrowserPart::slotMountActionTriggered( bool /*checked*/ )
 {
-  Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( m_widget->currentItem() );
-
-  if ( item )
+  QList<QTreeWidgetItem *> items = m_widget->selectedItems();
+  
+  if ( items.size() > 1 )
   {
-    switch ( item->type() )
+    // In the case of multiple selected network items, selectedItems() 
+    // only contains shares. Thus, we do not need to test for the type.
+    // For deciding what the mount action is supposed to do, i.e. mount
+    // the (remaining) selected unmounted shares or unmounting all selected
+    // mounted shares, we use the number of unmounted shares. If that is
+    // greater than 0, we mount all shares that need to be mounted, otherwise
+    // we unmount all selected shares.
+    QList<Smb4KShare *> unmounted, mounted;
+    
+    for ( int i = 0; i < items.size(); ++i )
     {
-      case Smb4KNetworkBrowserItem::Share:
+      Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( items.at( i ) );
+      
+      if ( item && item->shareItem()->isMounted() )
       {
-        if ( !item->shareItem()->isMounted() )
-        {
-          Smb4KMounter::self()->mountShare( item->shareItem(), m_widget );
-        }
-        else
-        {
-          Smb4KMounter::self()->unmountShare( item->shareItem(), false, m_widget );
-        }
-        break;
+        mounted << item->shareItem();
       }
-      default:
+      else if ( item && !item->shareItem()->isMounted() )
       {
-        break;
+        unmounted << item->shareItem();
       }
+      else
+      {
+        // Do nothing
+      }
+    }
+    
+    if ( unmounted.size() > 0 )
+    {
+      // Mount the (remaining) unmounted shares.
+      Smb4KMounter::self()->mountShares( unmounted, m_widget );
+    }
+    else
+    {
+      // Unmount all shares.
+      Smb4KMounter::self()->unmountShares( mounted, m_widget );
     }
   }
   else
   {
-    // Do nothing
+    // If only one network item is selected, we need to test for the type
+    // of the item. Only in case of a share we need to do something.
+    Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( m_widget->currentItem() );
+
+    if ( item )
+    {
+      switch ( item->type() )
+      {
+        case Share:
+        {
+          if ( !item->shareItem()->isMounted() )
+          {
+            Smb4KMounter::self()->mountShare( item->shareItem(), m_widget );
+          }
+          else
+          {
+            Smb4KMounter::self()->unmountShare( item->shareItem(), false, m_widget );
+          }
+          break;
+        }
+        default:
+        {
+          break;
+        }
+      }
+    }
+    else
+    {
+      // Do nothing
+    }
   }
 }
 
@@ -2134,7 +2106,7 @@ void Smb4KNetworkBrowserPart::slotShareMounted( Smb4KShare *share )
   {
     Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( *it );
     
-    if ( item->type() == Smb4KNetworkBrowserItem::Share )
+    if ( item->type() == Share )
     {
       if ( QString::compare( item->shareItem()->unc(), share->unc(), Qt::CaseInsensitive ) == 0 )
       {
@@ -2164,7 +2136,7 @@ void Smb4KNetworkBrowserPart::slotShareUnmounted( Smb4KShare *share )
   {
     Smb4KNetworkBrowserItem *item = static_cast<Smb4KNetworkBrowserItem *>( *it );
     
-    if ( item->type() == Smb4KNetworkBrowserItem::Share )
+    if ( item->type() == Share )
     {
       if ( QString::compare( item->shareItem()->unc(), share->unc(), Qt::CaseInsensitive ) == 0 )
       {
