@@ -2,7 +2,7 @@
     smb4kconfigdialog  -  The configuration dialog of Smb4K
                              -------------------
     begin                : Sa Apr 14 2007
-    copyright            : (C) 2004-2013 by Alexander Reinholdt
+    copyright            : (C) 2004-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -35,13 +35,23 @@
 #include "smb4kauthoptionspage.h"
 #include "smb4ksambaoptionspage.h"
 #include "smb4krsyncoptionspage.h"
-#include "smb4klaptopsupportoptionspage.h"
 #include "smb4kcustomoptionspage.h"
+#include "smb4kprofilespage.h"
+#include "smb4kmountoptionspage.h"
 #include "core/smb4ksettings.h"
 #include "core/smb4kglobal.h"
 #include "core/smb4kauthinfo.h"
 #include "core/smb4kwalletmanager.h"
 #include "core/smb4kcustomoptionsmanager.h"
+#include "core/smb4kprofilemanager.h"
+
+#if defined(Q_OS_LINUX)
+#include "core/smb4kmountsettings_linux.h"
+#elif defined(Q_OS_FREEBSD) || defined(Q_OS_NETBSD)
+#include "core/smb4kmountsettings_freebsd.h"
+#else
+#define UNSUPPORTED_PLATFORM
+#endif
 
 // Qt includes
 #include <QRadioButton>
@@ -51,6 +61,8 @@
 #include <QSize>
 #include <QScrollArea>
 #include <QPointer>
+#include <QPair>
+#include <QList>
 
 // KDE includes
 #include <klineedit.h>
@@ -64,6 +76,7 @@
 #include <kpassworddialog.h>
 #include <kapplication.h>
 #include <kconfigdialogmanager.h>
+#include <keditlistwidget.h>
 
 using namespace Smb4KGlobal;
 
@@ -129,6 +142,14 @@ void Smb4KConfigDialog::setupDialog()
   samba_area->setWidgetResizable( true );
   samba_area->setFrameStyle( QFrame::NoFrame );
 
+#if !defined(UNSUPPORTED_PLATFORM)  
+  Smb4KMountOptionsPage *mount_options = new Smb4KMountOptionsPage(this);
+  QScrollArea *mount_area = new QScrollArea(this);
+  mount_area->setWidget(mount_options);
+  mount_area->setWidgetResizable(true);
+  mount_area->setFrameStyle(QFrame::NoFrame);
+#endif
+
   Smb4KRsyncOptionsPage *rsync_options = new Smb4KRsyncOptionsPage( this );
   QScrollArea *rsync_area = new QScrollArea( this );
   rsync_area->setWidget( rsync_options );
@@ -137,50 +158,56 @@ void Smb4KConfigDialog::setupDialog()
   
   rsync_options->setEnabled( !KStandardDirs::findExe( "rsync" ).isEmpty() );
 
-  Smb4KLaptopSupportOptionsPage *laptop_options = new Smb4KLaptopSupportOptionsPage( this );
-  QScrollArea *laptop_area = new QScrollArea( this );
-  laptop_area->setWidget( laptop_options );
-  laptop_area->setWidgetResizable( true );
-  laptop_area->setFrameStyle( QFrame::NoFrame );
-  
   Smb4KCustomOptionsPage *custom_options = new Smb4KCustomOptionsPage( this );
   QScrollArea *custom_area = new QScrollArea( this );
   custom_area->setWidget( custom_options );
   custom_area->setWidgetResizable( true );
   custom_area->setFrameStyle( QFrame::NoFrame );
+  
+  Smb4KProfilesPage *profiles_page = new Smb4KProfilesPage(this);
+  QScrollArea *profiles_area = new QScrollArea(this);
+  profiles_area->setWidget(profiles_page);
+  profiles_area->setWidgetResizable(true);
+  profiles_area->setFrameStyle(QFrame::NoFrame);
 
   // Now add the pages to the configuration dialog
-  m_user_interface  = addPage( interface_area, i18n( "User Interface" ), "view-choose" );
-  m_network         = addPage( network_area, i18n( "Network" ), "network-workgroup" );
-  m_shares          = addPage( share_area, i18n( "Shares" ), "folder-remote" );
-  m_authentication  = addPage( auth_area, i18n( "Authentication" ), "dialog-password" );
-  m_samba           = addPage( samba_area, i18n( "Samba" ), "preferences-system-network" );
-  m_synchronization = addPage( rsync_area, i18n( "Synchronization" ), "folder-sync" );
-  m_laptop_support  = addPage( laptop_area, i18n( "Laptop Support" ), "computer-laptop" );
-  m_custom_options  = addPage( custom_area, i18n( "Custom Options" ), "preferences-system-network" );
+  m_user_interface  = addPage(interface_area, i18n( "User Interface" ), "view-choose" );
+  m_network         = addPage(network_area, i18n( "Network" ), "network-workgroup" );
+  m_shares          = addPage(share_area, i18n( "Shares" ), "folder-remote" );
+  m_authentication  = addPage(auth_area, i18n( "Authentication" ), "dialog-password" );
+  m_samba           = addPage(samba_area, i18n( "Samba" ), "preferences-system-network" );
+#if !defined(UNSUPPORTED_PLATFORM)
+  m_mounting        = addPage(mount_area, Smb4KMountSettings::self(), i18n("Mounting"), "system-run");
+#endif
+  m_synchronization = addPage(rsync_area, i18n( "Synchronization" ), "folder-sync" );
+  m_custom_options  = addPage(custom_area, i18n( "Custom Options" ), "preferences-system-network" );
+  m_profiles        = addPage(profiles_area, i18n("Profiles"), "format-list-unordered");
 
   // Stuff that's not managed by KConfig XT is loaded by
   // Smb4KConfigDialog::showEvent()!
 
   // Connections
-  connect( custom_options,     SIGNAL(customSettingsModified()),
-           this,               SLOT(slotEnableApplyButton()) );
+  connect(custom_options, SIGNAL(customSettingsModified()),
+          this,           SLOT(slotEnableApplyButton()));
   
-  connect( custom_options,     SIGNAL(reloadCustomSettings()),
-           this,               SLOT(slotReloadCustomOptions()) );
+  connect(custom_options, SIGNAL(reloadCustomSettings()),
+          this,           SLOT(slotReloadCustomOptions()) );
 
-  connect( auth_options,       SIGNAL(loadWalletEntries()),
-           this,               SLOT(slotLoadAuthenticationInformation()) );
+  connect( auth_options,  SIGNAL(loadWalletEntries()),
+           this,          SLOT(slotLoadAuthenticationInformation()) );
            
-  connect( auth_options,       SIGNAL(saveWalletEntries()),
-           this,               SLOT(slotSaveAuthenticationInformation()) );
+  connect( auth_options,  SIGNAL(saveWalletEntries()),
+           this,          SLOT(slotSaveAuthenticationInformation()) );
            
-  connect( auth_options,       SIGNAL(setDefaultLogin()),
-           this,               SLOT(slotSetDefaultLogin()) );
+  connect( auth_options,  SIGNAL(setDefaultLogin()),
+           this,          SLOT(slotSetDefaultLogin()) );
            
-  connect( auth_options,       SIGNAL(walletEntriesModified()),
-           this,               SLOT(slotEnableApplyButton()) );
-
+  connect( auth_options,  SIGNAL(walletEntriesModified()),
+           this,          SLOT(slotEnableApplyButton()) );
+  
+  connect( this,          SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)),
+           this,          SLOT(slotCheckPage(KPageWidgetItem*,KPageWidgetItem*)));
+  
   setInitialSize( QSize( 800, 600 ) );
 
   KConfigGroup group( Smb4KSettings::self()->config(), "ConfigDialog" );
@@ -216,19 +243,67 @@ void Smb4KConfigDialog::saveCustomOptions()
 }
 
 
-bool Smb4KConfigDialog::checkSettings()
+void Smb4KConfigDialog::propagateProfilesChanges()
 {
-  //
-  // Network page
-  //
-  QRadioButton *query_custom_master = m_network->widget()->findChild<QRadioButton *>( "kcfg_QueryCustomMaster" );
-  KLineEdit *custom_master_input    = m_network->widget()->findChild<KLineEdit *>( "kcfg_CustomMasterBrowser" );
-
-  if ( (query_custom_master && query_custom_master->isChecked()) &&
-       (custom_master_input && custom_master_input->text().trimmed().isEmpty()) )
+  Smb4KProfilesPage *profiles_page = m_profiles->widget()->findChild<Smb4KProfilesPage *>();
+  
+  if (profiles_page)
   {
-    KMessageBox::sorry( this, i18n( "The custom master browser that is to be queried has not been filled in.\nPlease enter it now." ) );
-    setCurrentPage( m_network );
+    // Remove the profiles.
+    QStringList removed_profiles = profiles_page->removedProfiles();
+    
+    if (!removed_profiles.isEmpty())
+    {
+      Smb4KProfileManager::self()->removeProfiles(removed_profiles, this);
+      profiles_page->clearRemovedProfiles();
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Rename the profiles.
+    QList< QPair<QString,QString> > renamed_profiles = profiles_page->renamedProfiles();
+    
+    if (!renamed_profiles.isEmpty())
+    {
+      Smb4KProfileManager::self()->migrateProfiles(renamed_profiles);
+      profiles_page->clearRenamedProfiles();
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    // Finally reload the custom options.
+    if (!removed_profiles.isEmpty() || !renamed_profiles.isEmpty())
+    {
+      loadCustomOptions();
+    }
+    else
+    {
+      // Do nothing
+    }
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
+
+bool Smb4KConfigDialog::checkNetworkPage()
+{
+  QRadioButton *query_custom_master = m_network->widget()->findChild<QRadioButton *>("kcfg_QueryCustomMaster");
+  KLineEdit *custom_master_input    = m_network->widget()->findChild<KLineEdit *>("kcfg_CustomMasterBrowser");
+  
+  QString msg = i18n("An incorrect setting has been found. You are now taken to the corresponding dialog page to fix it.");
+
+  if ((query_custom_master && query_custom_master->isChecked()) &&
+      (custom_master_input && custom_master_input->text().trimmed().isEmpty()))
+  {
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_network);
     custom_master_input->setFocus();
     return false;
   }
@@ -237,31 +312,36 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  QRadioButton *scan_bcast_areas = m_network->widget()->findChild<QRadioButton *>( "kcfg_ScanBroadcastAreas" );
-  KLineEdit *bcast_areas_input   = m_network->widget()->findChild<KLineEdit *>( "kcfg_BroadcastAreas" );
+  QRadioButton *scan_bcast_areas = m_network->widget()->findChild<QRadioButton *>("kcfg_ScanBroadcastAreas");
+  KLineEdit *bcast_areas_input   = m_network->widget()->findChild<KLineEdit *>("kcfg_BroadcastAreas");
   
-  if ( (scan_bcast_areas && scan_bcast_areas->isChecked()) &&
-       (bcast_areas_input && bcast_areas_input->text().trimmed().isEmpty()) )
+  if ((scan_bcast_areas && scan_bcast_areas->isChecked()) &&
+      (bcast_areas_input && bcast_areas_input->text().trimmed().isEmpty()))
   {
-    KMessageBox::sorry( this, i18n( "The broadcast areas that are to be scanned have not been filled in.\nPlease enter them now." ) );
-    setCurrentPage( m_network );
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_network);
     bcast_areas_input->setFocus();
     return false;
   }
   else
   {
     // Do nothing
-  }
+  }  
   
-  //
-  // Shares page
-  //
-  KUrlRequester *mount_prefix = m_shares->widget()->findChild<KUrlRequester *>( "kcfg_MountPrefix" );
+  return true;
+}
+
+
+bool Smb4KConfigDialog::checkSharesPage()
+{
+  KUrlRequester *mount_prefix = m_shares->widget()->findChild<KUrlRequester *>("kcfg_MountPrefix");
   
-  if ( mount_prefix && mount_prefix->url().path().trimmed().isEmpty() )
+  QString msg = i18n("An incorrect setting has been found. You are now taken to the corresponding dialog page to fix it.");
+  
+  if (mount_prefix && mount_prefix->url().path().trimmed().isEmpty())
   {
-    KMessageBox::sorry( this, i18n( "The mount prefix has not been filled in.\nPlease enter it now." ) );
-    setCurrentPage( m_shares );
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_shares);
     mount_prefix->setFocus();
     return false;
   }
@@ -270,26 +350,21 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  //
-  // Authentication page
-  //
-  // FIXME: Should we check the presence of the default login here? A 
-  // disadvantage would be that we need to open the wallet even if it 
-  // wasn't open until now.
+  return true;
+}
 
-  //
-  // Samba page
-  //
-  KLineEdit *file_mask = m_samba->widget()->findChild<KLineEdit *>( "kcfg_FileMask" );
+
+bool Smb4KConfigDialog::checkMountingPage()
+{
+#if !defined(UNSUPPORTED_PLATFORM)
+  KLineEdit *file_mask = m_mounting->widget()->findChild<KLineEdit *>("kcfg_FileMask");
   
-  if ( file_mask && file_mask->text().trimmed().isEmpty() )
+  QString msg = i18n("An incorrect setting has been found. You are now taken to the corresponding dialog page to fix it.");
+  
+  if (file_mask && file_mask->text().trimmed().isEmpty())
   {
-    KMessageBox::sorry( this, i18n( "The file mask has not been filled in.\nPlease enter it now." ) );
-    setCurrentPage( m_samba );
-    
-    Smb4KSambaOptionsPage *samba_options = m_samba->widget()->findChild<Smb4KSambaOptionsPage *>();
-    samba_options->setCurrentIndex( Smb4KSambaOptionsPage::MountingTab );
-    
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_mounting);
     file_mask->setFocus();
     return false;
   }
@@ -298,16 +373,12 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  KLineEdit *directory_mask = m_samba->widget()->findChild<KLineEdit *>( "kcfg_DirectoryMask" );
+  KLineEdit *directory_mask = m_mounting->widget()->findChild<KLineEdit *>("kcfg_DirectoryMask");
   
-  if ( directory_mask && directory_mask->text().trimmed().isEmpty() )
+  if (directory_mask && directory_mask->text().trimmed().isEmpty())
   {
-    KMessageBox::sorry( this, i18n( "The directory mask has not been filled in.\nPlease enter it now." ) );
-    setCurrentPage( m_samba );
-    
-    Smb4KSambaOptionsPage *samba_options = m_samba->widget()->findChild<Smb4KSambaOptionsPage *>();
-    samba_options->setCurrentIndex( Smb4KSambaOptionsPage::MountingTab );
-    
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_samba);
     directory_mask->setFocus();
     return false;
   }
@@ -315,19 +386,33 @@ bool Smb4KConfigDialog::checkSettings()
   {
     // Do nothing
   }
+#endif
   
-  //
-  // Synchronization page
-  // 
-  KUrlRequester *sync_prefix = m_synchronization->widget()->findChild<KUrlRequester *>( "kcfg_RsyncPrefix" );
+  return true;
+}
+
+
+bool Smb4KConfigDialog::checkSynchronizationPage()
+{
+  KUrlRequester *sync_prefix = m_synchronization->widget()->findChild<KUrlRequester *>("kcfg_RsyncPrefix");
   
-  if ( sync_prefix && sync_prefix->url().path().trimmed().isEmpty() )
+  QString msg = i18n("An incorrect setting has been found. You are now taken to the corresponding dialog page to fix it.");
+  
+  if (sync_prefix && sync_prefix->url().path().trimmed().isEmpty())
   {
-    KMessageBox::sorry( this, i18n( "The synchronization prefix has not been filled in.\nPlease enter it now." ) );
-    setCurrentPage( m_synchronization );
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
     
     Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-    sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::CopyingTab );
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::CopyingTab);
+    }
+    else
+    {
+      // Do nothing
+    }
     
     sync_prefix->setFocus();
     return false;
@@ -337,19 +422,106 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
+  QCheckBox *max_delete = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseMaximumDelete");
+  KIntNumInput *max_delete_val = m_synchronization->widget()->findChild<KIntNumInput *>("kcfg_MaximumDeleteValue");
+  
+  if ((max_delete && max_delete->isChecked()) && (max_delete_val && max_delete_val->value() == 0))
+  {
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
+    
+    Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::DelTransTab);
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    max_delete_val->setFocus();
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  QCheckBox *min_trans_size = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseMinimalTransferSize");
+  KIntNumInput *min_trans_size_val = m_synchronization->widget()->findChild<KIntNumInput *>("kcfg_MinimalTransferSize");
+  
+  if ((min_trans_size && min_trans_size->isChecked()) && (min_trans_size_val && min_trans_size_val->value() == 0))
+  {
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
+    
+    Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::DelTransTab);
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    min_trans_size_val->setFocus();
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  QCheckBox *max_trans_size = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseMaximalTransferSize");
+  KIntNumInput *max_trans_size_val = m_synchronization->widget()->findChild<KIntNumInput *>("kcfg_MaximalTransferSize");
+  
+  if ((max_trans_size && max_trans_size->isChecked()) && (max_trans_size_val && max_trans_size_val->value() == 0))
+  {
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
+    
+    Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::DelTransTab);
+    }
+    else
+    {
+      // Do nothing
+    }
+    
+    max_trans_size_val->setFocus();
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
   QCheckBox *use_partial_directory = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_UsePartialDirectory" );
   KUrlRequester *partial_directory = m_synchronization->widget()->findChild<KUrlRequester *>( "kcfg_PartialDirectory" );
   
-  if ( (use_partial_directory && use_partial_directory->isChecked()) &&
-       (partial_directory && partial_directory->url().path().trimmed().isEmpty()) )
+  if ((use_partial_directory && use_partial_directory->isChecked()) &&
+      (partial_directory && partial_directory->url().path().trimmed().isEmpty()))
   {
-    KMessageBox::sorry( this, i18n( "The directory where partially transferred files should\n"
-                                    "be stored has not been filled in.\n"
-                                    "Please enter it now." ) );
+    KMessageBox::sorry(this, msg);
     setCurrentPage( m_synchronization );
     
     Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-    sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::DelTransTab );
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::DelTransTab);
+    }
+    else
+    {
+      // Do nothing
+    }
     
     partial_directory->setFocus();
     return false;
@@ -359,18 +531,25 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  QCheckBox *use_exclude_pattern = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_UseExcludePattern" );
-  KLineEdit *exclude_pattern     = m_synchronization->widget()->findChild<KLineEdit *>( "kcfg_ExcludePattern" );
+  QCheckBox *use_exclude_pattern = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseExcludePattern");
+  KLineEdit *exclude_pattern = m_synchronization->widget()->findChild<KLineEdit *>("kcfg_ExcludePattern");
   
-  if ( (use_exclude_pattern && use_exclude_pattern->isChecked()) &&
-       (exclude_pattern && exclude_pattern->text().trimmed().isEmpty()) )
+  if ((use_exclude_pattern && use_exclude_pattern->isChecked()) &&
+      (exclude_pattern && exclude_pattern->text().trimmed().isEmpty()))
   {
-    KMessageBox::sorry( this, i18n( "The exclude pattern for synchronization has not been filled in.\n"
-                                    "Please enter it now." ) );
-    setCurrentPage( m_synchronization );
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
     
     Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-    sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::FilteringTab );
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::FilteringTab);
+    }
+    else
+    {
+      // Do nothing
+    }
     
     exclude_pattern->setFocus();
     return false;
@@ -380,18 +559,25 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  QCheckBox *use_exclude_file = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_UseExcludeFrom" );
-  KUrlRequester *exclude_file = m_synchronization->widget()->findChild<KUrlRequester *>( "kcfg_ExcludeFrom" );
+  QCheckBox *use_exclude_file = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseExcludeFrom");
+  KUrlRequester *exclude_file = m_synchronization->widget()->findChild<KUrlRequester *>("kcfg_ExcludeFrom");
   
-  if ( (use_exclude_file && use_exclude_file->isChecked()) &&
-       (exclude_file && exclude_file->url().path().trimmed().isEmpty()) )
+  if ((use_exclude_file && use_exclude_file->isChecked()) &&
+      (exclude_file && exclude_file->url().path().trimmed().isEmpty()))
   {
-    KMessageBox::sorry( this, i18n( "The file from which the exclude pattern for synchronization are to be read\n"
-                                    "has not been filled in.\nPlease enter it now." ) );
-    setCurrentPage( m_synchronization );
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
     
     Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-    sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::FilteringTab );
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::FilteringTab);
+    }
+    else
+    {
+      // Do nothing
+    }
     
     exclude_file->setFocus();
     return false;
@@ -401,18 +587,25 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  QCheckBox *use_include_pattern = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_UseIncludePattern" );
-  KLineEdit *include_pattern     = m_synchronization->widget()->findChild<KLineEdit *>( "kcfg_IncludePattern" );
+  QCheckBox *use_include_pattern = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseIncludePattern");
+  KLineEdit *include_pattern = m_synchronization->widget()->findChild<KLineEdit *>("kcfg_IncludePattern");
   
-  if ( (use_include_pattern && use_include_pattern->isChecked()) &&
-       (include_pattern && include_pattern->text().trimmed().isEmpty()) )
+  if ((use_include_pattern && use_include_pattern->isChecked()) &&
+      (include_pattern && include_pattern->text().trimmed().isEmpty()))
   {
-    KMessageBox::sorry( this, i18n( "The include pattern for synchronization has not been filled in.\n"
-                                    "Please enter it now." ) );
-    setCurrentPage( m_synchronization );
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
     
     Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-    sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::FilteringTab );
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::FilteringTab);
+    }
+    else
+    {
+      // Do nothing
+    }
     
     include_pattern->setFocus();
     return false;
@@ -422,18 +615,25 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  QCheckBox *use_include_file = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_UseIncludeFrom" );
-  KUrlRequester *include_file = m_synchronization->widget()->findChild<KUrlRequester *>( "kcfg_IncludeFrom" );
+  QCheckBox *use_include_file = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseIncludeFrom");
+  KUrlRequester *include_file = m_synchronization->widget()->findChild<KUrlRequester *>("kcfg_IncludeFrom");
   
-  if ( (use_include_file && use_include_file->isChecked()) &&
-       (include_file && include_file->url().path().trimmed().isEmpty()) )
+  if ((use_include_file && use_include_file->isChecked()) &&
+      (include_file && include_file->url().path().trimmed().isEmpty()))
   {
-    KMessageBox::sorry( this, i18n( "The file from which the include pattern for synchronization are to be read\n"
-                                    "has not been filled in.\nPlease enter it now." ) );
-    setCurrentPage( m_synchronization );
+    KMessageBox::sorry(this, msg);
+    setCurrentPage(m_synchronization);
     
     Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-    sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::FilteringTab );
+    
+    if (sync_options)
+    {
+      sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::FilteringTab);
+    }
+    else
+    {
+      // Do nothing
+    }
     
     include_file->setFocus();
     return false;
@@ -443,21 +643,29 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
-  QCheckBox *make_backups = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_MakeBackups" );
+  QCheckBox *make_backups = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_MakeBackups");
   
-  if ( make_backups && make_backups->isChecked() )
+  if (make_backups && make_backups->isChecked())
   {
-    QCheckBox *use_backup_suffix = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_UseBackupSuffix" );
-    KLineEdit *backup_suffix     = m_synchronization->widget()->findChild<KLineEdit *>( "kcfg_BackupSuffix" );
+    QCheckBox *use_backup_suffix = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseBackupSuffix");
+    KLineEdit *backup_suffix = m_synchronization->widget()->findChild<KLineEdit *>("kcfg_BackupSuffix");
     
-    if ( (use_backup_suffix && use_backup_suffix->isChecked()) &&
-         (backup_suffix && backup_suffix->text().trimmed().isEmpty()) )
+    if ((use_backup_suffix && use_backup_suffix->isChecked()) &&
+        (backup_suffix && backup_suffix->text().trimmed().isEmpty()))
     {
-      KMessageBox::sorry( this, i18n( "The backup suffix for synchronization has not been filled in.\nPlease enter it now." ) );
-      setCurrentPage( m_synchronization );
+      KMessageBox::sorry(this, msg);
+      setCurrentPage(m_synchronization);
       
       Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-      sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::AdvancedTab );
+      
+      if (sync_options)
+      {
+        sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::AdvancedTab);
+      }
+      else
+      {
+        // Do nothing
+      }
       
       backup_suffix->setFocus();
       return false;
@@ -467,17 +675,25 @@ bool Smb4KConfigDialog::checkSettings()
       // Do nothing
     }
     
-    QCheckBox *use_backup_dir = m_synchronization->widget()->findChild<QCheckBox *>( "kcfg_UseBackupDirectory" );
-    KUrlRequester *backup_dir = m_synchronization->widget()->findChild<KUrlRequester *>( "kcfg_BackupDirectory" );
+    QCheckBox *use_backup_dir = m_synchronization->widget()->findChild<QCheckBox *>("kcfg_UseBackupDirectory");
+    KUrlRequester *backup_dir = m_synchronization->widget()->findChild<KUrlRequester *>("kcfg_BackupDirectory");
     
-    if ( (use_backup_dir && use_backup_dir->isChecked()) &&
-         (backup_dir && backup_dir->url().path().trimmed().isEmpty()) )
+    if ((use_backup_dir && use_backup_dir->isChecked()) &&
+        (backup_dir && backup_dir->url().path().trimmed().isEmpty()))
     {
-      KMessageBox::sorry( this, i18n( "The backup directory for synchronization has not been filled in.\nPlease enter it now." ) );
-      setCurrentPage( m_synchronization );
+      KMessageBox::sorry(this, msg);
+      setCurrentPage(m_synchronization);
       
       Smb4KRsyncOptionsPage *sync_options = m_synchronization->widget()->findChild<Smb4KRsyncOptionsPage *>();
-      sync_options->setCurrentIndex( Smb4KRsyncOptionsPage::AdvancedTab );
+      
+      if (sync_options)
+      {
+        sync_options->setCurrentIndex(Smb4KRsyncOptionsPage::AdvancedTab);
+      }
+      else
+      {
+        // Do nothing
+      }
       
       backup_dir->setFocus();
       return false;
@@ -492,6 +708,57 @@ bool Smb4KConfigDialog::checkSettings()
     // Do nothing
   }
   
+  // FIXME: Also check --block-size (kcfg_UseBlockSize & kcfg_BlockSize) and 
+  // --checksum-seed (kcfg_UseChecksumSeed & kcfg_ChecksumSeed), if necessary.
+  // However, I need more info on what values are needed ...
+
+  return true;
+}
+
+
+
+bool Smb4KConfigDialog::checkSettings()
+{
+  // Check Network page
+  if (!checkNetworkPage())
+  {
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Check Shares page
+  if (!checkSharesPage())
+  {
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Check Mounting page
+  if (!checkMountingPage())
+  {
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
+  // Check Synchronization page
+  if (!checkSynchronizationPage())
+  {
+    return false;
+  }
+  else
+  {
+    // Do nothing
+  }
+  
   return true;
 }
 
@@ -516,38 +783,42 @@ void Smb4KConfigDialog::showEvent( QShowEvent *e )
 /////////////////////////////////////////////////////////////////////////////
 
 
-void Smb4KConfigDialog::slotButtonClicked( int button )
+void Smb4KConfigDialog::updateSettings()
 {
-  switch( button )
+  saveCustomOptions();
+  slotSaveAuthenticationInformation();
+  propagateProfilesChanges();
+      
+  KConfigDialog::updateSettings();
+}
+
+
+void Smb4KConfigDialog::slotButtonClicked(int button)
+{
+  switch(button)
   {
     case Apply:
     {
       // If some settings are not complete, stop here and give
       // the user the opportunity to fill in the needed string(s).
-      if ( !checkSettings() )
+      if (!checkSettings())
       {
         return;
       }
 
-      saveCustomOptions();
-      slotSaveAuthenticationInformation();
-
       break;
     }
     case Ok:
     {
       // If some settings are not complete, stop here and give
       // the user the opportunity to fill in the needed string(s).
-      if ( !checkSettings() )
+      if (!checkSettings())
       {
         return;
       }
 
-      saveCustomOptions();
-      slotSaveAuthenticationInformation();
-
-      KConfigGroup group( Smb4KSettings::self()->config(), "ConfigDialog" );
-      saveDialogSize( group, KConfigGroup::Normal );
+      KConfigGroup group(Smb4KSettings::self()->config(), "ConfigDialog");
+      saveDialogSize(group, KConfigGroup::Normal);
 
       break;
     }
@@ -557,7 +828,7 @@ void Smb4KConfigDialog::slotButtonClicked( int button )
     }
   }
 
-  KConfigDialog::slotButtonClicked( button );
+  KConfigDialog::slotButtonClicked(button);
 }
 
 
@@ -600,12 +871,12 @@ void Smb4KConfigDialog::slotSetDefaultLogin()
     
     QPointer<KPasswordDialog> dlg = new KPasswordDialog( this, KPasswordDialog::ShowUsernameLine );
     dlg->setPrompt( i18n( "Enter the default login information." ) );
-    dlg->setUsername( authInfo.login() );
+    dlg->setUsername( authInfo.userName() );
     dlg->setPassword( authInfo.password() );
     
     if ( dlg->exec() == KPasswordDialog::Accepted )
     {
-      authInfo.setLogin( dlg->username() );
+      authInfo.setUserName( dlg->username() );
       authInfo.setPassword( dlg->password() );
       
       Smb4KWalletManager::self()->writeDefaultAuthInfo( &authInfo );
@@ -657,8 +928,8 @@ void Smb4KConfigDialog::slotEnableApplyButton()
              (QString::compare( old_wallet_entries.at( i )->workgroupName(),
                                 new_wallet_entries.at( j )->workgroupName(),
                                 Qt::CaseInsensitive ) != 0 ||
-              QString::compare( old_wallet_entries.at( i )->login(),
-                                new_wallet_entries.at( j )->login(),
+              QString::compare( old_wallet_entries.at( i )->userName(),
+                                new_wallet_entries.at( j )->userName(),
                                 Qt::CaseInsensitive ) != 0 ||
               QString::compare( old_wallet_entries.at( i )->password(),
                                 new_wallet_entries.at( j )->password(),
@@ -689,7 +960,7 @@ void Smb4KConfigDialog::slotEnableApplyButton()
   }
   
   // Check the custom settings.
-  Smb4KCustomOptionsPage *custom_options = m_custom_options->findChild<Smb4KCustomOptionsPage *>();
+  Smb4KCustomOptionsPage *custom_options = m_custom_options->widget()->findChild<Smb4KCustomOptionsPage *>();
   
   if ( !enable && custom_options && custom_options->customSettingsMaybeChanged() )
   {
@@ -743,4 +1014,50 @@ void Smb4KConfigDialog::slotReloadCustomOptions()
 }
 
 
+void Smb4KConfigDialog::slotCheckPage(KPageWidgetItem* /*current*/, KPageWidgetItem* before)
+{
+  if (before == m_user_interface)
+  {
+    // Nothing to do
+  }
+  else if (before == m_network)
+  {
+    (void)checkNetworkPage();
+  }
+  else if (before == m_shares)
+  {
+    (void)checkSharesPage();
+  }
+  else if (before == m_authentication)
+  {
+    // Do nothing
+  }
+  else if (before == m_samba)
+  {
+    // Do nothing
+  }
+#if !defined(UNSUPPORTED_PLATFORM)
+  else if (before == m_mounting)
+  {
+    (void)checkMountingPage();
+  }
+#endif
+  else if (before == m_synchronization)
+  {
+    (void)checkSynchronizationPage();
+  }
+  else if (before == m_custom_options)
+  {
+    // Do nothing
+  }
+  else if (before == m_profiles)
+  {
+    // Do nothing
+  }
+  else
+  {
+    // Do nothing
+  }
+}
+
 #include "smb4kconfigdialog.moc"
