@@ -2,7 +2,7 @@
     smb4ksharesview_part  -  This Part includes the shares view of Smb4K.
                              -------------------
     begin                : Sa Jun 30 2007
-    copyright            : (C) 2007-2012 by Alexander Reinholdt
+    copyright            : (C) 2007-2014 by Alexander Reinholdt
     email                : alexander.reinholdt@kdemail.net
  ***************************************************************************/
 
@@ -410,7 +410,7 @@ void Smb4KSharesViewPart::loadSettings()
     setupView();
 
     m_list_view->setColumnHidden( Smb4KSharesListView::Owner, !Smb4KSettings::showOwner() );
-#ifndef Q_OS_FREEBSD
+#ifdef Q_OS_LINUX
     m_list_view->setColumnHidden( Smb4KSharesListView::Login, !Smb4KSettings::showLoginName() );
 #endif
     m_list_view->setColumnHidden( Smb4KSharesListView::FileSystem, !Smb4KSettings::showFileSystem() );
@@ -434,7 +434,7 @@ void Smb4KSharesViewPart::loadSettings()
 
     QMap<int, int> map;
     map.insert( configGroup.readEntry( "ColumnPositionItem", (int)Smb4KSharesListView::Item ), Smb4KSharesListView::Item );
-#ifndef Q_OS_FREEBSD
+#ifdef Q_OS_LINUX
     map.insert( configGroup.readEntry( "ColumnPositionLogin", (int)Smb4KSharesListView::Login ), Smb4KSharesListView::Login );
 #endif
     map.insert( configGroup.readEntry( "ColumnPositionFileSystem", (int)Smb4KSharesListView::FileSystem ), Smb4KSharesListView::FileSystem );
@@ -479,7 +479,7 @@ void Smb4KSharesViewPart::saveSettings()
       KConfigGroup configGroup( Smb4KSettings::self()->config(), "SharesViewPart" );
 
       configGroup.writeEntry( "ColumnPositionItem", m_list_view->header()->visualIndex( Smb4KSharesListView::Item ) );
-#ifndef Q_OS_FREEBSD
+#ifdef Q_OS_LINUX
       configGroup.writeEntry( "ColumnPositionLogin", m_list_view->header()->visualIndex( Smb4KSharesListView::Login ) );
 #endif
       configGroup.writeEntry( "ColumnPositionFileSystem", m_list_view->header()->visualIndex( Smb4KSharesListView::FileSystem ) );
@@ -1309,7 +1309,7 @@ void Smb4KSharesViewPart::slotKonsole( bool /*checked*/ )
 
         if ( item && !item->shareItem()->isInaccessible() )
         {
-          open( item->shareItem(), Konsole );
+          openShare( item->shareItem(), Konsole );
         }
         else
         {
@@ -1329,7 +1329,7 @@ void Smb4KSharesViewPart::slotKonsole( bool /*checked*/ )
 
         if ( item && !item->shareItem()->isInaccessible() )
         {
-          open( item->shareItem(), Konsole );
+          openShare( item->shareItem(), Konsole );
         }
         else
         {
@@ -1361,7 +1361,7 @@ void Smb4KSharesViewPart::slotFileManager( bool /*checked*/ )
 
         if ( item && !item->shareItem()->isInaccessible() )
         {
-          open( item->shareItem(), FileManager );
+          openShare( item->shareItem(), FileManager );
         }
         else
         {
@@ -1381,7 +1381,7 @@ void Smb4KSharesViewPart::slotFileManager( bool /*checked*/ )
 
         if ( item && !item->shareItem()->isInaccessible() )
         {
-          open( item->shareItem(), FileManager );
+          openShare( item->shareItem(), FileManager );
         }
         else
         {
