@@ -31,6 +31,7 @@
 #include "smb4kmainwindow.h"
 #include "smb4ksystemtray.h"
 #include "smb4kbookmarkmenu.h"
+#include "smb4kprofilesmenu.h"
 #include "core/smb4ksettings.h"
 #include "core/smb4kglobal.h"
 #include "core/smb4kwalletmanager.h"
@@ -122,6 +123,10 @@ void Smb4KMainWindow::setupActions()
   m_dock_widgets = new QActionGroup( actionCollection() );
   m_dock_widgets->setExclusive( false );
 
+  //
+  // FIXME: Move the shares view actionmenu to its own class
+  //  
+  
   // Shares view action menu
   KActionMenu *shares_view_menu = new KActionMenu( KIcon( "view-choose" ), i18n( "Shares View" ), actionCollection() );
   actionCollection()->addAction( "shares_view_menu", shares_view_menu );
@@ -162,6 +167,10 @@ void Smb4KMainWindow::setupActions()
   actionCollection()->addAction( "bookmarks_menu", bookmarks );
   actionCollection()->addAction( "bookmark_action", bookmarks->addBookmarkAction() );
   connect( bookmarks->addBookmarkAction(), SIGNAL(triggered(bool)), SLOT(slotAddBookmark()) );
+  
+  // Profiles menu
+  Smb4KProfilesMenu *profiles = new Smb4KProfilesMenu(this);
+  actionCollection()->addAction("profiles_menu", profiles);
 }
 
 
@@ -562,6 +571,20 @@ void Smb4KMainWindow::loadSettings()
 
 void Smb4KMainWindow::saveSettings()
 {
+  // Save the active part.
+  KConfigGroup config_group( Smb4KSettings::self()->config(), "MainWindow" );
+  
+  if (m_manager->activePart())
+  {
+    config_group.writeEntry( "ActivePart", m_manager->activePart()->objectName() );
+  }
+  else
+  {
+    // Do nothing
+  }
+
+  // Save if the main window should be started docked.
+  Smb4KSettings::setStartMainWindowDocked( !isVisible() );
 }
 
 
@@ -579,23 +602,11 @@ bool Smb4KMainWindow::queryClose()
   }
   else
   {
+    saveSettings();
     return true;
   }
 }
 
-
-bool Smb4KMainWindow::queryExit()
-{
-  // Save the active part.
-  KConfigGroup config_group( Smb4KSettings::self()->config(), "MainWindow" );
-  config_group.writeEntry( "ActivePart", m_manager->activePart()->objectName() );
-  
-  // Save if the main window should be started docked.
-  Smb4KSettings::setStartMainWindowDocked( !isVisible() );
-  return true;
-}
-
-
 void Smb4KMainWindow::setupMountIndicator()
 {
   QStringList overlays;
@@ -760,37 +771,24 @@ void Smb4KMainWindow::slotWalletManagerInitialized()
 {
   if ( Smb4KWalletManager::self()->useWalletSystem() )
   {
-    switch ( Smb4KWalletManager::self()->currentState() )
+    if ( Smb4KWalletManager::self()->walletIsOpen() )
     {
-      case Smb4KWalletManager::UseWallet:
-      {
-        m_pass_icon->setPixmap( KIconLoader::global()->loadIcon( "wallet-open",
-                                KIconLoader::Small, 0, KIconLoader::DefaultState ) );
-        m_pass_icon->setToolTip( i18n( "The wallet is in use." ) );
-        break;
-      }
-      default:
-      {
-        m_pass_icon->setPixmap( KIconLoader::global()->loadIcon( "wallet-closed",
-                                KIconLoader::Small, 0, KIconLoader::DefaultState ) );
-        m_pass_icon->setToolTip( i18n( "The wallet is not in use." ) );
-        break;
-      }
+      m_pass_icon->setPixmap( KIconLoader::global()->loadIcon( "wallet-open",
+                              KIconLoader::Small, 0, KIconLoader::DefaultState ) );
+      m_pass_icon->setToolTip( i18n( "The wallet is in use." ) );
+    }
+    else
+    {
+      m_pass_icon->setPixmap( KIconLoader::global()->loadIcon( "wallet-closed",
+                              KIconLoader::Small, 0, KIconLoader::DefaultState ) );
+      m_pass_icon->setToolTip( i18n( "The wallet is not in use." ) );
     }
   }
   else
   {
     m_pass_icon->setPixmap( KIconLoader::global()->loadIcon( "dialog-password",
                             KIconLoader::Small, 0, KIconLoader::DefaultState ) );
-
-    if ( Smb4KSettings::rememberLogins() )
-    {
-      m_pass_icon->setToolTip( i18n( "The password dialog mode is used and logins are remembered." ) );
-    }
-    else
-    {
-      m_pass_icon->setToolTip( i18n( "The password dialog mode is used and logins are not remembered." ) );
-    }
+    m_pass_icon->setToolTip( i18n( "The password dialog mode is used." ) );
   }
 }
 
